
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NNL initialise and calculator .m file
% Created 2012-10-25
% Last modified 2017-07-31

% Guidelines on use:
% This is a root .m file that contains many functions in it
% It allows the compiling of a single .m file to do different tasks.
% New functions can be added at will - to call an already existing function within the file, the handle is 'nnl_root.<existing_function_name>'.
% To use this file: open a new data folder and copy the .m file across into it. 
% Ensure that the <current folder> in Matlab is this folder, and type 'import nnl_root.*'in the command window.
% All functions below can now be called from the command window.  If you 'clear all' during the matlab session, the .m must be imported again.
% - Niraj Lal, niraj@nirajlal.org

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

classdef nnl_root               % initialise NNL calculator functions and data structure
    methods (Static = true)     % create static functions that can be called globally

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%Initialising Functions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        function nnl_import_check
            fprintf('nnl_root imported ok\n');
        end
        
        % Initialise things
        function nnl_initialise         % create the basic wavelength, ev arrays, load the AM15G spectrum
            nnl_root.clearthings        % clear, clear global, clear variables
            wl = 300:1:1200;            % create wl and set the desired wavelength range and spacing
            %assignin('base', 'wl',wl);  % assign ev wl in workspace
            global WL;
            WL = wl;
            ev = nnl_root.wlev(wl);     % create ev array
            global EV;
            EV = ev;
            %assignin('base', 'ev',ev);  % assign ev in workspace
%             am15g_external = importdata('reference_data\ASTMG173_FullSpectrum.txt');
            am15g_ext_spec=am15g_external(:,2);
            am15g_ext_wl=am15g_external(:,1);
            am15g=interp1(am15g_ext_wl,am15g_ext_spec,wl);
            global AM15G;
            AM15G=am15g;
            assignin('base', 'am15g',am15g);
            assignin('base', 'wl',wl);
            %nnl_root.setconstants;
        end
        
        % Initialise spectra
        function nnl_initialise_spectra(wl)  % load AM1.5G spectra (W/m^2/nm^2), interpolate to the given wavelength range set in the cell array and set in the workspace
            am15g_external = importdata('reference_data\ASTMG173_FullSpectrum.txt');
            am15g_ext_spec=am15g_external(:,2);
            am15g_ext_wl=am15g_external(:,1);
            am15g=interp1(am15g_ext_wl,am15g_ext_spec,wl);
            light.wl = wl;
            light.spacing = wl(2) - wl(1);
            dimwl = length(wl);
            light.sunshine.spectra=am15g;
            light.sunshine.n_of_photons = light.sunshine.spectra./nnl_root.energy_of_photon(light.wl);
            %light.sunshine.energy = trapz(light.wl,light.sunshine.spectra);    % energy flux in %W/m^2 over the wl range - from 300:1200 it gives an irradiance of 836.0815 W/m^2
            light.sunshine.energy = 1.0004e+003;                                % energy flux in W/m^2 taken from ASTMG173 full spectrum from 280nm to 4um.
            light.ev = nnl_root.wlev(light.wl);
            assignin('base', 'am15g',am15g);
            assignin('base', 'light',light);
            %assignin('base', 'dimwl',dimwl);
            %nnl_root.setconstants;
        end
        
        function nnl_import_materials(wavelengths)
            nnl_root.nnl_import_Si(wavelengths);
            nnl_root.nnl_import_Ag(wavelengths);
            nnl_root.nnl_import_Au_JC(wavelengths);
            %nnl_root.nnl_import_Sb2S3_azul(wavelengths);
            %nnl_root.nnl_import_Sb2S3(wavelengths);
            nnl_root.nnl_import_modelmat(wavelengths);
            nnl_root.nnl_import_SiO2(wavelengths);
            nnl_root.nnl_import_TiO2(wavelengths);
            nnl_root.nnl_import_ZnO(wavelengths);
            nnl_root.nnl_import_air(wavelengths);
            nnl_root.nnl_import_glass(wavelengths);
            nnl_root.nnl_import_pdms(wavelengths);
            nnl_root.nnl_import_PMMA(wavelengths);
            %nnl_root.nnl_import_aSiH(wavelengths);
            %nnl_root.nnl_import_CIS(wavelengths);
            %nnl_root.nnl_import_CZTS(wavelengths);
            %nnl_root.nnl_import_CIGS(wavelengths);
            nnl_root.nnl_import_ITO(wavelengths);
            %nnl_root.nnl_import_perovskite(wavelengths);
            nnl_root.nnl_import_SiON_low(wavelengths)
            nnl_root.nnl_import_SiON_high(wavelengths)
            nnl_root.nnl_import_FTO(wavelengths)
            nnl_root.nnl_import_FTO_Wenger(wavelengths)
            nnl_root.nnl_import_Perovskite_Loper(wavelengths)
            nnl_root.nnl_import_Spiro(wavelengths)
            nnl_root.nnl_import_TiO2_anatase(wavelengths)
            nnl_root.nnl_import_MG(wavelengths)
            nnl_root.nnl_import_Al_Rakic(wavelengths)
            nnl_root.nnl_import_Ni_JC(wavelengths)
            nnl_root.nnl_import_ZnO_Bond(wavelengths)
            nnl_root.nnl_import_Al2O3_Malitson(wavelengths)
            
        end
        
        % Import Si data
        function nnl_import_Si(wavelengths)     % load the Si data from Keevers, interpolate and create the Si material structure in the workspace
            Si_ext = importdata('reference_data\GreenKeeverscSi.xls');  %load AM1.5G spectra
            Si_wl_raw = Si_ext.data.Sheet1(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            Si_alpha_raw = Si_ext.data.Sheet1(:,2);
            Si.alpha=(1e-7)*interp1(Si_wl_raw,Si_alpha_raw,wavelengths);    % converting alpha from (/cm) into units of (/nm)
            Si_absdepth_raw = Si_ext.data.Sheet1(:,3);
            Si.absdepth=(1e3)*interp1(Si_wl_raw,Si_absdepth_raw,wavelengths);   % converting absdepth from (um) into units of (nm)
            Si_n_raw = Si_ext.data.Sheet1(:,4);
            Si.n=interp1(Si_wl_raw,Si_n_raw,wavelengths);
            Si_k_raw = Si_ext.data.Sheet1(:,5);
            Si.k=interp1(Si_wl_raw,Si_k_raw,wavelengths);
            Si_R_raw = Si_ext.data.Sheet1(:,6);
            Si.R=interp1(Si_wl_raw,Si_R_raw,wavelengths);
            Si.Eg = 1.12;
            Si.wlEg = nnl_root.wlev(Si.Eg);
            Si.wlEg_index = find(wavelengths==floor(Si.wlEg));
            %Si.alpha(Si.wlEg_index:end)=0;
            Si.eps1 = Si.n.^2 - Si.k.^2;
            Si.eps2 = 2*Si.n.*Si.k;
            Si.eps = complex(Si.eps1,Si.eps2);
            Si.nk = complex(Si.n,Si.k);
            Si.Voc=0.706;           % set Voc to the record UNSW PERL cell output under AM15g spectrum
            %Si.J0=42.7/(exp((1.60e-19*Si.Voc)/((1.3806488e-23)*300)) - 1);   % set J0 to the corresponding value for PERL cell using standard solar cell equation
            Si.J0=427/(exp((1.60218e-19*Si.Voc)/((1.3806488e-23)*298)) - 1);   % in A/m^2: set J0 to the corresponding value for PERL cell using standard solar cell equation
            Si.FF = 0.828;          % set FF to the record UNSW PERL cell output under AM15g spectrum
            %Si.collection_probability = 0.958; % Following TW modelling parameter to calculate the correct Jsc is obtained for the UNSW PERL record cell
            Si.collection_probability = 0.97955; % new? Following TW modelling parameter to calculate the correct Jsc is obtained for the UNSW PERL record cell
            Si.name = 'silicon';
            assignin('base', 'Si',Si);
        end
        
        function nnl_import_Ag(wavelengths)     % load the Si data from Keevers, interpolate and create the Si material structure in the workspace
            Ag_ext = importdata('reference_data\Ag_Palik.xls');  %load AM1.5G spectra
            Ag_wl_raw = Ag_ext.data.Sheet1(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            Ag_n_raw = Ag_ext.data.Sheet1(:,4);
            Ag.n=interp1(Ag_wl_raw,Ag_n_raw,wavelengths);
            Ag_k_raw = Ag_ext.data.Sheet1(:,5);
            Ag.k=interp1(Ag_wl_raw,Ag_k_raw,wavelengths);
            Ag_R_raw = Ag_ext.data.Sheet1(:,6);
            Ag.R=interp1(Ag_wl_raw,Ag_R_raw,wavelengths);
            Ag.eps1 = Ag.n.^2 - Ag.k.^2;
            Ag.eps2 = 2*Ag.n.*Ag.k;
            Ag.eps = complex(Ag.eps1,Ag.eps2);
            Ag.nk = complex(Ag.n,Ag.k);
            Ag.name = 'Ag';
            assignin('base', 'Ag',Ag);
        end
        
        % Import and calculate Sb2S3 data
        function nnl_import_Sb2S3(wavelengths)      % calculate the absorption coefficient for Sb2S3 using the empirical formula below
            Sb2S3.name = 'antimony_trisulfide';
            %Sb2S3.Astar0 = 3.1e7;   % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
            Sb2S3.Astar0 = 1e4;   % Astar0 with dimensions of cm^-1!  With kT in the denominator of the expression below
            Sb2S3.Eg = 1.95;         % Eg in Ev, value taken from TW fit
            %Sb2S3.Eg = 1.4;         % Eg in Ev, value taken from TW fit
            Sb2S3.Ld = 100;          % Diffusion length Ld in nm
            Sb2S3.phi = 1e-5;          % luminescence efficiency
            %Sb2S3.phi = 1;          % luminescence efficiency
            Sb2S3.alpha = Sb2S3.Astar0.*(( (nnl_root.wlev(wavelengths) - Sb2S3.Eg)/(0.0256) ).^(1/2)); % denominator kT, with T = 298K
            Sb2S3.wlEg = nnl_root.wlev(Sb2S3.Eg);
            Sb2S3.wlEg_index = find(wavelengths==floor(Sb2S3.wlEg));
            %Sb2S3.alpha(Sb2S3.wlEg_index:end)=0;
            Sb2S3.alpha = Sb2S3.alpha*1e-7;
            %Sb2S3.alpha = Sb2S3.alpha*1e-9;
            Sb2S3.Voc = 1.1;        % Voc set assuming Sb2S3 luminescent efficiency of 5e-6, from TW paper
            Sb2S3.FF = 0.8;        % set fill factor of top cell to 0.8 as a first estimate, cf. TW model
            %Ag_ext = importdata('reference_data\Ag_Palik.xls');  %load AM1.5G spectra
            Sb2S3_ext = importdata('reference_data\nSbS.xls');  %load AM1.5G spectra
            Sb2S3_ext2 = importdata('reference_data\kSbS.xls');  %load AM1.5G spectra
            Sb2S3_wl_raw = (1e9)*Sb2S3_ext.Sheet1(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            Sb2S3_n_raw = Sb2S3_ext.Sheet1(:,2);
            Sb2S3.n=interp1(Sb2S3_wl_raw,Sb2S3_n_raw,wavelengths,'spline','extrap');
            Sb2S3_k_raw = Sb2S3_ext2.Sheet1(:,2);
            Sb2S3.k=interp1(Sb2S3_wl_raw,Sb2S3_k_raw,wavelengths,'spline','extrap');
            % make sure this is ok:
            %Sb2S3.k(Sb2S3.wlEg_index:end)=0;
            %
            
            %Sb2S3.n(1:length(wavelengths)) = 3.0;          % set refractive index of Sb2S3 - should set this to a proper vector of the values, but for now, a constant
            %Sb2S3.k(1:length(wavelengths)) = (Sb2S3.alpha .* wavelengths.*1e-9)./(4*pi);
            Sb2S3.Vbi=Sb2S3.Eg;
            Sb2S3.eps1 = Sb2S3.n.^2 - Sb2S3.k.^2;
            Sb2S3.eps2 = 2*Sb2S3.n.*Sb2S3.k;
            % Check this!!!
            Sb2S3.eps2(Sb2S3.wlEg_index:end)=0;
            %
            
            Sb2S3.eps = complex(Sb2S3.eps1,Sb2S3.eps2);
            Sb2S3.nk = complex(Sb2S3.n,Sb2S3.k);
            assignin('base', 'Sb2S3',Sb2S3);
            %assignin('base', 'Sb2S3_n_raw',Sb2S3_n_raw)
            %assignin('base', 'Sb2S3_wl_raw',Sb2S3_wl_raw)
            
        end
        
        function nnl_import_modelmat(wavelengths)      % calculate the absorption coefficient for Sb2S3 using the empirical formula below
            modelmat.name = 'model material';
            %Sb2S3.Astar0 = 3.1e7;   % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
            modelmat.Astar0 = 1e4;   % Astar0 with dimensions of cm^-1!  With kT in the denominator of the expression below
            modelmat.Eg = 1.55;         % Eg in Ev, value taken from TW fit
            %Sb2S3.Eg = 1.4;         % Eg in Ev, value taken from TW fit
            modelmat.Ld = 100;          % Diffusion length Ld in nm
            modelmat.phi = 1e-5;          % luminescence efficiency
            %Sb2S3.phi = 1;          % luminescence efficiency
            modelmat.alpha = modelmat.Astar0.*(( (nnl_root.wlev(wavelengths) - modelmat.Eg)/(0.0256) ).^(1/2)); % denominator kT, with T = 298K
            modelmat.wlEg = nnl_root.wlev(modelmat.Eg);
            modelmat.wlEg_index = find(wavelengths==floor(modelmat.wlEg));
            modelmat.alpha(modelmat.wlEg_index:end)=0;
            modelmat.alpha = modelmat.alpha*1e-7;
            %Sb2S3.alpha = Sb2S3.alpha*1e-9;
            modelmat.Voc = 1.1;        % Voc set assuming Sb2S3 luminescent efficiency of 5e-6, from TW paper
            modelmat.FF = 0.8;        % set fill factor of top cell to 0.8 as a first estimate, cf. TW model
            %Ag_ext = importdata('reference_data\Ag_Palik.xls');  %load AM1.5G spectra
            modelmat_ext = importdata('reference_data\nSbS.xls');  %load AM1.5G spectra
            modelmat_ext2 = importdata('reference_data\kSbS.xls');  %load AM1.5G spectra
            modelmat_wl_raw = (1e9)*modelmat_ext.Sheet1(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            modelmat_n_raw = modelmat_ext.Sheet1(:,2);
            modelmat.n=interp1(modelmat_wl_raw,modelmat_n_raw,wavelengths,'spline','extrap');
            modelmat_k_raw = modelmat_ext2.Sheet1(:,2);
            modelmat.k=interp1(modelmat_wl_raw,modelmat_k_raw,wavelengths,'spline','extrap');
            % make sure this is ok:
            %modelmat.k(Sb2S3.wlEg_index:end)=0;
            %
            
            %modelmat.n(1:length(wavelengths)) = 3.0;          % set refractive index of Sb2S3 - should set this to a proper vector of the values, but for now, a constant
            %modelmat.k(1:length(wavelengths)) = (modelmat.alpha .* wavelengths.*1e-9)./(4*pi);
            modelmat.Vbi=modelmat.Eg;
            modelmat.eps1 = modelmat.n.^2 - modelmat.k.^2;
            modelmat.eps2 = 2*modelmat.n.*modelmat.k;
            % Check this!!!
            modelmat.eps2(modelmat.wlEg_index:end)=0;
            %
            
            modelmat.eps = complex(modelmat.eps1,modelmat.eps2);
            modelmat.nk = complex(modelmat.n,modelmat.k);
            assignin('base', 'modelmat',modelmat);
            %assignin('base', 'Sb2S3_n_raw',Sb2S3_n_raw)
            %assignin('base', 'Sb2S3_wl_raw',Sb2S3_wl_raw)
            
        end
        
        function nnl_import_modelmatalpha(wavelengths, modelmat, aSiH)
            modelmatalpha = modelmat;
            modelmatalpha.name = 'model material with alpha';
            modelmatalpha.alpha(1,modelmat.wlEg_index: modelmat.wlEg_index + (find(aSiH.alpha==0,1) ) - (find(wavelengths==671))) = aSiH.alpha(1, find(wavelengths==671): find(aSiH.alpha==0,1));
                        
            assignin('base', 'modelmatalpha',modelmatalpha);
        end
        
        function nnl_import_modelmatalpha2(modelmat, Sb2S3)
            modelmatalpha2 = modelmat;
            modelmatalpha2.name = 'model material with alpha2';
            modelmatalpha2.alpha(1,modelmat.wlEg_index: end)= Sb2S3.alpha(1, modelmat.wlEg_index: end)./(Sb2S3.alpha(1,modelmat.wlEg_index-1)/modelmat.alpha(modelmat.wlEg_index-1));
                        
            assignin('base', 'modelmatalpha2',modelmatalpha2);
        end
        
        function nnl_import_Sb2S3_azul(wavelengths)      % calculate the absorption coefficient for Sb2S3 using the empirical formula below
            Sb2S3.name = 'antimony_trisulfide';
            %Sb2S3.Astar0 = 3.1e7;   % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
            Sb2S3.Astar0 = 1e4;   % Astar0 with dimensions of cm^-1!  With kT in the denominator of the expression below
            Sb2S3.Eg = 1.7;         % Eg in Ev, value taken from TW fit
            %Sb2S3.Eg = 1.4;         % Eg in Ev, value taken from TW fit
            Sb2S3.Ld = 33;          % Diffusion length Ld in nm
            Sb2S3.phi = 1e-5;          % luminescence efficiency
            %Sb2S3.phi = 1;          % luminescence efficiency
            %Sb2S3.alpha = Sb2S3.Astar0.*(( (nnl_root.wlev(wavelengths) - Sb2S3.Eg)/(0.0256) ).^(1/2)); % denominator kT, with T = 298K
            Sb2S3.wlEg = nnl_root.wlev(Sb2S3.Eg);
            Sb2S3.wlEg_index = find(wavelengths==floor(Sb2S3.wlEg));
            %Sb2S3.alpha(Sb2S3.wlEg_index:end)=0;
            %Sb2S3.alpha = Sb2S3.alpha*1e-7;
            %Sb2S3.alpha = Sb2S3.alpha*1e-9;
            Sb2S3.Voc = 1.1;        % Voc set assuming Sb2S3 luminescent efficiency of 5e-6, from TW paper
            Sb2S3.FF = 0.8;        % set fill factor of top cell to 0.8 as a first estimate, cf. TW model
            %Ag_ext = importdata('reference_data\Ag_Palik.xls');  %load AM1.5G spectra
            Sb2S3_ext = importdata('reference_data\Sb2S_ on_Si_Azul_2013-06-18.txt');  %load AM1.5G spectra
            Sb2S3_wl_raw = Sb2S3_ext.data(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            Sb2S3_n_raw = Sb2S3_ext.data(:,2);
            Sb2S3.n=interp1(Sb2S3_wl_raw,Sb2S3_n_raw,wavelengths,'spline','extrap');
            Sb2S3.n(Sb2S3.n<0)=0; %set all negative values to zero
            Sb2S3_k_raw = Sb2S3_ext.data(:,3);
            Sb2S3.k=interp1(Sb2S3_wl_raw,Sb2S3_k_raw,wavelengths,'spline','extrap');
            Sb2S3.k(Sb2S3.k<0)=0; %set all negative values to zero
            %make sure this is ok:
            %Sb2S3.k(Sb2S3.wlEg_index:end)=0;
            Sb2S3.alpha = (4.*pi.*Sb2S3.k)./(wavelengths.*1e-9); 
            Sb2S3.alpha = 1e-9.*Sb2S3.alpha; %getting into units of nm^-1
            Sb2S3.alpha(Sb2S3.alpha<0)=0; %set all negative values to zero
%            Sb2S3.n(1:length(wavelengths)) = 3.0;          % set refractive index of Sb2S3 - should set this to a proper vector of the values, but for now, a constant
%            Sb2S3.k(1:length(wavelengths)) = (Sb2S3.alpha .* wavelengths.*1e-9)./(4*pi);
            Sb2S3.Vbi=Sb2S3.Eg;
            Sb2S3.eps1 = Sb2S3.n.^2 - Sb2S3.k.^2;
            Sb2S3.eps2 = 2*Sb2S3.n.*Sb2S3.k;
%           Check this!!!
            Sb2S3.eps2(Sb2S3.wlEg_index:end)=0;
            
            
            Sb2S3.eps = complex(Sb2S3.eps1,Sb2S3.eps2);
            Sb2S3.nk = complex(Sb2S3.n,Sb2S3.k);
            assignin('base', 'Sb2S3',Sb2S3);
%            assignin('base', 'Sb2S3_n_raw',Sb2S3_n_raw)
%            assignin('base', 'Sb2S3_wl_raw',Sb2S3_wl_raw)
            
        end
        
         function nnl_import_aSiH(wavelengths)      % calculate the absorption coefficient for Sb2S3 using the empirical formula below
            aSiH.name = 'amorphous silicon';
            %Sb2S3.Astar0 = 3.1e7;   % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
            %aSiH.Astar0 = 1e4;   % Astar0 with dimensions of cm^-1!  With kT in the denominator of the expression below
            aSiH.Eg = 1.7;         % Eg in Ev, value taken from TW fit
            %Sb2S3.Eg = 1.4;         % Eg in Ev, value taken from TW fit
            aSiH.Ld = 180;          % Diffusion length Ld in nm % Layek et al 2012 J Phys Chem and Chem of Solids
            aSiH.phi = 1e-5;          % luminescence efficiency
            %Sb2S3.phi = 1;          % luminescence efficiency
            %aSiH.alpha = aSiH.Astar0.*(( (nnl_root.wlev(wavelengths) - aSiH.Eg)/(0.0256) ).^(1/2)); % denominator kT, with T = 298K
            aSiH.wlEg = nnl_root.wlev(aSiH.Eg);
            aSiH.wlEg_index = find(wavelengths==floor(aSiH.wlEg));
            %aSiH.alpha(aSiH.wlEg_index:end)=0;
            %aSiH.alpha = aSiH.alpha*1e-7;
            %Sb2S3.alpha = Sb2S3.alpha*1e-9;
            %aSiH.Voc = 1.1;        % Voc set assuming Sb2S3 luminescent efficiency of 5e-6, from TW paper
            aSiH.FF = 0.8;        % set fill factor of top cell to 0.8 as a first estimate, cf. TW model
            %Ag_ext = importdata('reference_data\Ag_Palik.xls');  %load AM1.5G spectra
            aSiH_ext = importdata('reference_data\Zeman_JApplPhys_aSi_wlnm_alphacm-1.csv');  %load AM1.5G spectra
            aSiH_wl_raw = aSiH_ext(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            %aSiH_n_raw = aSiH_ext.data(:,2);
            %aSiH.n=interp1(aSiH_wl_raw,aSiH_n_raw,wavelengths,'spline','extrap');
            aSiH_alpha_raw = aSiH_ext(:,2);
            aSiH.alpha=interp1(aSiH_wl_raw,aSiH_alpha_raw,wavelengths,'spline','extrap');
            aSiH.cutoff_index = find(wavelengths==848); % from inspection of Zeman JApplPhys 2000 optical data for intrinsic a-Si:H
            aSiH.alpha(aSiH.cutoff_index:end)=0;
            aSiH.alpha = 1e-7.*aSiH.alpha; %getting into units of nm^-1
            %aSiH_k_raw = aSiH_ext.data(:,3);
            %aSiH.k=interp1(aSiH_wl_raw,aSiH_k_raw,wavelengths,'spline','extrap');
            %make sure this is ok:
            %aSiH.k(aSiH.wlEg_index:end)=0;
            %aSiH.alpha = (4.*pi.*aSiH.k)./(wavelengths.*1e-9); %            
%            Sb2S3.n(1:length(wavelengths)) = 3.0;          % set refractive index of Sb2S3 - should set this to a proper vector of the values, but for now, a constant
%            Sb2S3.k(1:length(wavelengths)) = (Sb2S3.alpha .* wavelengths.*1e-9)./(4*pi);
           aSiH.Vbi=aSiH.Eg;
            %aSiH.eps1 = aSiH.n.^2 - aSiH.k.^2;
            %aSiH.eps2 = 2*aSiH.n.*aSiH.k;
%           Check this!!!
            %aSiH.eps2(aSiH.wlEg_index:end)=0;
            
            
%            aSiH.eps = complex(aSiH.eps1,aSiH.eps2);
 %           aSiH.nk = complex(aSiH.n,aSiH.k);
             assignin('base', 'aSiH',aSiH);
%            assignin('base', 'Sb2S3_n_raw',Sb2S3_n_raw)
%            assignin('base', 'Sb2S3_wl_raw',Sb2S3_wl_raw)
            
         end
         
         function nnl_import_CZTS(wavelengths)      % calculate the absorption coefficient for Sb2S3 using the empirical formula below
            CZTS.name = 'CZTS';
            %Sb2S3.Astar0 = 3.1e7;   % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
            %aSiH.Astar0 = 1e4;   % Astar0 with dimensions of cm^-1!  With kT in the denominator of the expression below
            CZTS.Eg = 1.5;         % Eg in Ev, value taken from TW fit
            %Sb2S3.Eg = 1.4;         % Eg in Ev, value taken from TW fit
            CZTS.Ld = 350;          % Diffusion length Ld in nm % Layek et al 2012 J Phys Chem and Chem of Solids
            CZTS.phi = 1e-5;          % luminescence efficiency
            %Sb2S3.phi = 1;          % luminescence efficiency
            %aSiH.alpha = aSiH.Astar0.*(( (nnl_root.wlev(wavelengths) - aSiH.Eg)/(0.0256) ).^(1/2)); % denominator kT, with T = 298K
            CZTS.wlEg = nnl_root.wlev(CZTS.Eg);
            CZTS.wlEg_index = find(wavelengths==floor(CZTS.wlEg));
            %aSiH.alpha(aSiH.wlEg_index:end)=0;
            %aSiH.alpha = aSiH.alpha*1e-7;
            %Sb2S3.alpha = Sb2S3.alpha*1e-9;
            %aSiH.Voc = 1.1;        % Voc set assuming Sb2S3 luminescent efficiency of 5e-6, from TW paper
            CZTS.FF = 0.8;        % set fill factor of top cell to 0.8 as a first estimate, cf. TW model
            %Ag_ext = importdata('reference_data\Ag_Palik.xls');  %load AM1.5G spectra
            CZTS_ext = importdata('reference_data\alpha_CZTS_LiOptExp_wl_alpha_m-1.dat');  %load AM1.5G spectra
            CZTS_wl_raw = CZTS_ext(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            CZTS_alpha_raw = CZTS_ext(:,2);
            %CZTS_n_raw = CZTS_ext.data(:,2);
            %CZTS.n=interp1(CZTS_wl_raw,CZTS_n_raw,wavelengths,'spline','extrap');
            %CZTS_k_raw = CZTS_ext.data(:,3);
            %CZTS.k=interp1(CZTS_wl_raw,CZTS_k_raw,wavelengths,'spline','extrap');
            %make sure this is ok:
            %CZTS.k(CZTS.wlEg_index:end)=0;
            CZTS.alpha = interp1(CZTS_wl_raw.*1e9,CZTS_alpha_raw,wavelengths,'spline','extrap'); %            
            %CZTS.alpha(CZTS.wlEg_index:end)=0;%            
            CZTS.alpha = 1e-9.*CZTS.alpha; %getting into units of nm^-1
            %CZTS.alpha = (4.*pi.*CZTS.k)./(wavelengths.*1e-9); %            
%            Sb2S3.n(1:length(wavelengths)) = 3.0;          % set refractive index of Sb2S3 - should set this to a proper vector of the values, but for now, a constant
%            Sb2S3.k(1:length(wavelengths)) = (Sb2S3.alpha .* wavelengths.*1e-9)./(4*pi);
            CZTS.Vbi=CZTS.Eg;
            %CZTS.eps1 = CZTS.n.^2 - CZTS.k.^2;
            %CZTS.eps2 = 2*CZTS.n.*CZTS.k;
%           Check this!!!
            %CZTS.eps2(CZTS.wlEg_index:end)=0;
            
            
            %CZTS.eps = complex(CZTS.eps1,CZTS.eps2);
            %CZTS.nk = complex(CZTS.n,CZTS.k);
            assignin('base', 'CZTS',CZTS);
%            assignin('base', 'Sb2S3_n_raw',Sb2S3_n_raw)
%            assignin('base', 'Sb2S3_wl_raw',Sb2S3_wl_raw)
            
         end
       
         function nnl_import_CIS(wavelengths)      % calculate the absorption coefficient for Sb2S3 using the empirical formula below
            CIS.name = 'CIS';
            %Sb2S3.Astar0 = 3.1e7;   % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
            %aSiH.Astar0 = 1e4;   % Astar0 with dimensions of cm^-1!  With kT in the denominator of the expression below
            CIS.Eg = 1.5;         % Eg in Ev, value taken from TW fit
            %Sb2S3.Eg = 1.4;         % Eg in Ev, value taken from TW fit
            CIS.Ld = 2000;          % Diffusion length Ld in nm % Unold APL 2006Unold APL 2006
            CIS.phi = 1e-5;          % luminescence efficiency
            %Sb2S3.phi = 1;          % luminescence efficiency
            %aSiH.alpha = aSiH.Astar0.*(( (nnl_root.wlev(wavelengths) - aSiH.Eg)/(0.0256) ).^(1/2)); % denominator kT, with T = 298K
            CIS.wlEg = nnl_root.wlev(CIS.Eg);
            CIS.wlEg_index = find(wavelengths==floor(CIS.wlEg));
            %aSiH.alpha(aSiH.wlEg_index:end)=0;
            %aSiH.alpha = aSiH.alpha*1e-7;
            %Sb2S3.alpha = Sb2S3.alpha*1e-9;
            %aSiH.Voc = 1.1;        % Voc set assuming Sb2S3 luminescent efficiency of 5e-6, from TW paper
            CIS.FF = 0.8;        % set fill factor of top cell to 0.8 as a first estimate, cf. TW model
            %Ag_ext = importdata('reference_data\Ag_Palik.xls');  %load AM1.5G spectra
            CIS_ext = importdata('reference_data\Guillen_SemicondSciTech_CIS_wl_m_alpha_m-1.csv');  %load AM1.5G spectra
            CIS_wl_raw = CIS_ext(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            %CIS_n_raw = CIS_ext(:,2);
            %CIS.n=interp1(CIS_wl_raw,CIS_n_raw,wavelengths,'spline','extrap');
            CIS_alpha_raw = CIS_ext(:,2);
            CIS.alpha=interp1(CIS_wl_raw.*1e9,CIS_alpha_raw,wavelengths,'spline','extrap');
            %CIS.alpha(CIS.wlEg_index:end)=0;
            CIS.lowercutoff_index = find(wavelengths==floor(339)); %from inspection of the CIS alpha data, this is where it goes negative
            CIS.alpha(1:CIS.lowercutoff_index) = 0;
            CIS.uppercutoff_index = find(wavelengths==floor(864)); %from inspection of the CIS alpha data, this is where it undergoes inflection due to the extrapolation
            CIS.alpha(CIS.uppercutoff_index:end) = 0;
           
            CIS.alpha = 1e-9.*CIS.alpha; %getting into units of nm^-1
            
            %CIS_k_raw = CIS_ext(:,3);
            %CIS.k=interp1(CIS_wl_raw,CIS_k_raw,wavelengths,'spline','extrap');
            %make sure this is ok:
            %CIS.k(CIS.wlEg_index:end)=0;
            %CIS.alpha = (4.*pi.*CIS.k)./(wavelengths.*1e-9); %            
%            Sb2S3.n(1:length(wavelengths)) = 3.0;          % set refractive index of Sb2S3 - should set this to a proper vector of the values, but for now, a constant
%            Sb2S3.k(1:length(wavelengths)) = (Sb2S3.alpha .* wavelengths.*1e-9)./(4*pi);
            CIS.Vbi=CIS.Eg;
            %CIS.eps1 = CIS.n.^2 - CIS.k.^2;
            %CIS.eps2 = 2*CIS.n.*CIS.k;
%           Check this!!!
            %CIS.eps2(CIS.wlEg_index:end)=0;
            
            
            %CIS.eps = complex(CIS.eps1,CIS.eps2);
            %CIS.nk = complex(CIS.n,CIS.k);
            assignin('base', 'CIS',CIS);
%            assignin('base', 'Sb2S3_n_raw',Sb2S3_n_raw)
%            assignin('base', 'Sb2S3_wl_raw',Sb2S3_wl_raw)
            
         end
         
         function nnl_import_CIGS(wavelengths)      % calculate the absorption coefficient for Sb2S3 using the empirical formula below
            CIGS.name = 'CIGS';
            %Sb2S3.Astar0 = 3.1e7;   % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
            %aSiH.Astar0 = 1e4;   % Astar0 with dimensions of cm^-1!  With kT in the denominator of the expression below
            CIGS.Eg = 1.12;         % Taken from Merdes PiP 2013 21:88-93 
            %Sb2S3.Eg = 1.4;         % Eg in Ev, value taken from TW fit
            CIGS.Ld = 2000;          % Diffusion length Ld in nm % Unold APL 2006Unold APL 2006
            CIGS.phi = 1e-5;          % luminescence efficiency
            %Sb2S3.phi = 1;          % luminescence efficiency
            %aSiH.alpha = aSiH.Astar0.*(( (nnl_root.wlev(wavelengths) - aSiH.Eg)/(0.0256) ).^(1/2)); % denominator kT, with T = 298K
            CIGS.wlEg = nnl_root.wlev(CIGS.Eg);
            CIGS.wlEg_index = find(wavelengths==floor(CIGS.wlEg));
            %aSiH.alpha(aSiH.wlEg_index:end)=0;
            %aSiH.alpha = aSiH.alpha*1e-7;
            %Sb2S3.alpha = Sb2S3.alpha*1e-9;
            %aSiH.Voc = 1.1;        % Voc set assuming Sb2S3 luminescent efficiency of 5e-6, from TW paper
            CIGS.FF = 0.8;        % set fill factor of top cell to 0.8 as a first estimate, cf. TW model
            %Ag_ext = importdata('reference_data\Ag_Palik.xls');  %load AM1.5G spectra
            CIGS_ext = importdata('reference_data\Han2004APL_alphas_CIGS_stochiometric_cleaned.txt');  %load AM1.5G spectra
            CIGS_ev_raw = smooth(CIGS_ext.data(:,1));                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            CIGS_alpha_raw = smooth(CIGS_ext.data(:,2)); %in cm^-1
            
            CIGS_wl_raw = nnl_root.wlev(CIGS_ev_raw);
            CIGS_wl_raw_ordered = nnl_root.swapwave(CIGS_wl_raw);
            CIGS_alpha_raw_ordered = nnl_root.swapwave(CIGS_alpha_raw); %in cm^-1
            wavelengths_transpose = transpose(wavelengths);
            CIGS.alpha=interp1(CIGS_wl_raw_ordered,CIGS_alpha_raw_ordered,wavelengths_transpose,'linear',0);
            %CIGS.alpha=interp1(CIGS_wl_raw,CIGS_alpha_raw,wavelengths_transpose,'spline','extrap');
            
            %CIS.alpha(CIS.wlEg_index:end)=0;
            CIGS.alpha = 1e-7.*CIGS.alpha; %getting into units of nm^-1
            CIGS.alpha = transpose(CIGS.alpha);
            %CIS_k_raw = CIS_ext(:,3);
            %CIS.k=interp1(CIS_wl_raw,CIS_k_raw,wavelengths,'spline','extrap');
            %make sure this is ok:
            %CIS.k(CIS.wlEg_index:end)=0;
            %CIS.alpha = (4.*pi.*CIS.k)./(wavelengths.*1e-9); %            
%            Sb2S3.n(1:length(wavelengths)) = 3.0;          % set refractive index of Sb2S3 - should set this to a proper vector of the values, but for now, a constant
%            Sb2S3.k(1:length(wavelengths)) = (Sb2S3.alpha .* wavelengths.*1e-9)./(4*pi);
            CIGS.Vbi=CIGS.Eg;
            %CIS.eps1 = CIS.n.^2 - CIS.k.^2;
            %CIS.eps2 = 2*CIS.n.*CIS.k;
%           Check this!!!
            %CIS.eps2(CIS.wlEg_index:end)=0;
            
            
            %CIS.eps = complex(CIS.eps1,CIS.eps2);
            %CIS.nk = complex(CIS.n,CIS.k);
            assignin('base', 'CIGS',CIGS);
%            assignin('base', 'Sb2S3_n_raw',Sb2S3_n_raw)
%            assignin('base', 'Sb2S3_wl_raw',Sb2S3_wl_raw)
            
         end
        function nnl_import_fictional(wavelengths)      % calculate the absorption coefficient for Sb2S3 using the empirical formula below
            fictional.name = 'fictional';
            %Sb2S3.Astar0 = 3.1e7;   % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
            fictional.Astar0 = 1e6;   % Astar0 with dimensions of cm^-1!  With kT in the denominator of the expression below
            fictional.Eg = 1.98;         % Eg in Ev, value taken from TW fit
            %Sb2S3.Eg = 1.4;         % Eg in Ev, value taken from TW fit
            fictional.Ld = 2000000;          % Diffusion length Ld in nm % Unold APL 2006Unold APL 2006
            fictional.phi = 1;          % luminescence efficiency
            %Sb2S3.phi = 1;          % luminescence efficiency
            fictional.wlEg = nnl_root.wlev(fictional.Eg);
            fictional.wlEg_index = find(wavelengths==floor(fictional.wlEg));
            fictional.alpha = fictional.Astar0.*(( (nnl_root.wlev(wavelengths) - fictional.Eg)/(0.0256) ).^(1/2)); % denominator kT, with T = 298K
            fictional.alpha(fictional.wlEg_index:end)=0;
            fictional.alpha = fictional.alpha*1e-7;
            %aSiH.alpha(aSiH.wlEg_index:end)=0;
            %aSiH.alpha = aSiH.alpha*1e-7;
            %Sb2S3.alpha = Sb2S3.alpha*1e-9;
            %aSiH.Voc = 1.1;        % Voc set assuming Sb2S3 luminescent efficiency of 5e-6, from TW paper
            fictional.FF = 1 ;%0.8;        % set fill factor of top cell to 0.8 as a first estimate, cf. TW model
            %Ag_ext = importdata('reference_data\Ag_Palik.xls');  %load AM1.5G spectra
            %CIS_ext = importdata('reference_data\Guillen_SemicondSciTech_CIS_wl_m_alpha_m-1.csv');  %load AM1.5G spectra
            %CIS_wl_raw = CIS_ext(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            %CIS_n_raw = CIS_ext(:,2);
            %CIS.n=interp1(CIS_wl_raw,CIS_n_raw,wavelengths,'spline','extrap');
            %CIS_alpha_raw = CIS_ext(:,2);
            %fictional.alpha=interp1(CIS_wl_raw.*1e9,CIS_alpha_raw,wavelengths,'spline','extrap');
            %fictional.alpha(fictional.wlEg_index:end)=0;
            %fictional.alpha = 1e-9.*fictional.alpha; %getting into units of nm^-1
            
            %CIS_k_raw = CIS_ext(:,3);
            %CIS.k=interp1(CIS_wl_raw,CIS_k_raw,wavelengths,'spline','extrap');
            %make sure this is ok:
            %CIS.k(CIS.wlEg_index:end)=0;
            %CIS.alpha = (4.*pi.*CIS.k)./(wavelengths.*1e-9); %            
%            Sb2S3.n(1:length(wavelengths)) = 3.0;          % set refractive index of Sb2S3 - should set this to a proper vector of the values, but for now, a constant
%            Sb2S3.k(1:length(wavelengths)) = (Sb2S3.alpha .* wavelengths.*1e-9)./(4*pi);
            fictional.Vbi =  fictional.Eg;
            %CIS.eps1 = CIS.n.^2 - CIS.k.^2;
            %CIS.eps2 = 2*CIS.n.*CIS.k;
%           Check this!!!
            %CIS.eps2(CIS.wlEg_index:end)=0;
            
            
            %CIS.eps = complex(CIS.eps1,CIS.eps2);
            %CIS.nk = complex(CIS.n,CIS.k);
            assignin('base', 'fictional',fictional);
%            assignin('base', 'Sb2S3_n_raw',Sb2S3_n_raw)
%            assignin('base', 'Sb2S3_wl_raw',Sb2S3_wl_raw)
            
        end
        
        function nnl_import_PMMA(wavelengths)      % calculate the absorption coefficient for Sb2S3 using the empirical formula below
            PMMA.name = 'PMMA';
            PMMA_ext = importdata('reference_data\PMMA_nk_Cauchy_txt.txt');  %load AM1.5G spectra Corresponds with Beadie 2015
            PMMA_wl_raw = PMMA_ext(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            PMMA_n_raw = PMMA_ext(:,2);
            PMMA.n=interp1(PMMA_wl_raw,PMMA_n_raw,wavelengths,'spline','extrap');
            PMMA.k = PMMA.n;
            PMMA.k(:) = 0;
            PMMA.nk = complex(PMMA.n,PMMA.k);
            assignin('base', 'PMMA',PMMA);
        end
        
        function nnl_import_NaYF4(wavelengths)      % calculate the absorption coefficient for Sb2S3 using the empirical formula below
            NaYF4.name = 'PMMA';
            NaYF4_ext = importdata('reference_data\NaYF4_txt.txt');  %load AM1.5G spectra
            NaYF4_wl_raw = (1e3)*NaYF4_ext(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            NaYF4_n_raw = NaYF4_ext(:,2);
            NaYF4.n=interp1(NaYF4_wl_raw,NaYF4_n_raw,wavelengths,'spline','extrap');
            assignin('base', 'NaYF4',NaYF4);
        end
        
        function nnl_import_SiO2(wavelengths)
            SiO2.name = 'SiO2';
            SiO2.n = zeros(1,length(wavelengths));
            SiO2.n(1,:) = 1.46;
            SiO2.k = zeros(1,length(wavelengths));
            SiO2.k(1,:) = 0.00;
            SiO2.eps1 = SiO2.n.^2 - SiO2.k.^2;
            SiO2.eps2 = 2*SiO2.n.*SiO2.k;
            SiO2.eps = complex(SiO2.eps1,SiO2.eps2);
            SiO2.nk = complex(SiO2.n,SiO2.k);
            assignin('base', 'SiO2',SiO2);
        end
        
        function nnl_import_TiO2(wavelengths)
            TiO2.name = 'TiO2';
            TiO2.n = zeros(1,length(wavelengths));
            TiO2.n(1,:) = 2.4;
            TiO2.k = zeros(1,length(wavelengths));
            TiO2.k(1,:) = 0.00;
            TiO2.eps1 = TiO2.n.^2 - TiO2.k.^2;
            TiO2.eps2 = 2*TiO2.n.*TiO2.k;
            TiO2.eps = complex(TiO2.eps1,TiO2.eps2);
            TiO2.nk = complex(TiO2.n,TiO2.k);
            assignin('base', 'TiO2',TiO2);
        end
        function nnl_import_ZnO(wavelengths)
            ZnO.name = 'ZnO';
            ZnO.n = zeros(1,length(wavelengths));
            ZnO.n(1,:) = 1.6;
            ZnO.k = zeros(1,length(wavelengths));
            ZnO.k(1,:) = 1.00;
            ZnO.eps1 = ZnO.n.^2 - ZnO.k.^2;
            ZnO.eps2 = 2*ZnO.n.*ZnO.k;
            ZnO.eps = complex(ZnO.eps1,ZnO.eps2);
            ZnO.nk = complex(ZnO.n,ZnO.k);
            assignin('base', 'ZnO',ZnO);
        end
        
         function nnl_import_air(wavelengths)
            air.name = 'air';
            air.n = zeros(1,length(wavelengths));
            air.n(1,:) = 1;
            air.k = zeros(1,length(wavelengths));
            air.k(1,:) = 0.00;
            air.eps1 = air.n.^2 - air.k.^2;
            air.eps2 = 2*air.n.*air.k;
            air.eps = complex(air.eps1,air.eps2);
            air.nk = complex(air.n,air.k);
            assignin('base', 'air',air);
         end
        
         function nnl_import_glass(wavelengths)
            glass.name = 'glassr';
            glass.n = zeros(1,length(wavelengths));
            glass.n(1,:) = 1.5;
            glass.k = zeros(1,length(wavelengths));
            glass.k(1,:) = 0.00;
            glass.eps1 = glass.n.^2 - glass.k.^2;
            glass.eps2 = 2*glass.n.*glass.k;
            glass.eps = complex(glass.eps1,glass.eps2);
            glass.nk = complex(glass.n,glass.k);
            assignin('base', 'glass',glass);
         end
         
         function nnl_import_pdms(wavelengths)
            pdms.name = 'pdms';
            pdms.n = zeros(1,length(wavelengths));
            pdms.n(1,:) = 1.4;
            pdms.k = zeros(1,length(wavelengths));
            pdms.k(1,:) = 0.00;
            pdms.eps1 = pdms.n.^2 - pdms.k.^2;
            pdms.eps2 = 2*pdms.n.*pdms.k;
            pdms.eps = complex(pdms.eps1,pdms.eps2);
            pdms.nk = complex(pdms.n,pdms.k);
            assignin('base', 'pdms',pdms);
         end
        
         function nnl_import_ITO(wavelengths)      % calculate the absorption coefficient for Sb2S3 using the empirical formula below
            ITO.name = 'ITO';
            ITO_extn = importdata('reference_data\synowicki_ITO_n.txt');  %load AM1.5G spectra
            ITO_nwl_raw = ITO_extn(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            ITO_n_raw = ITO_extn(:,2);
            ITO.n=interp1(ITO_nwl_raw,ITO_n_raw,wavelengths,'spline','extrap');
            ITO.n(ITO.n>2.5)=2.5; % setting an upper bound so the interpolation isn't crazy
            ITO.n = transpose(smooth(ITO.n,101));
            ITO_extk = importdata('reference_data\synowicki_ITO_k.txt');  %load AM1.5G spectra
            ITO_kwl_raw = ITO_extk(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            ITO_k_raw = ITO_extk(:,2);
            ITO.k=interp1(ITO_kwl_raw,ITO_k_raw,wavelengths,'spline','extrap');
            ITO.k(ITO.k<0)=0; %ensuring no negative k comes from the interpolation
            ITO.k = transpose(smooth(ITO.k,101));
            ITO.alpha = (4.*pi.*ITO.k)./(wavelengths.*1e-9); 
            ITO.alpha = 1e-9.*ITO.alpha; %getting into units of nm^-1
            ITO.nk = complex(ITO.n,ITO.k);
            ITO.Eg = 4;         % Eg in Ev, value taken from TW fit
            ITO.wlEg = nnl_root.wlev(ITO.Eg);
            ITO.wlEg_index = find(wavelengths==floor(ITO.wlEg));
            
            assignin('base', 'ITO_Synowicki',ITO);
        end
         
         function nnl_effective_medium(wavelengths, n1, n2, fillfraction)
             effmedium.name = 'effective_medium';
             effmedium.inclusioneps = n1.^2;
             effmedium.matrixeps = n2.^2;
             effmedium.eps1 = effmedium.matrixeps.*( ( effmedium.inclusioneps.*(1+2*fillfraction) - effmedium.matrixeps.*(2*fillfraction-2))./ ( effmedium.matrixeps.*(2+fillfraction) + effmedium.inclusioneps.*(1 - fillfraction) ) );
             effmedium.eps2 = zeros(1,length(wavelengths));
             effmedium.n = effmedium.eps1.^(1/2);
             effmedium.k = zeros(1,length(wavelengths));
             effmedium.eps = complex(effmedium.eps1, effmedium.eps2);
             effmedium.nk = complex(effmedium.n, effmedium.k);
             assignin('base','effmedium', effmedium);
         end
       
         function nnl_import_perovskite(wavelengths)      % calculate the absorption coefficient for Perovskite using the empirical formula below
            perovskite.name = 'perovskite';
            %Sb2S3.Astar0 = 3.1e7;   % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
            perovskite.Astar0 = 1e4;   % Astar0 with dimensions of cm^-1!  With kT in the denominator of the expression below
            perovskite.Eg = 1.55;         % Eg in Ev, value taken from TW fit
            %Sb2S3.Eg = 1.4;         % Eg in Ev, value taken from TW fit
            %perovskite.Ld = 1100;          % Diffusion length Ld in nm
            perovskite.Ld = 1100;          % Diffusion length Ld in nm
            perovskite.phi = 0.55;          % luminescence efficiency
            %Sb2S3.phi = 1;          % luminescence efficiency
            perovskite.alpha = perovskite.Astar0.*(( (nnl_root.wlev(wavelengths) - perovskite.Eg)/(0.0256) ).^(1/2)); % denominator kT, with T = 298K
            perovskite.wlEg = nnl_root.wlev(perovskite.Eg);
            perovskite.wlEg_index = find(wavelengths==floor(perovskite.wlEg));
            perovskite.alpha(perovskite.wlEg_index:end)=0;
            perovskite.alpha = perovskite.alpha*1e-7;
            %Sb2S3.alpha = Sb2S3.alpha*1e-9;
            %perovskite.Voc = 1.1;        % Voc set assuming Sb2S3 luminescent efficiency of 5e-6, from TW paper
            perovskite.FF = 0.8;        % set fill factor of top cell to 0.8 as a first estimate, cf. TW model
            %Ag_ext = importdata('reference_data\Ag_Palik.xls');  %load AM1.5G spectra
%             modelmat_ext = importdata('reference_data\nSbS.xls');  %load AM1.5G spectra
%             modelmat_ext2 = importdata('reference_data\kSbS.xls');  %load AM1.5G spectra
%             modelmat_wl_raw = (1e9)*modelmat_ext.Sheet1(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
%             modelmat_n_raw = modelmat_ext.Sheet1(:,2);
%             perovskite.n=interp1(modelmat_wl_raw,modelmat_n_raw,wavelengths,'spline','extrap');
%             modelmat_k_raw = modelmat_ext2.Sheet1(:,2);
%             perovskite.k=interp1(modelmat_wl_raw,modelmat_k_raw,wavelengths,'spline','extrap');
%             % make sure this is ok:
            %modelmat.k(Sb2S3.wlEg_index:end)=0;
            %
            %modelmat.n(1:length(wavelengths)) = 3.0;          % set refractive index of Sb2S3 - should set this to a proper vector of the values, but for now, a constant
            %modelmat.k(1:length(wavelengths)) = (modelmat.alpha .* wavelengths.*1e-9)./(4*pi);
            perovskite.Vbi=perovskite.Eg;
%             perovskite.eps1 = perovskite.n.^2 - perovskite.k.^2;
%             perovskite.eps2 = 2*perovskite.n.*perovskite.k;
%             % Check this!!!
%             perovskite.eps2(perovskite.wlEg_index:end)=0;
%             %
%             
%             perovskite.eps = complex(perovskite.eps1,perovskite.eps2);
%             perovskite.nk = complex(perovskite.n,perovskite.k);
            assignin('base', 'perovskite',perovskite);
            %assignin('base', 'Sb2S3_n_raw',Sb2S3_n_raw)
            %assignin('base', 'Sb2S3_wl_raw',Sb2S3_wl_raw)
            
         end
        
        function nnl_import_perovskite_1(wavelengths, perovskite, aSiH)
            perovskite_1 = perovskite;
            perovskite_1.name = 'model perovskite with parasitic alpha 1';
            %perovskite_1.alpha(1,perovskite.wlEg_index: perovskite.wlEg_index + (find(aSiH.alpha==0,1) ) - (find(wavelengths==671))) = aSiH.alpha(1, find(wavelengths==671): find(aSiH.alpha==0,1));
            perovskite_1.alpha(1,perovskite.wlEg_index: end)= aSiH.alpha(1, perovskite.wlEg_index: end)./(aSiH.alpha(1,perovskite.wlEg_index-1)/perovskite.alpha(perovskite.wlEg_index-1));
            perovskite_1exp = perovskite_1;     
            perovskite_1exp.alpha(1,450:569)=4.882e10.*exp(-0.04475.*(wavelengths(1,450:569)));% Indices need to be changed if wl neq 280:1:1400
            assignin('base', 'perovskite_1exp',perovskite_1exp);
            assignin('base', 'perovskite_1',perovskite_1);
        end
        
        function nnl_import_perovskite_2(wavelengths, perovskite, Sb2S3)
            perovskite_2 = perovskite;
            perovskite_2.name = 'model perovskite with parasitic alpha 2';
            perovskite_2.alpha(1,perovskite.wlEg_index: end)= Sb2S3.alpha(1, perovskite.wlEg_index: end)./(Sb2S3.alpha(1,perovskite.wlEg_index-1)/perovskite.alpha(perovskite.wlEg_index-1));
            perovskite_2exp = perovskite_2;       
            perovskite_2exp.alpha(1,450:end) = 0.005276.*exp(-0.003909.*wavelengths(1,450:end)); % Indices need to be changed if wl neq 280:1:1400
            assignin('base', 'perovskite_2exp',perovskite_2exp);
            assignin('base', 'perovskite_2',perovskite_2);
        end
         
         function nnl_import_SiON_low(wavelengths)      % get the SiON n data from Andy Thomson's experimental values
            SiON_low.name = 'SiON_low';
            
            SiON_low_ext = importdata('reference_data\Andy_SiON_ref_n_data\Andy_SiON_high_low_ns.xlsx');  %load AM1.5G spectra
            SiON_low_wl_raw = SiON_low_ext.data.Sheet1(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            SiON_low_n_raw = SiON_low_ext.data.Sheet1(:,2);
            SiON_low.n=interp1(SiON_low_wl_raw,SiON_low_n_raw,wavelengths,'spline','extrap');
            SiON_low.n(SiON_low.n<0)=0; %set all negative values to zero
            SiON_low_k_raw = SiON_low_ext.data.Sheet1(:,3);
            SiON_low.k=interp1(SiON_low_wl_raw,SiON_low_k_raw,wavelengths,'spline','extrap');
            SiON_low.k(SiON_low.k<0)=0; %set all negative values to zero
            %make sure this is ok:
            SiON_low.alpha = (4.*pi.*SiON_low.k)./(wavelengths.*1e-9); 
            SiON_low.alpha = 1e-9.*SiON_low.alpha; %getting into units of nm^-1
            SiON_low.alpha(SiON_low.alpha<0)=0; %set all negative values to zero
            SiON_low.eps1 = SiON_low.n.^2 - SiON_low.k.^2;
            SiON_low.eps2 = 2*SiON_low.n.*SiON_low.k;
%           Check this!!!
            SiON_low.eps = complex(SiON_low.eps1,SiON_low.eps2);
            SiON_low.nk = complex(SiON_low.n,SiON_low.k);
            assignin('base', 'SiON_low',SiON_low);
            
          end
          
         function nnl_import_SiON_high(wavelengths)      % get the SiON n data from Andy Thomson's experimental values
            SiON_high.name = 'SiON_high';
            SiON_high_ext = importdata('reference_data\Andy_SiON_ref_n_data\Andy_SiON_high_low_ns.xlsx');  %load AM1.5G spectra
            SiON_high_wl_raw = SiON_high_ext.data.Sheet1(:,4);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            SiON_high_n_raw = SiON_high_ext.data.Sheet1(:,5);
            SiON_high.n=interp1(SiON_high_wl_raw,SiON_high_n_raw,wavelengths,'spline','extrap');
            SiON_high.n(SiON_high.n<0)=0; %set all negative values to zero
            SiON_high_k_raw = SiON_high_ext.data.Sheet1(:,6);
            SiON_high.k=interp1(SiON_high_wl_raw,SiON_high_k_raw,wavelengths,'spline','extrap');
            SiON_high.k(SiON_high.k<0)=0; %set all negative values to zero
            %make sure this is ok:
            SiON_high.alpha = (4.*pi.*SiON_high.k)./(wavelengths.*1e-9); 
            SiON_high.alpha = 1e-9.*SiON_high.alpha; %getting into units of nm^-1
            SiON_high.alpha(SiON_high.alpha<0)=0; %set all negative values to zero
            SiON_high.eps1 = SiON_high.n.^2 - SiON_high.k.^2;
            SiON_high.eps2 = 2*SiON_high.n.*SiON_high.k;
%           Check this!!!
            SiON_high.eps = complex(SiON_high.eps1,SiON_high.eps2);
            SiON_high.nk = complex(SiON_high.n,SiON_high.k);
            assignin('base', 'SiON_high',SiON_high);
         end 
       
       function nnl_import_FTO(wavelengths)      % get the SiON n data from Andy Thomson's experimental values
            FTO.name = 'FTO';
            FTO_ext = importdata('reference_data\full_range_nk_data_Dale\FTO.txt');  %load AM1.5G spectra
            FTO_wl_raw = FTO_ext(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            FTO_n_raw = FTO_ext(:,2);
            FTO.n=interp1(FTO_wl_raw,FTO_n_raw,wavelengths,'spline','extrap');
            FTO.n(FTO.n<0)=0; %set all negative values to zero
            FTO_k_raw = FTO_ext(:,3);
            FTO.k=interp1(FTO_wl_raw,FTO_k_raw,wavelengths,'spline','extrap');
            FTO.k(FTO.k<0)=0; %set all negative values to zero
            %make sure this is ok:
            FTO.alpha = (4.*pi.*FTO.k)./(wavelengths.*1e-9); 
            FTO.alpha = 1e-9.*FTO.alpha; %getting into units of nm^-1
            FTO.alpha(FTO.alpha<0)=0; %set all negative values to zero
            FTO.eps1 = FTO.n.^2 - FTO.k.^2;
            FTO.eps2 = 2*FTO.n.*FTO.k;
%           Check this!!!
            FTO.eps = complex(FTO.eps1,FTO.eps2);
            FTO.nk = complex(FTO.n,FTO.k);
            assignin('base', 'FTO',FTO);
       end 
         
       function nnl_import_FTO_Wenger(wavelengths)      % get the SiON n data from Andy Thomson's experimental values
            FTO_Wenger.name = 'FTO_Wenger';
            FTO_Wenger_ext = importdata('reference_data\full_range_nk_data_Dale\FTO_Wenger.txt');  %load AM1.5G spectra
            FTO_Wenger_wl_raw = FTO_Wenger_ext(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            FTO_Wenger_n_raw = FTO_Wenger_ext(:,2);
            FTO_Wenger.n=interp1(FTO_Wenger_wl_raw,FTO_Wenger_n_raw,wavelengths,'spline','extrap');
            FTO_Wenger.n(FTO_Wenger.n<0)=0; %set all negative values to zero
            FTO_Wenger_k_raw = FTO_Wenger_ext(:,3);
            FTO_Wenger.k=interp1(FTO_Wenger_wl_raw,FTO_Wenger_k_raw,wavelengths,'spline','extrap');
            FTO_Wenger.k(FTO_Wenger.k<0)=0; %set all negative values to zero
            %make sure this is ok:
            FTO_Wenger.alpha = (4.*pi.*FTO_Wenger.k)./(wavelengths.*1e-9); 
            FTO_Wenger.alpha = 1e-9.*FTO_Wenger.alpha; %getting into units of nm^-1
            FTO_Wenger.alpha(FTO_Wenger.alpha<0)=0; %set all negative values to zero
            FTO_Wenger.eps1 = FTO_Wenger.n.^2 - FTO_Wenger.k.^2;
            FTO_Wenger.eps2 = 2*FTO_Wenger.n.*FTO_Wenger.k;
%           Check this!!!
            FTO_Wenger.eps = complex(FTO_Wenger.eps1,FTO_Wenger.eps2);
            FTO_Wenger.nk = complex(FTO_Wenger.n,FTO_Wenger.k);
            assignin('base', 'FTO_Wenger',FTO_Wenger);
       end 
        
       function nnl_import_Al_Rakic(wavelengths)      % get the SiON n data from Andy Thomson's experimental values
            Al_Rakic.name = 'Al_Rakic';
            Al_Rakic_ext_n = importdata('reference_data\Al_Rakic_1998_n.txt');  %load AM1.5G spectra
            Al_Rakic_ext_k = importdata('reference_data\Al_Rakic_1998_k.txt');  %load AM1.5G spectra
            Al_Rakic_wl_raw = 1e3*Al_Rakic_ext_n.data(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            Al_Rakic_n_raw = Al_Rakic_ext_n.data(:,2);
            Al_Rakic.n=interp1(Al_Rakic_wl_raw,Al_Rakic_n_raw,wavelengths,'spline','extrap');
            Al_Rakic.n(Al_Rakic.n<0)=0; %set all negative values to zero
            Al_Rakic_k_raw = Al_Rakic_ext_k.data(:,2);
            Al_Rakic.k=interp1(Al_Rakic_wl_raw,Al_Rakic_k_raw,wavelengths,'spline','extrap');
            Al_Rakic.k(Al_Rakic.k<0)=0; %set all negative values to zero
            %make sure this is ok:
            Al_Rakic.alpha = (4.*pi.*Al_Rakic.k)./(wavelengths.*1e-9); 
            Al_Rakic.alpha = 1e-9.*Al_Rakic.alpha; %getting into units of nm^-1
            Al_Rakic.alpha(Al_Rakic.alpha<0)=0; %set all negative values to zero
            Al_Rakic.eps1 = Al_Rakic.n.^2 - Al_Rakic.k.^2;
            Al_Rakic.eps2 = 2*Al_Rakic.n.*Al_Rakic.k;
%           Check this!!!
            Al_Rakic.eps = complex(Al_Rakic.eps1,Al_Rakic.eps2);
            Al_Rakic.nk = complex(Al_Rakic.n,Al_Rakic.k);
            assignin('base', 'Al_Rakic',Al_Rakic);
       end
       
       function nnl_import_Ni_JC(wavelengths)      % get the SiON n data from Andy Thomson's experimental values
            Ni_JC.name = 'Ni_JC';
            Ni_JC_ext_n = importdata('reference_data\Ni_Johnson_Christy_PRB_1974_n.txt');  %load AM1.5G spectra
            Ni_JC_ext_k = importdata('reference_data\Ni_Johnson_Christy_PRB_1974_k.txt');  %load AM1.5G spectra
            Ni_JC_wl_raw = 1e3*Ni_JC_ext_n.data(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            Ni_JC_n_raw = Ni_JC_ext_n.data(:,2);
            Ni_JC.n=interp1(Ni_JC_wl_raw,Ni_JC_n_raw,wavelengths,'spline','extrap');
            Ni_JC.n(Ni_JC.n<0)=0; %set all negative values to zero
            Ni_JC_k_raw = Ni_JC_ext_k.data(:,2);
            Ni_JC.k=interp1(Ni_JC_wl_raw,Ni_JC_k_raw,wavelengths,'spline','extrap');
            Ni_JC.k(Ni_JC.k<0)=0; %set all negative values to zero
            %make sure this is ok:
            Ni_JC.alpha = (4.*pi.*Ni_JC.k)./(wavelengths.*1e-9); 
            Ni_JC.alpha = 1e-9.*Ni_JC.alpha; %getting into units of nm^-1
            Ni_JC.alpha(Ni_JC.alpha<0)=0; %set all negative values to zero
            Ni_JC.eps1 = Ni_JC.n.^2 - Ni_JC.k.^2;
            Ni_JC.eps2 = 2*Ni_JC.n.*Ni_JC.k;
%           Check this!!!
            Ni_JC.eps = complex(Ni_JC.eps1,Ni_JC.eps2);
            Ni_JC.nk = complex(Ni_JC.n,Ni_JC.k);
            assignin('base', 'Ni_JC',Ni_JC);
       end 
         
       function nnl_import_ZnO_Bond(wavelengths)      % get the SiON n data from Andy Thomson's experimental values
            ZnO_Bond.name = 'ZnO_Bond';
            ZnO_Bond_ext_n = importdata('reference_data\ZnO_Bond_JAP_1965.txt');  %load AM1.5G spectra
            %ZnO_Bond_ext_k = importdata('reference_data\Ni_Johnson_Christy_PRB_1974_k.txt');  %load AM1.5G spectra
            ZnO_Bond_wl_raw = 1e3.*ZnO_Bond_ext_n.data(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            ZnO_Bond_n_raw = ZnO_Bond_ext_n.data(:,2);
            ZnO_Bond.n=interp1(ZnO_Bond_wl_raw,ZnO_Bond_n_raw,wavelengths,'spline','extrap');
            ZnO_Bond.n(ZnO_Bond.n<0)=0; %set all negative values to zero
            ZnO_Bond.k = ZnO_Bond.n;
            ZnO_Bond.k(:)=0;  %no k values in literature?
            
            %make sure this is ok:
            ZnO_Bond.alpha = (4.*pi.*ZnO_Bond.k)./(wavelengths.*1e-9); 
            ZnO_Bond.alpha = 1e-9.*ZnO_Bond.alpha; %getting into units of nm^-1
            ZnO_Bond.alpha(ZnO_Bond.alpha<0)=0; %set all negative values to zero
            ZnO_Bond.eps1 = ZnO_Bond.n.^2 - ZnO_Bond.k.^2;
            ZnO_Bond.eps2 = 2*ZnO_Bond.n.*ZnO_Bond.k;
%           Check this!!!
            ZnO_Bond.eps = complex(ZnO_Bond.eps1,ZnO_Bond.eps2);
            ZnO_Bond.nk = complex(ZnO_Bond.n,ZnO_Bond.k);
            assignin('base', 'ZnO_Bond',ZnO_Bond);
       end 
       
       function nnl_import_Al2O3_Malitson(wavelengths)      % get the SiON n data from Andy Thomson's experimental values
            Al2O3_Malitson.name = 'Al2O3_Malitson';
            Al2O3_Malitson_ext_n = importdata('reference_data\Al2O3_Malitson_Dodge_JOSA_1972.txt');  %load AM1.5G spectra
            %ZnO_Bond_ext_k = importdata('reference_data\Ni_Johnson_Christy_PRB_1974_k.txt');  %load AM1.5G spectra
            Al2O3_Malitson_wl_raw = 1e3*Al2O3_Malitson_ext_n.data(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            Al2O3_Malitson_n_raw = Al2O3_Malitson_ext_n.data(:,2);
            Al2O3_Malitson.n=interp1(Al2O3_Malitson_wl_raw,Al2O3_Malitson_n_raw,wavelengths,'spline','extrap');
            Al2O3_Malitson.n(Al2O3_Malitson.n<0)=0; %set all negative values to zero
            Al2O3_Malitson.k = Al2O3_Malitson.n;
            Al2O3_Malitson.k(:)=0;  %no k values in literature?
            
            %make sure this is ok:
            Al2O3_Malitson.alpha = (4.*pi.*Al2O3_Malitson.k)./(wavelengths.*1e-9); 
            Al2O3_Malitson.alpha = 1e-9.*Al2O3_Malitson.alpha; %getting into units of nm^-1
            Al2O3_Malitson.alpha(Al2O3_Malitson.alpha<0)=0; %set all negative values to zero
            Al2O3_Malitson.eps1 = Al2O3_Malitson.n.^2 - Al2O3_Malitson.k.^2;
            Al2O3_Malitson.eps2 = 2*Al2O3_Malitson.n.*Al2O3_Malitson.k;
%           Check this!!!
            Al2O3_Malitson.eps = complex(Al2O3_Malitson.eps1,Al2O3_Malitson.eps2);
            Al2O3_Malitson.nk = complex(Al2O3_Malitson.n,Al2O3_Malitson.k);
            assignin('base', 'Al2O3_Malitson',Al2O3_Malitson);
       end 
       
         function nnl_import_Perovskite_Loper(wavelengths)      % get the SiON n data from Andy Thomson's experimental values
            Perovskite_Loper.name = 'Perovskite_Loper';
            Perovskite_Loper_ext = importdata('reference_data\full_range_nk_data_Dale\Perovskite_Loper_E_u_080.txt');  %load AM1.5G spectra
            Perovskite_Loper_wl_raw = Perovskite_Loper_ext(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            Perovskite_Loper_n_raw = Perovskite_Loper_ext(:,2);
            Perovskite_Loper.n=interp1(Perovskite_Loper_wl_raw,Perovskite_Loper_n_raw,wavelengths,'spline','extrap');
            Perovskite_Loper.n(Perovskite_Loper.n<0)=0; %set all negative values to zero
            Perovskite_k_raw = Perovskite_Loper_ext(:,3);
            Perovskite_Loper.k=interp1(Perovskite_Loper_wl_raw,Perovskite_k_raw,wavelengths,'spline','extrap');
            Perovskite_Loper.k(Perovskite_Loper.k<0)=0; %set all negative values to zero
            %make sure this is ok:
            Perovskite_Loper.alpha = (4.*pi.*Perovskite_Loper.k)./(wavelengths.*1e-9); 
            Perovskite_Loper.alpha = 1e-9.*Perovskite_Loper.alpha; %getting into units of nm^-1
            Perovskite_Loper.alpha(Perovskite_Loper.alpha<0)=0; %set all negative values to zero
            Perovskite_Loper.eps1 = Perovskite_Loper.n.^2 - Perovskite_Loper.k.^2;
            Perovskite_Loper.eps2 = 2*Perovskite_Loper.n.*Perovskite_Loper.k;
%           Check this!!!
            Perovskite_Loper.eps = complex(Perovskite_Loper.eps1,Perovskite_Loper.eps2);
            Perovskite_Loper.nk = complex(Perovskite_Loper.n,Perovskite_Loper.k);
            assignin('base', 'Perovskite_Loper',Perovskite_Loper);
         end 
         
         function nnl_import_Spiro(wavelengths)      % get the SiON n data from Andy Thomson's experimental values
            Spiro.name = 'Spiro';
            Spiro_ext = importdata('reference_data\full_range_nk_data_Dale\Spiro_nk_Filipic.txt');  %load AM1.5G spectra
            Spiro_wl_raw = Spiro_ext(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            Spiro_n_raw = Spiro_ext(:,2);
            Spiro.n=interp1(Spiro_wl_raw,Spiro_n_raw,wavelengths,'spline','extrap');
            Spiro.n(Spiro.n<0)=0; %set all negative values to zero
            Spiro_k_raw = Spiro_ext(:,3);
            Spiro.k=interp1(Spiro_wl_raw,Spiro_k_raw,wavelengths,'spline','extrap');
            Spiro.k(Spiro.k<0)=0; %set all negative values to zero
            %make sure this is ok:
            Spiro.alpha = (4.*pi.*Spiro.k)./(wavelengths.*1e-9); 
            Spiro.alpha = 1e-9.*Spiro.alpha; %getting into units of nm^-1
            Spiro.alpha(Spiro.alpha<0)=0; %set all negative values to zero
            Spiro.eps1 = Spiro.n.^2 - Spiro.k.^2;
            Spiro.eps2 = 2*Spiro.n.*Spiro.k;
%           Check this!!!
            Spiro.eps = complex(Spiro.eps1,Spiro.eps2);
            Spiro.nk = complex(Spiro.n,Spiro.k);
            assignin('base', 'Spiro',Spiro);
         end 
         
         function nnl_import_TiO2_anatase(wavelengths)      % get the SiON n data from Andy Thomson's experimental values
            TiO2_anatase.name = 'TiO2_anatase';
            TiO2_anatase_ext = importdata('reference_data\full_range_nk_data_Dale\TiO2_anatase.txt');  %load AM1.5G spectra Can Reference Thomson 2015
            TiO2_anatase_wl_raw = TiO2_anatase_ext(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            TiO2_anatase_n_raw = TiO2_anatase_ext(:,2);
            TiO2_anatase.n=interp1(TiO2_anatase_wl_raw,TiO2_anatase_n_raw,wavelengths,'spline','extrap');
            TiO2_anatase.n(TiO2_anatase.n<0)=0; %set all negative values to zero
            TiO2_anatase_k_raw = TiO2_anatase_ext(:,3);
            TiO2_anatase.k=interp1(TiO2_anatase_wl_raw,TiO2_anatase_k_raw,wavelengths,'spline','extrap');
            TiO2_anatase.k(TiO2_anatase.k<0)=0; %set all negative values to zero
            %make sure this is ok:
            TiO2_anatase.alpha = (4.*pi.*TiO2_anatase.k)./(wavelengths.*1e-9); 
            TiO2_anatase.alpha = 1e-9.*TiO2_anatase.alpha; %getting into units of nm^-1
            TiO2_anatase.alpha(TiO2_anatase.alpha<0)=0; %set all negative values to zero
            TiO2_anatase.eps1 = TiO2_anatase.n.^2 - TiO2_anatase.k.^2;
            TiO2_anatase.eps2 = 2*TiO2_anatase.n.*TiO2_anatase.k;
%           Check this!!!
            TiO2_anatase.eps = complex(TiO2_anatase.eps1,TiO2_anatase.eps2);
            TiO2_anatase.nk = complex(TiO2_anatase.n,TiO2_anatase.k);
            assignin('base', 'TiO2_anatase',TiO2_anatase);
         end 
       
         function nnl_import_Au_JC(wavelengths)      % get the SiON n data from Andy Thomson's experimental values
            Au_JC.name = 'Au_JC';
            Au_JC_ext = importdata('reference_data\full_range_nk_data_Dale\Au_JC.txt');  %load AM1.5G spectra
            Au_JC_wl_raw = Au_JC_ext(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            Au_JC_n_raw = Au_JC_ext(:,2);
            Au_JC.n=interp1(Au_JC_wl_raw,Au_JC_n_raw,wavelengths,'spline','extrap');
            Au_JC.n(Au_JC.n<0)=0; %set all negative values to zero
            Au_JC_k_raw = Au_JC_ext(:,3);
            Au_JC.k=interp1(Au_JC_wl_raw,Au_JC_k_raw,wavelengths,'spline','extrap');
            Au_JC.k(Au_JC.k<0)=0; %set all negative values to zero
            %make sure this is ok:
            Au_JC.alpha = (4.*pi.*Au_JC.k)./(wavelengths.*1e-9); 
            Au_JC.alpha = 1e-9.*Au_JC.alpha; %getting into units of nm^-1
            Au_JC.alpha(Au_JC.alpha<0)=0; %set all negative values to zero
            Au_JC.eps1 = Au_JC.n.^2 - Au_JC.k.^2;
            Au_JC.eps2 = 2*Au_JC.n.*Au_JC.k;
%           Check this!!!
            Au_JC.eps = complex(Au_JC.eps1,Au_JC.eps2);
            Au_JC.nk = complex(Au_JC.n,Au_JC.k);
            assignin('base', 'Au_JC',Au_JC);
         end 
         
         function nnl_import_MG(wavelengths)     % load the Si data from Keevers, interpolate and create the Si material structure in the workspace
            MG_ext = importdata('reference_data\Index_of_Refraction_library.xls');  %load AM1.5G spectra
            wl_raw = MG_ext.data(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            
            SiO2_MG_n_raw = MG_ext.data(:,12);
            SiO2_MG.n=interp1(wl_raw,SiO2_MG_n_raw,wavelengths);
            SiO2_MG_k_raw = MG_ext.data(:,13);
            SiO2_MG.k=interp1(wl_raw,SiO2_MG_k_raw,wavelengths);
            SiO2_MG.nk = complex(SiO2_MG.n,SiO2_MG.k);
            SiO2_MG.name = 'SiO2_MG';
            assignin('base', 'SiO2_MG',SiO2_MG);
            
            ITO_MG_n_raw = MG_ext.data(:,20);
            ITO_MG.n=interp1(wl_raw,ITO_MG_n_raw,wavelengths);
            ITO_MG_k_raw = MG_ext.data(:,21);
            ITO_MG.k=interp1(wl_raw,ITO_MG_k_raw,wavelengths);
            ITO_MG.nk = complex(ITO_MG.n,ITO_MG.k);
            ITO_MG.name = 'ITO_MG';
            assignin('base', 'ITO_MG',ITO_MG);
            
            PEDOT_MG_n_raw = MG_ext.data(:,14);
            PEDOT_MG.n=interp1(wl_raw,PEDOT_MG_n_raw,wavelengths);
            PEDOT_MG_k_raw = MG_ext.data(:,15);
            PEDOT_MG.k=interp1(wl_raw,PEDOT_MG_k_raw,wavelengths);
            PEDOT_MG.nk = complex(PEDOT_MG.n,PEDOT_MG.k);
            PEDOT_MG.name = 'PEDOT_MG';
            assignin('base', 'PEDOT_MG',PEDOT_MG);
            
            P3HTPCBM_MG_n_raw = MG_ext.data(:,26);
            P3HTPCBM_MG.n=interp1(wl_raw,P3HTPCBM_MG_n_raw,wavelengths);
            P3HTPCBM_MG_k_raw = MG_ext.data(:,27);
            P3HTPCBM_MG.k=interp1(wl_raw,P3HTPCBM_MG_k_raw,wavelengths);
            P3HTPCBM_MG.nk = complex(P3HTPCBM_MG.n,P3HTPCBM_MG.k);
            P3HTPCBM_MG.name = 'P3HTPCBM_MG';
            assignin('base', 'P3HTPCBM_MG',P3HTPCBM_MG);
            
            Ca_MG_n_raw = MG_ext.data(:,6);
            Ca_MG.n=interp1(wl_raw,Ca_MG_n_raw,wavelengths);
            Ca_MG_k_raw = MG_ext.data(:,7);
            Ca_MG.k=interp1(wl_raw,Ca_MG_k_raw,wavelengths);
            Ca_MG.nk = complex(Ca_MG.n,Ca_MG.k);
            Ca_MG.name = 'Ca_MG';
            assignin('base', 'Ca_MG',Ca_MG);
            
            Al_MG_n_raw = MG_ext.data(:,4);
            Al_MG.n=interp1(wl_raw,Al_MG_n_raw,wavelengths);
            Al_MG_k_raw = MG_ext.data(:,5);
            Al_MG.k=interp1(wl_raw,Al_MG_k_raw,wavelengths);
            Al_MG.nk = complex(Al_MG.n,Al_MG.k);
            Al_MG.name = 'Al_MG';
            assignin('base', 'Al_MG',Al_MG);
            
            
         end
         
          function nnl_import_shockley_queisser_AM0(eV)      %  
            shockley_queisser_AM0.name = 'shockley_queisser_AM0';
            shockley_queisser_AM0_ext = importdata('reference_data\shockley_queisser_limit_AM0_1961.txt');  %load AM1.5G spectra
            shockley_queisser_AM0_eV_raw = shockley_queisser_AM0_ext(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            shockley_queisser_AM0_I_raw = shockley_queisser_AM0_ext(:,2);
            [sqeV_raw,indx]=unique(shockley_queisser_AM0_eV_raw);
            sqI_raw = shockley_queisser_AM0_I_raw(indx);
            shockley_queisser_AM0.I=interp1(sqeV_raw,sqI_raw,eV,'spline','extrap');
            %shockley_queisser_AM0.I(shockley_queisser_AM0.I<0)=0; %set all negative values to zero
            assignin('base', 'shockley_queisser_AM0',shockley_queisser_AM0);
         end 
         
        function sq = nnl_import_shockley_queisser_AM1p5G(eV)      %  
            shockley_queisser_AM1p5G.name = 'shockley_queisser_AM1p5G';
            shockley_queisser_AM1p5G_ext = importdata('reference_data\shockley_queisser_limit_AM1p5G_peter_2011.txt');  %load AM1.5G spectra
            shockley_queisser_AM1p5G_eV_raw = shockley_queisser_AM1p5G_ext(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
            shockley_queisser_AM1p5G_I_raw = shockley_queisser_AM1p5G_ext(:,2);
            [sqeV_raw,indx]=unique(shockley_queisser_AM1p5G_eV_raw);
            sqI_raw = shockley_queisser_AM1p5G_I_raw(indx);
            shockley_queisser_AM1p5G.I=interp1(sqeV_raw,sqI_raw,eV,'spline','extrap');
            %shockley_queisser_AM0.I(shockley_queisser_AM0.I<0)=0; %set all negative values to zero
            sq = shockley_queisser_AM1p5G;
            assignin('base', 'shockley_queisser_AM1p5G',shockley_queisser_AM1p5G);
            
        end 
         
        function nnl_import_PC(PC_parameters)     % load the Si data from Keevers, interpolate and create the Si material structure in the workspace
            PC_ext = importdata(PC_parameters.filename);  %load AM1.5G spectra
            PC.info = PC_ext.textdata;
            PC.rawdata = PC_ext.data; %converting between volts to amps
            PC.data = PC_parameters.VA.*PC_ext.data; %converting between volts to amps
            PC.filename = PC_parameters.filename;
            PC.ppl = PC_parameters.ppl; %points per line
            PC.lpi = PC_parameters.lpi;
            PC.width = PC_parameters.width;
            PC.height = PC_parameters.height;
            PC.pixelarea = (PC.width/PC.ppl)*(PC.height/PC.lpi)*1e-12; %m^2
            PC.xx = [PC.width/PC.ppl:PC.width/PC.ppl:PC.width];
            PC.yy = [PC.height/PC.lpi:PC.height/PC.lpi:PC.height];
            PC.summed_height = sum(PC.data,1);
            PC.vertical_average = PC.summed_height./PC.lpi;
            PC.average = sum(PC.summed_height)./(PC.lpi*PC.ppl);
            PC.avecurrdensity = PC.average./PC.pixelarea; %A/m^2
            
            %             Si_wl_raw = Si_ext.data.Sheet1(:,1);                            % load alpha, absdepth, n, k and Reflection, interpolate and append to Si structure
%             Si_alpha_raw = Si_ext.data.Sheet1(:,2);
%             Si.alpha=(1e-7)*interp1(Si_wl_raw,Si_alpha_raw,wavelengths);    % converting alpha from (/cm) into units of (/nm)
%             Si_absdepth_raw = Si_ext.data.Sheet1(:,3);
%             Si.absdepth=(1e3)*interp1(Si_wl_raw,Si_absdepth_raw,wavelengths);   % converting absdepth from (um) into units of (nm)
%             Si_n_raw = Si_ext.data.Sheet1(:,4);
%             Si.n=interp1(Si_wl_raw,Si_n_raw,wavelengths);
%             Si_k_raw = Si_ext.data.Sheet1(:,5);
%             Si.k=interp1(Si_wl_raw,Si_k_raw,wavelengths);
%             Si_R_raw = Si_ext.data.Sheet1(:,6);
%             Si.R=interp1(Si_wl_raw,Si_R_raw,wavelengths);
%             Si.Eg = 1.12;
%             Si.wlEg = nnl_root.wlev(Si.Eg);
%             Si.wlEg_index = find(wavelengths==floor(Si.wlEg));
%             %Si.alpha(Si.wlEg_index:end)=0;
%             Si.eps1 = Si.n.^2 - Si.k.^2;
%             Si.eps2 = 2*Si.n.*Si.k;
%             Si.eps = complex(Si.eps1,Si.eps2);
%             Si.nk = complex(Si.n,Si.k);
%             Si.Voc=0.706;           % set Voc to the record UNSW PERL cell output under AM15g spectrum
%             %Si.J0=42.7/(exp((1.60e-19*Si.Voc)/((1.3806488e-23)*300)) - 1);   % set J0 to the corresponding value for PERL cell using standard solar cell equation
%             Si.J0=427/(exp((1.60218e-19*Si.Voc)/((1.3806488e-23)*298)) - 1);   % in A/m^2: set J0 to the corresponding value for PERL cell using standard solar cell equation
%             Si.FF = 0.828;          % set FF to the record UNSW PERL cell output under AM15g spectrum
%             %Si.collection_probability = 0.958; % Following TW modelling parameter to calculate the correct Jsc is obtained for the UNSW PERL record cell
%             Si.collection_probability = 0.97955; % new? Following TW modelling parameter to calculate the correct Jsc is obtained for the UNSW PERL record cell
%             Si.name = 'silicon';
             assignin('base', PC_parameters.desc, PC);
             ;
        end

        
        % Set constants in workspace
        function setconstants           % set global constants
            constants.cc = 3e8;
            constants.ee = 1.61e-19;
            canberra.insolation.jan =6.83;
            canberra.insolation.feb =6.14;
            canberra.insolation.mar =4.99;
            canberra.insolation.apr =3.63;
            canberra.insolation.may =2.66;
            canberra.insolation.jun =2.17;
            canberra.insolation.jul =2.34;
            canberra.insolation.aug =3.06;
            canberra.insolation.sep =4.18;
            canberra.insolation.oct =5.40;
            canberra.insolation.nov =6.36;
            canberra.insolation.dec =6.85;
            canberra.insolation.average = 4.76;
            assignin('base', 'constants',constants);
            assignin('base', 'canberra',canberra);
        end
        


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%The Transfer Matrix function%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       function TM_complete = transfermatrixcomplete(input, wavelength)
            %TM.TM = zeros(2,2);
            TM.TM = [1,0;0,1];
            TE.TM = [1,0;0,1];

            wavelengthindex = find(input.spectra.wl==floor(wavelength));
            kk0 = 2*pi./(wavelength./1e-9);
            input.angle = nnl_root.rad(input.angledeg); % angle in radians)

            for ii = 1:1:input.numberoflayers;
                layerTM(ii).CM = zeros(2,2);
                layerTE(ii).CM = zeros(2,2);
                if (ii==1);
                presentangle = asin( (input.superstrate.nk(wavelengthindex)/input.layer(ii).nk(wavelengthindex)).*sin(input.angle));    
                elseif (ii>1);
                presentangle = asin( (input.layer(ii-1).nk(wavelengthindex)/input.layer(ii).nk(wavelengthindex)).*sin(presentangle));    
                end
                outputangle = asin( (input.superstrate.nk(wavelengthindex)/input.substrate.nk(wavelengthindex)).*sin(input.angle));
                
               
                    layerTM(ii).P1 = ((1.26e-6)/input.layer(ii).nk(wavelengthindex)).*cos(presentangle);
                    PsupTM = ((1.26e-6)/input.superstrate.nk(wavelengthindex)).*cos(input.angle);
                    PsubTM = ((1.26e-6)/input.substrate.nk(wavelengthindex)).*cos(outputangle);
                    
                    layerTE(ii).P1 = (input.layer(ii).nk(wavelengthindex)/1.26e-6).*cos(presentangle);
                    PsupTE = (input.superstrate.nk(wavelengthindex)/1.26e-6).*cos(input.angle);
                    PsubTE = (input.substrate.nk(wavelengthindex)/1.26e-6).*cos(outputangle);
                
                    
                kkh = kk0*input.layer(ii).nk(wavelengthindex).*(input.thickness(ii)/1e-9).*cos(presentangle);
                layerTM(ii).CM = [cos(kkh) ,  (-1i/layerTM(ii).P1)*sin(kkh); (-1i.*layerTM(ii).P1)*sin(kkh), cos(kkh)];
                layerTE(ii).CM = [cos(kkh) ,  (-1i/layerTE(ii).P1)*sin(kkh); (-1i.*layerTE(ii).P1)*sin(kkh), cos(kkh)];
                
                TM.TM = TM.TM*layerTM(ii).CM;
                TE.TM = TE.TM*layerTE(ii).CM;
            end
            
            TM.rr = ((TM.TM(1,1) + TM.TM(1,2).*PsubTM).*PsupTM - (TM.TM(2,1) + TM.TM(2,2).*PsubTM))./((TM.TM(1,1) + TM.TM(1,2).*PsubTM).*PsupTM + (TM.TM(2,1) + TM.TM(2,2).*PsubTM));
            TM.tt = (2*PsupTM)./((TM.TM(1,1) + TM.TM(1,2).*PsubTM).*PsupTM + (TM.TM(2,1) + TM.TM(2,2).*PsubTM));
            TM.RR = real(TM.rr*conj(TM.rr));
            TM.TT = real((PsubTM./PsupTM)).*real(TM.tt*conj(TM.tt));
            TM.AA = 1-TM.RR - TM.TT;

            TE.rr = ((TE.TM(1,1) + TE.TM(1,2).*PsubTE).*PsupTE - (TE.TM(2,1) + TE.TM(2,2).*PsubTE))./((TE.TM(1,1) + TE.TM(1,2).*PsubTE).*PsupTE + (TE.TM(2,1) + TE.TM(2,2).*PsubTE));
            TE.tt = (2*PsupTE)./((TE.TM(1,1) + TE.TM(1,2).*PsubTE).*PsupTE + (TE.TM(2,1) + TE.TM(2,2).*PsubTE));
            TE.RR = real(TE.rr*conj(TE.rr));
            TE.TT = real((PsubTE./PsupTE)).*real(TE.tt*conj(TE.tt));
            TE.AA = 1-TE.RR - TE.TT;
            
            TM_complete.TM = TM;
            TM_complete.TE = TE;
            TM_complete.mixed.rr = (TM.rr + TE.rr)./2;
            TM_complete.mixed.RR = (TM.RR + TE.RR)./2;
            TM_complete.mixed.tt = (TM.tt + TE.tt)./2;
            TM_complete.mixed.TT = (TM.TT + TE.TT)./2;
            TM_complete.mixed.AA = (TM.AA + TE.AA)./2;
            
 
            %  Error checking to ensure R+T+A = 1  (or between 1.01 and 0.99)%%%%%%%%%%%
            if max(TM_complete.TM.RR+TM_complete.TM.AA+TM_complete.TM.TT)>1.01;
                fprintf('errorTMmax\n');
            end
            if max(TM_complete.TE.RR+TM_complete.TE.AA+TM_complete.TE.TT)>1.01;
                fprintf('errorTEmax\n');
            end
            if max(TM_complete.TM.RR+TM_complete.TM.AA+TM_complete.TM.TT)<0.99;
                fprintf('errorTMmin\n');
            end
            if max(TM_complete.TE.RR+TM_complete.TE.AA+TM_complete.TE.TT)<0.99;
                fprintf('errorTEmin\n');
            end
            
       end
       

       
        function TM_cell = transfermatrixcell(input, wavelength)
           % This function uses the Ghent/McGehee/Petterson formalism (see  J. Appl Phys 86, 487 (1999)
           
           wavelengthindex = find(input.spectra.wl==floor(wavelength));
           kk0 = 2*pi./(wavelength./1e-9);
           I0 = (1/2)*2.998e8*8.85e-12; % I = 1/2 c eps0
           input.angle = nnl_root.rad(input.angledeg); % angle in radians)
           input.layer(input.numberoflayers+1).nk = input.substrate.nk; %including the substrate as the last layer for the I_mat calculation
           thicknesscumulativesum = cumsum(input.thickness);
           thicknesscumulativesum0 = zeros(1,length(input.thickness) + 1);
           thicknesscumulativesum0(1) =0;
           thicknesscumulativesum0(2:end) = thicknesscumulativesum;
           
           
           %--- First finding the reflection from the air/superstrate interface (assuming a thick superstrate and incoherent reflection/transmission)
           presentangle = asin((1/input.superstrate.nk(wavelengthindex)).*sin(input.angle));  
           
           rr_sup.TE = (1*cos(input.angle) - input.superstrate.nk(wavelengthindex)*cos(presentangle)) / (1*cos(input.angle) + input.superstrate.nk(wavelengthindex)*cos(presentangle));
           RR_sup.TE = real(rr_sup.TE*conj(rr_sup.TE));
           rr_sup.TM = (input.superstrate.nk(wavelengthindex)*cos(input.angle) - 1*cos(presentangle)) / (input.superstrate.nk(wavelengthindex)*cos(input.angle) + 1*cos(presentangle));
           RR_sup.TM = real(rr_sup.TM*conj(rr_sup.TM));
           
           tt_sup.TE = (2*cos(input.angle))/(1*cos(input.angle) + input.superstrate.nk(wavelengthindex)*cos(presentangle));
           TT_sup.TE = (real(input.superstrate.nk(wavelengthindex))/real(1))*(cos(presentangle)/cos(input.angle))*real(tt_sup.TE*conj(tt_sup.TE));
           tt_sup.TM = (2*cos(input.angle))/(input.superstrate.nk(wavelengthindex)*cos(input.angle) + 1*cos(presentangle));
           TT_sup.TM = (real(input.superstrate.nk(wavelengthindex))/real(1))*(cos(presentangle)/cos(input.angle))*real(tt_sup.TM*conj(tt_sup.TM));
           
           %--- Set up parameters
           x_pos=input.xgrain/2:input.xgrain:sum(input.thickness); %positions to evaluate field
           %x_mat specifies what layer number the corresponding point in x_pos is in:
           %x_mat= sum(repmat(x_pos,length(t),1)>repmat(t_cumsum',1,length(x_pos)),1)+1; 
           E.TE=zeros(1,length(x_pos));
           E.TM=zeros(1,length(x_pos));
           E2.TE=zeros(1,length(x_pos));
           E2.TM=zeros(1,length(x_pos));
           Pnorm.TE=zeros(1,length(x_pos));
           Pnorm.TM=zeros(1,length(x_pos));
           Gen.TE=zeros(1,length(x_pos));
           Gen.TM=zeros(1,length(x_pos));
           AA.TE = zeros(input.numberoflayers+1,1);
           AA.TM = zeros(input.numberoflayers+1,1);
           
           
           % Calculate the total transfer matrix using incoherent reflection and transmission at the first interface
           S.TE=nnl_root.I_mat_TE(input,input.superstrate,input.layer(1), wavelength);
           S.TM=nnl_root.I_mat_TM(input,input.superstrate,input.layer(1), wavelength);

            for ii=1:input.numberoflayers
                S.TE=S.TE*nnl_root.L_mat(input,input.layer(ii),input.thickness(ii),wavelength)*nnl_root.I_mat_TE(input,input.layer(ii),input.layer(ii+1),wavelength);
                S.TM=S.TM*nnl_root.L_mat(input,input.layer(ii),input.thickness(ii),wavelength)*nnl_root.I_mat_TM(input,input.layer(ii),input.layer(ii+1),wavelength);
            end
            
            % Calculate the reflection from the layer stack (ie. at the superstrate-layer1 interface).
            RR1.TE=abs(S.TE(2,1)/S.TE(1,1))^2; %JAP Vol 86 p.487 Eq 9 Power Reflection from layers other than superstrate
            RR1.TM=abs(S.TM(2,1)/S.TM(1,1))^2; %JAP Vol 86 p.487 Eq 9 Power Reflection from layers other than superstrate
            
            % I think this is what the Transmission into the layers should
            % be... But the McGehee version below gives the right result...?
            %TT1.TE = TT_sup.TE*(1-RR1.TE)/(1-RR1.TE*RR_sup.TE); % This is the light going into layer 1, considering transmission through the superstrate, and multiple reflections with the first layer and the superstrate.
            %TT1.TM = TT_sup.TM*(1-RR1.TM)/(1-RR1.TM*RR_sup.TM); % This is the light going into layer 1, considering transmission through the superstrate, and multiple reflections with the first layer and the superstrate.
           
            %DIFFERENT - CHECK WHY%
            %McGehee version of TT1
            TT1.TE = abs(tt_sup.TE)/sqrt((1-RR1.TE*RR_sup.TE)); % This is the light going into layer 1, considering transmission through the superstrate, and multiple reflections with the first layer and the superstrate.
            TT1.TM = abs(tt_sup.TM)/sqrt((1-RR1.TM*RR_sup.TM)); % This is the light going into layer 1, considering transmission through the superstrate, and multiple reflections with the first layer and the superstrate.
            
            
            
            lastangle = asin((1/input.substrate.nk(wavelengthindex)).*sin(input.angle));  
            TT_stack.TE=(real(input.substrate.nk(wavelengthindex))/real(1))*(cos(lastangle)/cos(input.angle))*abs(1/S.TE(1,1))^2; % This is |t|^2 for the stack
            TT_stack.TM=(real(input.substrate.nk(wavelengthindex))/real(1))*(cos(lastangle)/cos(input.angle))*abs(1/S.TM(1,1))^2; % This is |t|^2 for the stack
            
            % Calculate individual layer transfer matrices to find the fields in each layer
                        
            for ii = 1:input.numberoflayers
                presentangle = asin((1/input.layer(ii).nk(wavelengthindex)).*sin(input.angle));
                current_thickness=input.thickness(ii);
                xi=2*pi*input.layer(ii).nk(wavelengthindex)*cos(presentangle)/wavelength;
                %x_indices=find(x_mat == ii); %indices of points which are in the material layer considered
                if ii==1;
                    dx = x_pos;
                else
                    dx=x_pos-thicknesscumulativesum(ii-1); %distance from interface with previous layer
                end
                %- Calculate S matrices (JAP Vol 86 p.487 Eq 12 and 13)
                %S_prime.TE=nnl_root.I_mat_TE(input,input.layer(1),input.layer(2),wavelength);
                %S_prime.TM=nnl_root.I_mat_TM(input,input.layer(1),input.layer(2),wavelength);
                %Check below - first S' from superstrate?
                S_prime.TE=nnl_root.I_mat_TE(input,input.superstrate,input.layer(1),wavelength);
                S_prime.TM=nnl_root.I_mat_TM(input,input.superstrate,input.layer(1),wavelength);
                
                for jj=2:ii
                    S_prime.TE=S_prime.TE*nnl_root.L_mat(input,input.layer(jj-1),input.thickness(jj-1),wavelength)*nnl_root.I_mat_TE(input,input.layer(jj-1), input.layer(jj), wavelength);
                    S_prime.TM=S_prime.TM*nnl_root.L_mat(input,input.layer(jj-1),input.thickness(jj-1),wavelength)*nnl_root.I_mat_TM(input,input.layer(jj-1), input.layer(jj), wavelength);
                end
                
                S_doubleprime.TE=[1,0;0,1];
                S_doubleprime.TM=[1,0;0,1];
                
                for jj=ii:input.numberoflayers-1
                    S_doubleprime.TE=S_doubleprime.TE*nnl_root.I_mat_TE(input,input.layer(jj),input.layer(jj+1),wavelength)*nnl_root.L_mat(input,input.layer(jj+1),input.thickness(jj+1),wavelength);
                    S_doubleprime.TM=S_doubleprime.TM*nnl_root.I_mat_TM(input,input.layer(jj),input.layer(jj+1),wavelength)*nnl_root.L_mat(input,input.layer(jj+1),input.thickness(jj+1),wavelength);
                end
                
                %DIFFERENT from MG%
                % These two lines deviate from McGehee - he doesn't include them... It gives the same final result - and I think it's correct to include them - it makes the final S matrices match up.
                S_doubleprime.TE=S_doubleprime.TE*nnl_root.I_mat_TE(input,input.layer(input.numberoflayers),input.layer(input.numberoflayers+1),wavelength);
                S_doubleprime.TM=S_doubleprime.TM*nnl_root.I_mat_TM(input,input.layer(input.numberoflayers),input.layer(input.numberoflayers+1),wavelength);
                
                
                % Checking to see if R and T are still ok:
                
                Lj = nnl_root.L_mat(input,input.layer(ii),input.thickness(ii),wavelength);
                S_temp.TE = S_prime.TE*Lj*S_doubleprime.TE;
                S_temp.TM = S_prime.TM*Lj*S_doubleprime.TM;
                
                RR_test.TE=abs(S_temp.TE(2,1)/S_temp.TE(1,1))^2; %JAP Vol 86 p.487 Eq 9 Power Reflection from layers other than substrate
                RR_test.TM=abs(S_temp.TM(2,1)/S_temp.TM(1,1))^2;
                TT_test.TE=(real(input.substrate.nk(wavelengthindex))/real(1))*(cos(lastangle)/cos(input.angle))*abs(1/S_temp.TE(1,1))^2; % This is |t|^2 for the stack
                TT_test.TM=(real(input.substrate.nk(wavelengthindex))/real(1))*(cos(lastangle)/cos(input.angle))*abs(1/S_temp.TM(1,1))^2; % This is |t|^2 for the stack
                
                %%%%
                
                xlayer_min = find(x_pos >= thicknesscumulativesum0(ii), 1 );
                %xlayer_max = find(x_pos >= thicknesscumulativesum0(ii+1), 1 );
                [~, xlayer_max] = min(abs(x_pos-thicknesscumulativesum0(ii+1)));
                
                %                dx = x_pos - thicknesscumulativesum(ii-1);
                sortvector = zeros(1,length(x_pos));
                
                for kk=xlayer_min:xlayer_max
                    
                    %Normalized Field profile (JAP Vol 86 p.487 Eq 22)
                    E.TE(kk)=TT1.TE*(S_doubleprime.TE(1,1)*exp(-1i*xi*(current_thickness-dx(kk)))+S_doubleprime.TE(2,1)*exp(1i*xi*(current_thickness-dx(kk))))...
                        ./(S_prime.TE(1,1)*S_doubleprime.TE(1,1)*exp(-1i*xi*current_thickness)+S_prime.TE(1,2)*S_doubleprime.TE(2,1)*exp(1i*xi*current_thickness)) ;
                    % from MG: E(x_indices,l)=T(l)* (S_doubleprime(1,1)*exp(-1i*xi*(dj-x))+S_doubleprime(2,1)*exp(1i*xi*(dj-x))) ./(S_prime(1,1)*S_doubleprime(1,1)*exp(-1i*xi*dj)+S_prime(1,2)*S_doubleprime(2,1)*exp(1i*xi*dj));
                    
                    % Below: With changed sign in front of S''_21 - Why? Cause this is the only way I can get E.TM and E.TE to equal...
                    %E.TM(kk)=TT1.TM.*(S_doubleprime.TM(1,1).*exp(-1i.*xi.*(current_thickness-dx(kk)))-S_doubleprime.TM(2,1).*exp(1i.*xi.*(current_thickness-dx(kk))) ./(S_prime.TM(1,1).*S_doubleprime.TM(1,1).*exp(-1i.*xi.*current_thickness)+S_prime.TM(1,2).*S_doubleprime.TM(2,1).*exp(1i.*xi.*current_thickness))) ;
                    %E.TM(kk)=TT1.TM*(S_doubleprime.TM(1,1)*exp(-1i*xi*(current_thickness-dx(kk)))-S_doubleprime.TM(2,1)*exp(1i*xi*(current_thickness-dx(kk))))...
                    % ./(S_prime.TM(1,1)*S_doubleprime.TM(1,1)*exp(-1i*xi*current_thickness)-S_prime.TM(1,2)*S_doubleprime.TM(2,1)*exp(1i*xi*current_thickness)) ;
                    
                    % With unchanged sign in front of S''_21 - This gives
                    % different E,A etc for TM polarisation - even at
                    % normal incidence! It must be wrong...
                    E.TM(kk)=TT1.TM*(S_doubleprime.TM(1,1)*exp(-1i*xi*(current_thickness-dx(kk)))+S_doubleprime.TM(2,1)*exp(1i*xi*(current_thickness-dx(kk))))...
                        ./(S_prime.TM(1,1)*S_doubleprime.TM(1,1)*exp(-1i*xi*current_thickness)+S_prime.TM(1,2)*S_doubleprime.TM(2,1)*exp(1i*xi*current_thickness)) ;
                    
                    
                    E2.TE(kk)=abs(E.TE(kk)).^2;
                    E2.TM(kk)=abs(E.TM(kk)).^2;
                    %                     E2.TE(kk)=real((E.TE(kk)).*conj(E.TE(kk)));
                    %                     E2.TM(kk)=real((E.TM(kk)).*conj(E.TM(kk)));
                    Pnorm.TE(kk) = (4*pi*imag(input.layer(ii).nk(wavelengthindex))./(wavelength*1e-9)).*real(input.layer(ii).nk(wavelengthindex)).*E2.TE(kk); % P = 2*w*eps0*Im(epsj)*E2*delta_x, normalised = P/I0, we calculate this in Si
                    Pnorm.TM(kk) = (4*pi*imag(input.layer(ii).nk(wavelengthindex))./(wavelength*1e-9)).*real(input.layer(ii).nk(wavelengthindex)).*E2.TM(kk); % P = 2*w*eps0*Im(epsj)*E2*delta_x, normalised = P/I0, we calculate this in Si
                    %                     Pnorm.TE(kk) = 16.*pi.*imag(input.layer(ii).nk(wavelengthindex)).*real(input.layer(ii).nk(wavelengthindex)).*E2.TE(kk).*input.xgrain.*1e-9./(wavelength*1e-9); % P = 2*w*eps0*Im(epsj)*E2*delta_x, normalised = P/I0, we calculate this in Si
                    %                     Pnorm.TM(kk) = 16.*pi.*imag(input.layer(ii).nk(wavelengthindex)).*real(input.layer(ii).nk(wavelengthindex)).*E2.TM(kk).*input.xgrain.*1e-9./(wavelength*1e-9); % P = 2*w*eps0*Im(epsj)*E2*delta_x, normalised = P/I0
                    if ii ==input.activelayer; % calculate charge-carrier generation if in the active layer
                        Gen.TE(kk) = input.sunspectra(wavelengthindex).*Pnorm.TE(kk)*wavelength*1e-9./(6.63e-34*2.998e8);
                        Gen.TE(kk) = input.sunspectra(wavelengthindex).*Pnorm.TE(kk)*wavelength*1e-9./(6.63e-34*2.998e8);
                    end
                    
                    sortvector(kk) = 1;
                end
                AA.TE(ii,1) = sum(Pnorm.TE.*sortvector).*input.xgrain*1e-9;
                AA.TM(ii,1) = sum(Pnorm.TM.*sortvector).*input.xgrain*1e-9;
                
            end
        
            AA.TE(input.numberoflayers+1,1)= sum(AA.TE);
            AA.TM(input.numberoflayers+1,1)= sum(AA.TM);
               
            
            
            % Overall Reflection from device with incoherent reflections at first
            % interface (typically air-glass)
            Reflection.TE=RR_sup.TE+TT_sup.TE.^2.*RR1.TE/(1-RR_sup.TE.*RR1.TE);
            Reflection.TM=RR_sup.TM+TT_sup.TM.^2.*RR1.TM/(1-RR_sup.TM.*RR1.TM);
            Transmission.TE = TT_sup.TE.*TT_stack.TE./(1-RR1.TE.*RR_sup.TE); %including multiple reflections from the superstrate/layer interface
            Transmission.TM = TT_sup.TM.*TT_stack.TM./(1-RR1.TM.*RR_sup.TM); %including multiple reflections from the superstrate/layer interface
            
            E2.TE = real(E.TE.*conj(E.TE));
            E2.TM = real(E.TM.*conj(E.TM));
            
            TM_cell.RR.TE = Reflection.TE;
            TM_cell.RR.TM = Reflection.TM;
            TM_cell.RR.mixed = (TM_cell.RR.TE + TM_cell.RR.TM)./2;

            TM_cell.TT.TE = Transmission.TE;
            TM_cell.TT.TM = Transmission.TM;
            TM_cell.TT.mixed = (TM_cell.TT.TE + TM_cell.TT.TM)./2;

            TM_cell.AAlayers.TE = AA.TE;
            TM_cell.AAlayers.TM = AA.TM;
            TM_cell.AAlayers.mixed = (TM_cell.AAlayers.TE + TM_cell.AAlayers.TM)./2;

                        
            TM_cell.AA.TE = AA.TE(input.numberoflayers+1,1);
            TM_cell.AA.TM = AA.TM(input.numberoflayers+1,1);
            TM_cell.AA.mixed = (TM_cell.AA.TE + TM_cell.AA.TM)./2;

            
            TM_cell.AAfractions.TE = TM_cell.AAlayers.TE;
            TM_cell.AAfractions.TM = TM_cell.AAlayers.TM;
            TM_cell.AAfractions.mixed = (TM_cell.AAfractions.TE + TM_cell.AAfractions.TM)./2;

            
            for ii=1:input.numberoflayers + 1
                TM_cell.AAfractions.TE(ii,1) = TM_cell.AAlayers.TE(ii,1)./TM_cell.AA.TE;
                TM_cell.AAfractions.TM(ii,1) = TM_cell.AAlayers.TM(ii,1)./TM_cell.AA.TM;
            end 
            
            
            
            TM_cell.E2.TE = transpose(E2.TE);
            TM_cell.E2.TM = transpose(E2.TM);
            TM_cell.E2.mixed = (TM_cell.E2.TE + TM_cell.E2.TM)./2;

            TM_cell.Gen.TE = transpose(Gen.TE);
            TM_cell.Gen.TM = transpose(Gen.TM);
            TM_cell.Gen.mixed = (TM_cell.Gen.TE + TM_cell.Gen.TM)./2;
            %TM_cell.Jsc.TE = 1.6e-19*1e-9*sum(TMcurrent.sunphotons.*TM_celloutput.TE.AAlayers(TMcurrent.activelayer,:)); % NOT correct yet...
            %TM_cell.Jsc.TM = 1.6e-19*sum(TMcurrent.sunphotons.*TM_celloutput.TM.AAlayers(TMcurrent.activelayer,:));

            
%             Q=repmat(a(activeLayer,:).*real(n(activeLayer,:)).*AM15,length(ActivePos),1).*(abs(E(ActivePos,:)).^2);
% 
%             % Exciton generation rate per second-cm3-nm at each position and wavelength
%             Gxl=(Q*1e-3).*repmat(lambda*1e-9,length(ActivePos),1)/(h*c);

           
            
       end
           
       function I = I_mat_TE(input,layer1,layer2,wavelength) %-- Function for returning the first 'interface' Transfer Matrix
        wavelengthindex = find(input.spectra.wl==floor(wavelength));
        thetai = asin((1)/layer1.nk(wavelengthindex).*sin(input.angle));
        thetaj = asin((1)/layer2.nk(wavelengthindex).*sin(input.angle));
        
        rr=(layer1.nk(wavelengthindex)*cos(thetai) - layer2.nk(wavelengthindex)*cos(thetaj)) / (layer1.nk(wavelengthindex)*cos(thetai) + layer2.nk(wavelengthindex)*cos(thetaj));
        tt=(2*layer1.nk(wavelengthindex)*cos(thetai)) / (layer1.nk(wavelengthindex)*cos(thetai) + layer2.nk(wavelengthindex)*cos(thetaj));
        I=[1 rr; rr 1]./tt;
       end
       
       function I = I_mat_TM(input,layer1,layer2,wavelength) %-- Function for returning the first 'interface' Transfer Matrix
        wavelengthindex = find(input.spectra.wl==floor(wavelength));
        thetai = asin((1)/layer1.nk(wavelengthindex).*sin(input.angle));
        thetaj = asin((1)/layer2.nk(wavelengthindex).*sin(input.angle));
        
        rr=(layer2.nk(wavelengthindex)*cos(thetai) - layer1.nk(wavelengthindex)*cos(thetaj)) / (layer2.nk(wavelengthindex)*cos(thetai) + layer1.nk(wavelengthindex)*cos(thetaj));
        tt=(2*layer1.nk(wavelengthindex)*cos(thetai)) / (layer2.nk(wavelengthindex)*cos(thetai) + layer1.nk(wavelengthindex)*cos(thetaj));
        I=[1 rr; rr 1]/tt;
       end
       
        % Function L_mat
        % This function calculates the propagation matrix, L, through a material of
        % complex dielectric constant n and thickness d for the wavelength lambda.
        function L = L_mat(input,layer,thickness,wavelength) %-- Function for returning the propagation Transfer Matrix
        wavelengthindex = find(input.spectra.wl==floor(wavelength));
        thetai = asin((1)/layer.nk(wavelengthindex).*sin(input.angle));
        phase = 2*pi*layer.nk(wavelengthindex)*thickness*cos(thetai)/wavelength;
        
        L=[exp(-1i*phase) 0; 0 exp(1i*phase)];
        end
       
       
       
      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% Additional Transfer Matrix functions%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
       function TMinputlongpassBragg = Create_LongPass_Bragg(TMinput,wl);
       
           TMinputlongpassBragg=TMinput;
           TMinputlongpassBragg.numberoflayers= 3 + TMinput.numHLblocks*2;  %(1/2H L (HL)*n 1/2H)
           lambda0 = TMinput.lambda0;
           qlambda0 = lambda0/4;
           wlEg_index = find(wl==floor(lambda0));
           refn_H = TMinputlongpassBragg.materialH.n(wlEg_index);
           refn_L = TMinputlongpassBragg.materialL.n(wlEg_index);
           
            TMinputlongpassBragg.layer(1).nk = TMinputlongpassBragg.materialH.nk;  %first layer 0.5H
            TMinputlongpassBragg.thickness(1) = 0.5*qlambda0/(refn_H); %in nm
            TMinputlongpassBragg.layer(2).nk = TMinputlongpassBragg.materialL.nk;  %second layer L
            TMinputlongpassBragg.thickness(2) = qlambda0/(refn_L); %in nm
            TMinputlongpassBragg.layer(TMinputlongpassBragg.numberoflayers).nk = TMinputlongpassBragg.materialH.nk;  %last layer 0.5H
            TMinputlongpassBragg.thickness(TMinputlongpassBragg.numberoflayers) = 0.5*qlambda0/(refn_H); %in nm
            
            for ii=2:1:floor(TMinputlongpassBragg.numberoflayers/2)
                TMinputlongpassBragg.layer(2*ii).nk = TMinputlongpassBragg.materialL.nk;  %first layer 0.5H
                TMinputlongpassBragg.thickness(2*ii) = qlambda0/(refn_L); %in nm
            end
            for jj=1:1:floor(TMinputlongpassBragg.numberoflayers/2-1)
                TMinputlongpassBragg.layer(2*jj+1).nk = TMinputlongpassBragg.materialH.nk;  %first layer 0.5H
                TMinputlongpassBragg.thickness(2*jj+1) = qlambda0/(refn_H); %in nm
            end
            
            
       end
        
       function TMinputlongpassBragg2 = Create_LongPass_Bragg2(TMinput,wl);
       
           TMinputlongpassBragg2=TMinput;
           TMinputlongpassBragg2.numberoflayers1= 3 + TMinput.numHLblocks1*2;  %(1/2H L (HL)*n 1/2H + 1/2H L (HL)*n 1/2H)
           TMinputlongpassBragg2.numberoflayers_total= 6 + TMinput.numHLblocks1*2 + TMinput.numHLblocks2*2;  %(1/2H L (HL)*n 1/2H + 1/2H L (HL)*n 1/2H)
           TMinputlongpassBragg2.numberoflayers = TMinputlongpassBragg2.numberoflayers_total;
           
           lambda01 = TMinput.lambda01;
           qlambda01 = lambda01/4;
           wlEg_index = find(wl==floor(lambda01));
           refn_H = TMinputlongpassBragg2.materialH.n(wlEg_index);
           refn_L = TMinputlongpassBragg2.materialL.n(wlEg_index);
           
           lambda02 = TMinput.lambda02;
           qlambda02 = lambda02/4;
           wlEg_index2 = find(wl==floor(lambda02));
           refn_H2 = TMinputlongpassBragg2.materialH.n(wlEg_index2);
           refn_L2 = TMinputlongpassBragg2.materialL.n(wlEg_index2);
           
           
            % First block
            TMinputlongpassBragg2.layer(1).nk = TMinputlongpassBragg2.materialH.nk;  %first layer 0.5H
            TMinputlongpassBragg2.thickness(1) = 0.5*qlambda01/(refn_H); %in nm
            TMinputlongpassBragg2.layer(2).nk = TMinputlongpassBragg2.materialL.nk;  %second layer L
            TMinputlongpassBragg2.thickness(2) = qlambda01/(refn_L); %in nm
            TMinputlongpassBragg2.layer(TMinputlongpassBragg2.numberoflayers1).nk = TMinputlongpassBragg2.materialH.nk;  %last layer 0.5H
            TMinputlongpassBragg2.thickness(TMinputlongpassBragg2.numberoflayers1) = 0.5*qlambda01/(refn_H); %in nm
            
            for ii=2:1:floor(TMinputlongpassBragg2.numberoflayers1/2)
                TMinputlongpassBragg2.layer(2*ii).nk = TMinputlongpassBragg2.materialL.nk;  
                TMinputlongpassBragg2.thickness(2*ii) = qlambda01/(refn_L); %in nm
            end
            for jj=1:1:floor(TMinputlongpassBragg2.numberoflayers1/2-1)
                TMinputlongpassBragg2.layer(2*jj+1).nk = TMinputlongpassBragg2.materialH.nk;  
                TMinputlongpassBragg2.thickness(2*jj+1) = qlambda01/(refn_H); %in nm
            end
            
            % Second block
            TMinputlongpassBragg2.layer(TMinputlongpassBragg2.numberoflayers1 + 1).nk = TMinputlongpassBragg2.materialH.nk;  %first layer 0.5H
            TMinputlongpassBragg2.thickness(TMinputlongpassBragg2.numberoflayers1 + 1) = 0.5*qlambda02/(refn_H2); %in nm
            TMinputlongpassBragg2.layer(TMinputlongpassBragg2.numberoflayers1 + 2).nk = TMinputlongpassBragg2.materialL.nk; 
            TMinputlongpassBragg2.thickness(TMinputlongpassBragg2.numberoflayers1 + 2) = qlambda02/(refn_L2); %in nm
            TMinputlongpassBragg2.layer(TMinputlongpassBragg2.numberoflayers_total).nk = TMinputlongpassBragg2.materialH.nk;  %last layer 0.5H
            TMinputlongpassBragg2.thickness(TMinputlongpassBragg2.numberoflayers_total) = 0.5*qlambda02/(refn_H2); %in nm
            
            for kk=(TMinputlongpassBragg2.numberoflayers1+3)/2:1:floor(TMinputlongpassBragg2.numberoflayers_total/2 - 1)
                TMinputlongpassBragg2.layer(2*kk).nk = TMinputlongpassBragg2.materialH.nk;  
                TMinputlongpassBragg2.thickness(2*kk) = qlambda02/(refn_H2); %in nm
            end
            for mm=(TMinputlongpassBragg2.numberoflayers1+3)/2:1:floor(TMinputlongpassBragg2.numberoflayers_total/2-1)
                TMinputlongpassBragg2.layer(2*mm+1).nk = TMinputlongpassBragg2.materialL.nk;  
                TMinputlongpassBragg2.thickness(2*mm+1) = qlambda02/(refn_L2); %in nm
            end
           
            
            
       end
      
        function TMinputshortpassBragg = Create_ShortPass_Bragg(TMinput,wl);
       
           TMinputshortpassBragg=TMinput;
           TMinputshortpassBragg.numberoflayers= 3 + TMinput.numHLblocks*2;  %(1/2H L (HL)*n 1/2H)
           lambda0 = TMinput.lambda0;
           qlambda0 = lambda0/4;
           wlEg_index = find(wl==floor(lambda0));
           refn_H = TMinputshortpassBragg.materialH.n(wlEg_index);
           refn_L = TMinputshortpassBragg.materialL.n(wlEg_index);
           
            TMinputshortpassBragg.layer(1).nk = TMinputshortpassBragg.materialL.nk;  %first layer 0.5H
            TMinputshortpassBragg.thickness(1) = 0.5*qlambda0/(refn_L); %in nm
            TMinputshortpassBragg.layer(2).nk = TMinputshortpassBragg.materialH.nk;  %second layer L
            TMinputshortpassBragg.thickness(2) = qlambda0/(refn_H); %in nm
            TMinputshortpassBragg.layer(TMinputshortpassBragg.numberoflayers).nk = TMinputshortpassBragg.materialL.nk;  %last layer 0.5H
            TMinputshortpassBragg.thickness(TMinputshortpassBragg.numberoflayers) = 0.5*qlambda0/(refn_L); %in nm
            
            for ii=2:1:floor(TMinputshortpassBragg.numberoflayers/2)
                TMinputshortpassBragg.layer(2*ii).nk = TMinputshortpassBragg.materialH.nk;  %first layer 0.5H
                TMinputshortpassBragg.thickness(2*ii) = qlambda0/(refn_H); %in nm
            end
            for jj=1:1:floor(TMinputshortpassBragg.numberoflayers/2-1)
                TMinputshortpassBragg.layer(2*jj+1).nk = TMinputshortpassBragg.materialL.nk;  %first layer 0.5H
                TMinputshortpassBragg.thickness(2*jj+1) = qlambda0/(refn_L); %in nm
            end
            
            
       end
      
       function TMinputshortpassBragg2 = Create_ShortPass_Bragg2(TMinput,wl);
       
           TMinputshortpassBragg2=TMinput;
           TMinputshortpassBragg2.numberoflayers1= 3 + TMinput.numHLblocks1*2;  %(1/2H L (HL)*n 1/2H + 1/2H L (HL)*n 1/2H)
           TMinputshortpassBragg2.numberoflayers_total= 6 + TMinput.numHLblocks1*2 + TMinput.numHLblocks2*2;  %(1/2H L (HL)*n 1/2H + 1/2H L (HL)*n 1/2H)
           TMinputshortpassBragg2.numberoflayers = TMinputshortpassBragg2.numberoflayers_total;
           
           lambda01 = TMinput.lambda01;
           qlambda01 = lambda01/4;
           wlEg_index = find(wl==floor(lambda01));
           refn_H = TMinputshortpassBragg2.materialH.n(wlEg_index);
           refn_L = TMinputshortpassBragg2.materialL.n(wlEg_index);
           
           lambda02 = TMinput.lambda02;
           qlambda02 = lambda02/4;
           wlEg_index2 = find(wl==floor(lambda02));
           refn_H2 = TMinputshortpassBragg2.materialH.n(wlEg_index2);
           refn_L2 = TMinputshortpassBragg2.materialL.n(wlEg_index2);
           
           
            % First block
            TMinputshortpassBragg2.layer(1).nk = TMinputshortpassBragg2.materialL.nk;  %SWITCHING for short-pass first layer 0.5H
            TMinputshortpassBragg2.thickness(1) = 0.5*qlambda01/(refn_L); %in nm
            TMinputshortpassBragg2.layer(2).nk = TMinputshortpassBragg2.materialH.nk;  %second layer L
            TMinputshortpassBragg2.thickness(2) = qlambda01/(refn_H); %in nm
            TMinputshortpassBragg2.layer(TMinputshortpassBragg2.numberoflayers1).nk = TMinputshortpassBragg2.materialL.nk;  %last layer 0.5H
            TMinputshortpassBragg2.thickness(TMinputshortpassBragg2.numberoflayers1) = 0.5*qlambda01/(refn_L); %in nm
            
            for ii=2:1:floor(TMinputshortpassBragg2.numberoflayers1/2)
                TMinputshortpassBragg2.layer(2*ii).nk = TMinputshortpassBragg2.materialH.nk;  
                TMinputshortpassBragg2.thickness(2*ii) = qlambda01/(refn_H); %in nm
            end
            for jj=1:1:floor(TMinputshortpassBragg2.numberoflayers1/2-1)
                TMinputshortpassBragg2.layer(2*jj+1).nk = TMinputshortpassBragg2.materialL.nk;  
                TMinputshortpassBragg2.thickness(2*jj+1) = qlambda01/(refn_L); %in nm
            end
            
            % Second block
            TMinputshortpassBragg2.layer(TMinputshortpassBragg2.numberoflayers1 + 1).nk = TMinputshortpassBragg2.materialL.nk;  %first layer 0.5H
            TMinputshortpassBragg2.thickness(TMinputshortpassBragg2.numberoflayers1 + 1) = 0.5*qlambda02/(refn_L2); %in nm
            TMinputshortpassBragg2.layer(TMinputshortpassBragg2.numberoflayers1 + 2).nk = TMinputshortpassBragg2.materialH.nk; 
            TMinputshortpassBragg2.thickness(TMinputshortpassBragg2.numberoflayers1 + 2) = qlambda02/(refn_H2); %in nm
            TMinputshortpassBragg2.layer(TMinputshortpassBragg2.numberoflayers_total).nk = TMinputshortpassBragg2.materialL.nk;  %last layer 0.5H
            TMinputshortpassBragg2.thickness(TMinputshortpassBragg2.numberoflayers_total) = 0.5*qlambda02/(refn_L2); %in nm
            
            for kk=(TMinputshortpassBragg2.numberoflayers1+3)/2:1:floor(TMinputshortpassBragg2.numberoflayers_total/2 - 1)
                TMinputshortpassBragg2.layer(2*kk).nk = TMinputshortpassBragg2.materialL.nk;  
                TMinputshortpassBragg2.thickness(2*kk) = qlambda02/(refn_L2); %in nm
            end
            for mm=(TMinputshortpassBragg2.numberoflayers1+3)/2:1:floor(TMinputshortpassBragg2.numberoflayers_total/2-1)
                TMinputshortpassBragg2.layer(2*mm+1).nk = TMinputshortpassBragg2.materialH.nk;  
                TMinputshortpassBragg2.thickness(2*mm+1) = qlambda02/(refn_H2); %in nm
            end
           
            
            
       end
          
       
       function scan_output = TMthicknessJscscan(TMinput,scan_input);
           numsteps = (scan_input.stop-scan_input.start)/scan_input.step + 1;
           scan_output.thickness = zeros(1,numsteps);
           scan_output.Jsc = zeros(1,numsteps);
           
           TMcurrent = TMinput;
           wl=TMcurrent.light.wl; % a hangover from a previous version of the code...
           tic
           for jj=1:1:numsteps
           TMcurrent.thickness(scan_input.layer) = scan_input.start +(jj-1)*scan_input.step;
           
           %%%%%%%%%%%%% TM calculation %%%%%%%%%%%%%%%%
                TMcurrent.sunphotons = TMcurrent.light.sunshine.n_of_photons;
                TMcurrent.sunspectra = TMcurrent.light.sunshine.spectra; %AM1.5G
                TMcurrent.xgrain = 1; % in nm
                TMcurrent.xpos=TMcurrent.xgrain/2:TMcurrent.xgrain:sum(TMcurrent.thickness);
                TMcurrent.fieldwls = [400 600 800];

                %TM_celloutput.TM.RR = zeros(1,length(wl));
                %TM_celloutput.TE.RR = zeros(1,length(wl));
                %TM_celloutput.mixed.RR = zeros(1,length(wl));
                %TM_celloutput.TM.TT = zeros(1,length(wl));
                %TM_celloutput.TE.TT = zeros(1,length(wl));
                %TM_celloutput.mixed.TT = zeros(1,length(wl));
                %TM_celloutput.TM.AA = zeros(1,length(wl));
          %  TM_celloutput.TE.AA = zeros(1,length(wl));  
                %TM_celloutput.mixed.AA = zeros(1,length(wl));
                %TM_celloutput.TM.E2 = zeros(length(TMcurrent.xpos),length(wl));
          %  TM_celloutput.TE.E2 = zeros(length(TMcurrent.xpos),length(wl));
                %TM_celloutput.mixed.E2 = zeros(length(TMcurrent.xpos),length(wl));
                %TM_celloutput.TM.Gen = zeros(length(TMcurrent.xpos),length(wl));
                TM_celloutput.TE.Gen = zeros(length(TMcurrent.xpos),length(wl));
                %TM_celloutput.mixed.Gen = zeros(length(TMcurrent.xpos),length(wl));
                %TM_celloutput.TM.AAlayers = zeros(TMcurrent.numberoflayers+1,length(wl));
          %  TM_celloutput.TE.AAlayers = zeros(TMcurrent.numberoflayers+1,length(wl));
                %TM_celloutput.mixed.AAlayers = zeros(TMcurrent.numberoflayers+1,length(wl));
                %TM_celloutput.TM.AAfractions = zeros(TMcurrent.numberoflayers+1,length(wl));
          %  TM_celloutput.TE.AAfractions = zeros(TMcurrent.numberoflayers+1,length(wl));
                %TM_celloutput.mixed.AAfractions = zeros(TMcurrent.numberoflayers+1,length(wl));
          %  TM_celloutput.TE.AAtotalfractions = zeros(TMcurrent.numberoflayers+1,1);
                %TM_celloutput.TM.AAtotalfractions = zeros(1,TMcurrent.numberoflayers+1);


                    for ii= 1:1:length(TMcurrent.light.wl)
                        wavelength = TMcurrent.light.wl(ii);
                        currentTM = nnl_root.transfermatrixcell(TMcurrent,wavelength);    
                     %   TM_celloutput.TM.RR(ii)=currentTM.RR.TM;
         %          TM_celloutput.TE.RR(ii)=currentTM.RR.TE;
                     %   TM_celloutput.mixed.RR(ii)=currentTM.RR.mixed;
                     %   TM_celloutput.TM.TT(ii)=currentTM.TT.TM;
         %          TM_celloutput.TE.TT(ii)=currentTM.TT.TE;
                     %   TM_celloutput.mixed.TT(ii)=currentTM.TT.mixed;
                     %   TM_celloutput.TM.E2(:,ii)=currentTM.E2.TM(:,1);
         %          TM_celloutput.TE.E2(:,ii)=currentTM.E2.TE(:,1);
                     %   TM_celloutput.mixed.E2(:,ii)=currentTM.E2.mixed(:,1);
                     %   TM_celloutput.TM.Gen(:,ii)=currentTM.Gen.TM(:,1);
                        TM_celloutput.TE.Gen(:,ii)=currentTM.Gen.TE(:,1);
                     %   TM_celloutput.mixed.Gen(:,ii)=currentTM.Gen.mixed(:,1);
                     %   TM_celloutput.TM.AA(ii)=currentTM.AA.TM;
         %          TM_celloutput.TE.AA(ii)=currentTM.AA.TE;
                     %   TM_celloutput.mixed.AA(ii)=currentTM.AA.mixed;
                     %   TM_celloutput.TM.AAlayers(:,ii)=currentTM.AAlayers.TM;
         %          TM_celloutput.TE.AAlayers(:,ii)=currentTM.AAlayers.TE;
                     %   TM_celloutput.mixed.AAlayers(:,ii)=currentTM.AAlayers.mixed;
                     %   TM_celloutput.TM.AAfractions(:,ii)=currentTM.AAfractions.TM;
         %          TM_celloutput.TE.AAfractions(:,ii)=currentTM.AAfractions.TE;
                     %   TM_celloutput.mixed.AAfractions(:,ii)=currentTM.AAfractions.mixed;


                    %      mixed.RR(ii) = currentTM.mixed.RR;
                    %      mixed.TT(ii) = currentTM.mixed.TT;
                    %      mixed.AA(ii) = currentTM.mixed.AA;

                    end

                %
%                 TM_cellabsnorm = zeros(1,length(light.wl));
%                 TM_cellabsnorm(1,:) = 1;
% 
%                 for ii=1:TMcurrent.numberoflayers + 1
%                             TM_celloutput.TE.AAtotalfractions(ii,1) = trapz(light.wl,TM_celloutput.TE.AAfractions(ii,:))/trapz(light.wl,TM_cellabsnorm);
%                             %TM_cell.AAtotalfractions.TM(1,ii) = trapz(light.wl,TM_cell.AAfractions.TM(ii,1))/trapz(light.wl,TM_cellabsnorm);
%                 end


                TM_celloutput.TE.activeGen = sum(TM_celloutput.TE.Gen,2).*(wl(2)-wl(1)); %charge-carriers per m^2, per m, per s.
                %TM_celloutput.TM.activeGen = sum(TM_celloutput.TM.Gen,2).*(wl(2)-wl(1)); %charge-carriers per m^2, per m, per s.
                %TM_celloutput.mixed.activeGen = (TM_celloutput.TE.activeGen + TM_celloutput.TM.activeGen)./2; %charge-carriers per m^2, per m, per s.
                TM_celloutput.TE.Jsc = 1.6e-19*1e-9*sum(TM_celloutput.TE.activeGen)/10; % in mA/cm^2;
                %TM_celloutput.TM.Jsc = 1.6e-19*1e-9*sum(TM_celloutput.TM.activeGen)/10; % in mA/cm^2;
                %TM_celloutput.mixed.Jsc = (TM_celloutput.TE.Jsc + TM_celloutput.TM.Jsc)./2; % in mA/cm^2;
                %TM_celloutput.mixed.Jsc = (light.wl(2) - light.wl(1))*(1/10)*1.6e-19*sum(TM_celloutput.mixed.AAlayers(TMcurrent.activelayer,:).*TMcurrent.sunphotons); % in mA/cm^2;

           
           
           
           scan_output.thickness(jj) = TMcurrent.thickness(scan_input.layer);
           scan_output.Jsc(jj) = TM_celloutput.TE.Jsc;
           
           end
           
           
           
          toc 
       end
       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%General functions%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        % Convert degrees to radians
        function radians = rad(theta)   % convert degrees to radians
            radians = theta*pi/180;
        end
        
        % Convert radians to degrees
        function degrees = deg(theta)   % convert radians to degrees
            degrees = theta*180/pi;
        end
        
        % Convert wl to ev
        function wlev = wlev(input)     % convert wl to ev and vice versa
            wlev = 1240./input;
        end
       
        function freqwl = freqwl(input) % convert wl to ev and vice versa
            freqwl = (1./input).*(3e17);
        end
        % Convert a photon's wavelength in nm to energy in joules
        function energy_in_joules = energy_of_photon(wavelength_nm)
            energy_in_joules = (6.63e-34)*(2.998e8)./((1e-9)*wavelength_nm);
        end
        
        function makegraphnice()
            %newfigure1=figure('Position', [400, 400, 550, 500]);
            %nnl_root.SetPlotSize ([3.5 1.5 12, 8], 'centimeters');
            %nnl_root.SetPlotSize ([1.6 2 8, 8], 'centimeters');
            nnl_root.SetPlotSize ([2 2 9, 9], 'centimeters');
            nnl_root.SetPlotFont ('Helvetica', 10);
            % Set the plot colours (darker colours)
            nnl_root.SetPlotColors;
            %newfigure1=figure('Position', [400, 400, 550, 500]);
            set(gca,'Fontsize',10);
            set(gcf,'Color',[1 1 1]);
           % set(gcf,'Position', [400, 400, 550, 550]);
            %set(gcf,'Position', [550, 550, 550, 500]);
            title('','Fontsize',10);
            %legend1 = legend(axes,'show');
            %set(legend1,'Visible','off','Units','centimeters');
        end
        
        function SetPlotSize (pos, units, varargin)
            % SetPlotSize Set the size of the current plot
            % SetPlotSize(pos,units,figurecolor)
            % SetPlotSize(pos,units,options)
            % SetPlotSize(pos) % default units "inches" assumed
            %
            % pos - axes position [xsize, ysize] or
            % [xleft, ybottom, xsize, ysize]
            % units - units for the dimensions, default 'inches'. The 
            % choices are 'centimeters', 'pixels', 'inches',
            % 'points', and 'normalized'.
            % figurecolor - Figure and axes background color.
            % Use 'none' to get transparent axes.
            % options - Paired options, e.g. to set the figure color to
            % transparent, and the axes color to white, use
            % 'Color', 'none', 'DefaultAxesColor', 'w'
            %
            % Note: The 'Units' for the current axes are set to the units
            % specified. If the units are absolute measurements
            % (i.e. not 'normalized'), then figure can be resized on
            % the screen using the mouse. This feature can be used to
            % "crop" white space above and to the left of the axes.
            % $Id: SetPlotSize.m,v 1.17 2006/01/27 22:32:38 pkabal Exp $
            % Notes:
            % - The default axes position for Matlab is given in normalized
            % units as [0.130 0.110 0.775 0.815]. These values are used
            % to determine the space around the axes.
            if (nargin <= 1)
                units = 'inches';
            end
            Np = length(pos);
            if (Np ~= 2 && Np ~= 4)
                error ('SetPlotSize: pos must have 2 or 4 elements');
            end
            posnorm = [0.130 0.110 0.775 0.815];
            if (Np == 2)
                figsize = pos ./ posnorm(3:4);
                axespos = [(posnorm(1:2) .* figsize) pos];
            else
                figsize = (pos(1:2) + pos(3:4)) ./ (posnorm(1:2) + posnorm(3:4));
                axespos = pos;
            end
            % Set the figure
            saveunits = get (gcf, 'Units');
            set (gcf, 'Units', units);
            figpos = get (gcf, 'Position');
            set (gcf, 'Position', [figpos(1:2) figsize]);
            set (gcf, 'Units', saveunits);
            % Set the axes
            set (gca, 'Units', units);
            set (gca, 'Position', axespos);
            Nv = length (varargin);
            if (Nv == 1)
                set (gcf, 'Color', varargin{1}, 'DefaultAxesColor', varargin{1});
            elseif (Nv > 1)
                set (gcf, varargin{:});
            end
            %disp (sprintf ('SetPlotSize: Figure size: %g x %g', figsize));
            %fsprintf ('SetPlotSize: Figure size: %g x %g', figsize);
            return
        end
        
        function SetPlotFont (FontName, FontSize)
            % SetPlotFont - Set font name and size for plots
            % SetPlotFont(FontName,FontSize)
            % SetPlotFont(FontName) % Default 10 pt font size
            % SetPlotFont % Default 10 pt Times
            % $Id: SetPlotFont.m,v 1.3 2006/06/05 19:01:37 pkabal Exp $
            if (nargin < 2)
                FontSize = 18;
            end
            if (nargin < 1)
                FontName = 'Times';
            end
            % For figure children, not yet created
            set (gcf, ...
            'DefaultTextFontName', FontName, 'DefaultTextFontSize', FontSize, ...
            'DefaultAxesFontName', FontName, 'DefaultAxesFontSize', FontSize, ...
            'DefaultLineMarkerSize', 0.5*FontSize);
            % For axes already set up in the figure
            set (gca, ...
            'FontName', FontName, 'FontSize', FontSize);
            return
        end
        
        function CO = SetPlotColors (Colors)
            % CO = SetPlotColors (Colors)
            % This routine sets the default colors for plot lines. The
            % default settings override the lighter standard Matlab colors
            % These darker colors are rendered better on a black and white
            % printer.
            %
            % The input cell array can be used to override the default
            % colors set by this routine. Any empty cells in the input leave
            % the corresponding colors unchanged.
            % SetPlotColors({[]; [0 0.2 0]}); % Darker green
            %
            % N.B. Call this routine before plotting
            % $Id: SetPlotColors.m,v 1.3 2006/01/27 22:36:51 pkabal Exp $
            % Original colors
            Blue = [ 0 0 1 ];
            Green = [ 0 0.5 0 ]; % Mid-green
            Red = [ 1 0 0 ]; Cyan = [ 0 0.75 0.75 ]; Magenta = [ 0.75 0 0.75 ]; Yellow = [ 0.75 0.75 0 ]; Gray = [ 0.25 0.25 0.25 ];
            % Darker colors
            DarkCyan = [ 0 0.6 0.6 ]; DarkMagenta = [ 0.5 0 0.5 ]; DarkYellow = [ 0.6 0.6 0 ];
            if (nargin < 1)
            Colors = {[]; []; []; []; []; [];};
            end
            CO = get (gcf, 'DefaultAxesColorOrder');
            % Matlab cycles through the first six of these colors
            CO(4:6,:) = [DarkCyan; DarkMagenta; DarkYellow];
            % Insert the user defined colors
            NC = length (Colors);
            for (i = 1:NC)
            if (~ isempty (Colors{i}))
            CO(i,:) = Colors{i};
            end
            end
            % Set the color order
            set (gcf, 'DefaultAxesColorOrder', CO);
            return
        end
       
        function comsol_scan_test = comsol_scan_test(parameter)
            comsol_scan_test.R = zeros(1,10);
            comsol_scan_test.R(:) = parameter;
            comsol_scan_test.T = zeros(1,10);
            comsol_scan_test.T(:) = -parameter;
            return 
        end
     
        function greentest(top_cell,sub_cell,light)
            top_cell.total_absorption = light.sunshine.spectra.*(1 - top_cell.Rext).*(1 - top_cell.Rb.*top_cell.Tp.*top_cell.Tm)./(1-top_cell.Rf.*top_cell.Rb.*top_cell.Tp.*top_cell.Tm);
            top_cell.reflector_abs = light.sunshine.spectra.*(1 - top_cell.Rext).*(1 - top_cell.Rb).*top_cell.Tp./(1-top_cell.Rf.*top_cell.Rb.*top_cell.Tp.*top_cell.Tm);
            top_cell.absorption = top_cell.total_absorption - top_cell.reflector_abs;
            top_cell.bot_light_trapping_loss = top_cell.reflector_abs;
            
            
            
            sub_cell.inputspectra = (top_cell.light_to_cell - top_cell.absorption).*(1-top_cell.intermediate_reflector).*(1 - top_cell.bot_light_trapping_loss);  % multiplicative light_loss for spectrum incident on substrate cell
            sub_cell.inputenergy = trapz(light.wl,sub_cell.inputspectra);
            %sub_cell.inputspectra = top_cell.inputspectra-top_cell.absorption - top_cell.light_trapping_loss;  % spectrum incident on substrate cell
            sub_cell.optical_thickness = (sub_cell.top_light_trapping + sub_cell.bot_light_trapping)*sub_cell.thickness;    % optical thickness = real thickness x path length enhancement
            sub_cell.absorption = nnl_root.beer_lambert_absorption(sub_cell.inputspectra,sub_cell.material,sub_cell.optical_thickness);    % calculate the sub_cell absorption with the optical thickness

        end
        
        function makee2graphnice()
        %reshape the z, contour against xlim,ylim
        
        nnl_root.makegraphnice();
        colormap gray;
        caxis([ -3 3 ]);
        set(gca,'YTick',[]);
        set(gca,'XTick',[]);
        end
        
        function swappedwave = swapwave(wave)
            swappedwave = zeros(length(wave),1);
            for ii=1:1:length(wave)                
                 swappedwave(ii,1) = wave(length(wave)+1 - ii);
            end
        end
        
        function switchfuncout = switchfunc(matrixinput,rangea,rangeb)
            if max(matrixinput)>rangeb;
                switchfuncout='errormax';
            end
            if max(matrixinput)<rangea;
                switchfuncout='errormin';
            end
                        
        end
        
        function carnoteff = carnot(temphigh,templow)
           carnoteff = 1 - (templow + 273.15)./(temphigh + 273.15);
        end
        
        function matrixrunout = matrixrun(scatteringtransformation)
            
            arta_r = 95;
            arta_h=17;
            
            for ii=1:1:length(BB04.wl)
                BB04.scatteringtransformation(ii,:)=BB04.anglerange;
            end

            %scatteringtransformationrad = rad(scatteringtransformation);
            %scatteringtransformationsinrad = sin(scatteringtransformationrad);
            %scatteringtransformation_complete = scatteringtransformationsinrad.*(2*pi*arta_r/arta_h);
            BB04.scatteringtransformation_complete = sin(rad(scatteringtransformation)).*(2*pi*arta_r/arta_h);

            BB04.mixed_transformed = BB04.mixed.*BB04.scatteringtransformation_complete;
            BB04.mixed_transformed_cumulativesum =cumsum(BB04.mixed_transformed,2);
            BB04.transformed_diffuse = BB04.mixed_transformed_cumulativesum(:,17);
            figure;
            plot(BB04.wl, BB04.transformed_diffuse);
            
            
            
        end
        
        function jscintegrate = jscintegrate(Eg1,Eg2,light) %Eg1 low, Eg2 high
            wl1index = find(light.wl==floor(nnl_root.wlev(Eg1)));
            wl2index = find(light.wl==floor(nnl_root.wlev(Eg2)));
            filterwave = light.wl;
            filterwave(1,:) = 1;
            filterwave(1,1:wl2index)= 0;
            filterwave(1,wl1index:end)= 0;
    
            jscintegrate=1.6e-19*trapz(light.wl,filterwave.*light.sunshine.n_of_photons)./10;
        end
        
        function plasmonshift = plasmonshift(wl1,n1,n2)
                w1 = 2*pi*3e8/(wl1*10^-9);
                eps1 = n1^2;
                eps2 = n2^2;
                wshift = w1.*((1+eps1)/(1+eps2))^(1/2);
                plasmonshift = 1e9*2*pi*3e8/(wshift);
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Tandem cell calculations%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function total_cell = calculate_cell(top_cell,sub_cell,light)
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % Light through the system
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            dimwl = length(light.wl);
            top_cell.intermediate_reflector = zeros(1,dimwl);
            top_cell.intermediate_reflector_index = find(light.wl==top_cell.bot_intermediate_wl);
            top_cell.intermediate_reflector(1:top_cell.intermediate_reflector_index) = top_cell.bot_intermediate_strength;
            top_cell.intermediate_reflector(top_cell.intermediate_reflector_index+1:end) = 0;
            top_cell.absorption = zeros(1,dimwl);
            top_cell.absorptivity = zeros(1,dimwl);
            top_cell.light_to_cell = (top_cell.inputspectra).*(1-top_cell.top_light_trapping_loss);  % light to cell gets diminished by light_trapping loss on the top of the cell
            top_cell.optical_thickness = (top_cell.top_light_trapping + top_cell.bot_light_trapping).*(1+top_cell.intermediate_reflector).*top_cell.thickness;    % optical thickness = real thickness x path length enhancement
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %Normal Beer-Lambert absorption
%            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 top_cell.absorption = nnl_root.beer_lambert_absorption(top_cell.light_to_cell,top_cell.material,top_cell.optical_thickness);  % calculate the top_cell absorption with the optical thickness
%                 top_cell.parasitic_light_trapping = top_cell.inputspectra.*(top_cell.top_light_trapping_loss + top_cell.bot_light_trapping_loss);
%                 top_cell.absorptivity = nnl_root.beer_lambert_absorptivity(top_cell.material,top_cell.optical_thickness);
%                 sub_cell.inputspectra = (top_cell.light_to_cell - top_cell.absorption).*(1-top_cell.intermediate_reflector).*(1 - top_cell.bot_light_trapping_loss);  % multiplicative light_loss for spectrum incident on substrate cell
%             
                
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%Absorption with Green's analytical solutions including transmission Rb,Ab,Tb
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            top_cell.absorptivity = (1 - top_cell.Rext).*(1 - top_cell.Tp + top_cell.Tp.*top_cell.Rb.*(1-top_cell.Tm))./(1-top_cell.Rf.*top_cell.Rb.*top_cell.Tp.*top_cell.Tm);
            top_cell.reflectivity = top_cell.Rext + (1-top_cell.Rext).*top_cell.Tp.*top_cell.Tm.*top_cell.Rb.*(1-top_cell.Rf)./(1-top_cell.Rf.*top_cell.Rb.*top_cell.Tp.*top_cell.Tm);
            top_cell.transmissivity = (1-top_cell.Rext).*top_cell.Tp.*top_cell.Tb./(1-top_cell.Rf.*top_cell.Rb.*top_cell.Tp.*top_cell.Tm);
            top_cell.reflector_absorptivity = (1-top_cell.Rext).*top_cell.Tp.*top_cell.Ab./(1-top_cell.Rf.*top_cell.Rb.*top_cell.Tp.*top_cell.Tm);
            
            top_cell.absorption = top_cell.inputspectra.*top_cell.absorptivity;
            top_cell.reflector_abs = top_cell.inputspectra.*top_cell.reflector_absorptivity;
            top_cell.total_absorption = top_cell.absorption + top_cell.reflector_abs;

            top_cell.bot_light_trapping_loss = top_cell.reflector_absorptivity;
            top_cell.parasitic_light_trapping = top_cell.reflector_abs;
            
           
            %sub_cell.inputspectra = top_cell.inputspectra.*top_cell.transmissivity; % if we're using a vertical stack
            sub_cell.inputspectra = top_cell.incident_light.*sub_cell.filter; % if we're using a reflector filter (defined in cell_script)          




            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %Selective light trapping with Beer-Lambert absorption
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%             wlEgindex = find(light.wl==floor(top_cell.material.wlEg));    
%             beer_lambert_lighttrapping = nnl_root.beer_lambert_absorption(top_cell.light_to_cell,top_cell.material,top_cell.optical_thickness);  % calculate the top_cell absorption with the optical thickness
%             beer_lambert_singlepass = nnl_root.beer_lambert_absorption(top_cell.light_to_cell,top_cell.material,top_cell.thickness);  % calculate the top_cell absorption with the optical thickness
%             
%             top_cell.absorption(1,1:wlEgindex) = beer_lambert_lighttrapping(1,1:wlEgindex);
%             top_cell.absorption(1,wlEgindex:end) = beer_lambert_singlepass(1,wlEgindex:end);
%             top_cell.parasitic_light_trapping = top_cell.inputspectra.*(top_cell.top_light_trapping_loss + top_cell.bot_light_trapping_loss);
%             absorptivity_lighttrapping = nnl_root.beer_lambert_absorptivity(top_cell.material,top_cell.optical_thickness);
%             absorptivity_singlepass = nnl_root.beer_lambert_absorptivity(top_cell.material,top_cell.thickness);
%             top_cell.absorptivity(1,1:wlEgindex)= absorptivity_lighttrapping(1,1:wlEgindex);
%             top_cell.absorptivity(1,wlEgindex:end)= absorptivity_singlepass(1,wlEgindex:end);
%             sub_cell.inputspectra = (top_cell.light_to_cell - top_cell.absorption).*(1-top_cell.intermediate_reflector).*(1 - top_cell.bot_light_trapping_loss);  % multiplicative light_loss for spectrum incident on substrate cell
%             
%             %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % Black body absorption
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
%             top_cell.xg = (top_cell.material.Eg.*1.6e-19)./(1.38e-23.*5762);  %xg = Eg.q/kT_sun
%             top_cell.absorption = 1.6e-19.*1.6e-5.*(2.*pi./(((6.623e-34)^3)*((3e8)^2))).*((1.38e-23*5762)^3).*(exp(-top_cell.xg)).*(2 + 2.*top_cell.xg + top_cell.xg.^2);
%             top_cell.absorptivity = nnl_root.beer_lambert_absorptivity(top_cell.material,top_cell.optical_thickness); %dummy var here
%             
                %sub_cell.inputspectra = (top_cell.light_to_cell - top_cell.absorption).*(1-top_cell.intermediate_reflector).*(1 - top_cell.bot_light_trapping_loss);  % multiplicative light_loss for spectrum incident on substrate cell
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%            Plasmon absorption
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
           
%            top_cell.plasmon_absorption = top_cell.plasmon_metal_loss;
%            top_cell.absorptivity = top_cell.top_light_trapping_abs;
%            top_cell.absorption = top_cell.inputspectra.*top_cell.top_light_trapping_abs;
%            %top_cell.bot_light_trapping_loss = top_cell.inputspectra.*top_cell.plasmon_absorption;
%            top_cell.parasitic_light_trapping = top_cell.inputspectra.*top_cell.plasmon_absorption;
            
            %sub_cell.inputspectra = (top_cell.light_to_cell - top_cell.absorption).*(1-top_cell.intermediate_reflector).*(1 - top_cell.bot_light_trapping_loss);  % multiplicative light_loss for spectrum incident on substrate cell
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%Absorption with Green's analytical solutions
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%             top_cell.absorptivity = (1 - top_cell.Rext).*(1 - top_cell.Rb.*top_cell.Tp.*top_cell.Tm)./(1-top_cell.Rf.*top_cell.Rb.*top_cell.Tp.*top_cell.Tm) - (1 - top_cell.Rext).*(1 - top_cell.Rb).*top_cell.Tp./(1-top_cell.Rf.*top_cell.Rb.*top_cell.Tp.*top_cell.Tm);
%             top_cell.total_absorption = light.sunshine.spectra.*(1 - top_cell.Rext).*(1 - top_cell.Rb.*top_cell.Tp.*top_cell.Tm)./(1-top_cell.Rf.*top_cell.Rb.*top_cell.Tp.*top_cell.Tm);
%             top_cell.reflector_abs = light.sunshine.spectra.*(1 - top_cell.Rext).*(1 - top_cell.Rb).*top_cell.Tp./(1-top_cell.Rf.*top_cell.Rb.*top_cell.Tp.*top_cell.Tm);
%             top_cell.absorption = top_cell.total_absorption - top_cell.reflector_abs;
%             %For absorbing reflector layer
%              top_cell.bot_light_trapping_loss = top_cell.reflector_abs./light.sunshine.spectra;
%             % % %  %top_cell.parasitic_light_trapping = top_cell.bot_light_trapping_loss;
%              top_cell.parasitic_light_trapping = top_cell.reflector_abs;
% %             For non absorbing reflector - ie, A_reflector is transmitted to sub cell
%            %top_cell.bot_light_trapping_loss = 0;
%            %top_cell.parasitic_light_trapping = top_cell.bot_light_trapping_loss;
            
            %sub_cell.inputspectra = (top_cell.light_to_cell - top_cell.absorption).*(1-top_cell.intermediate_reflector).*(1 - top_cell.bot_light_trapping_loss);  % multiplicative light_loss for spectrum incident on substrate cell
            
       
           

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% Sub cell characteristics
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %sub_cell.inputspectra = (top_cell.light_to_cell - top_cell.absorption).*(1-top_cell.intermediate_reflector).*(1 - top_cell.bot_light_trapping_loss);  % multiplicative light_loss for spectrum incident on substrate cell
            sub_cell.inputenergy = trapz(light.wl,sub_cell.inputspectra);
            %sub_cell.inputspectra = top_cell.inputspectra-top_cell.absorption - top_cell.light_trapping_loss;  % spectrum incident on substrate cell
            sub_cell.optical_thickness = (sub_cell.top_light_trapping + sub_cell.bot_light_trapping)*sub_cell.thickness;    % optical thickness = real thickness x path length enhancement
            %sub_cell.absorption = nnl_root.beer_lambert_absorption(sub_cell.inputspectra,sub_cell.material,sub_cell.optical_thickness);    % calculate the sub_cell absorption with the optical thickness
            sub_cell.absorption = nnl_root.green_Si_absorption(sub_cell.inputspectra,sub_cell.material,sub_cell.thickness);    % calculate the sub_cell absorption with the optical thickness
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % Electrical calculations
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            top_cell.collection_probability = nnl_root.collection_probability(top_cell.thickness,top_cell.material);              % calculate the collection probability with given Ld and thickness
            sub_cell.collection_probability = nnl_root.collection_probability(sub_cell.thickness,sub_cell.material);

            
            % Jsc, Voc and efficiency of the individual cells
            if top_cell.thickness == 0
                top_cell.Jsc = 0;
                top_cell.Voc = 0;
            else
                top_cell.Jsc = nnl_root.Jsc(top_cell,light.wl);
                top_cell.Voc = nnl_root.Voc(top_cell, light);
                top_cell.J0 = nnl_root.J0(top_cell,light);
                top_cell.I0 = top_cell.J0./1.6e-19;
            end

            if sub_cell.thickness == 0
                sub_cell.Jsc = 0;
                sub_cell.Voc = 0;
            else
                sub_cell.Jsc = nnl_root.Jsc(sub_cell,light.wl);
                sub_cell.Voc = nnl_root.Voc(sub_cell, light);   
            end
            
            top_cell.FF = top_cell.material.FF;
            sub_cell.FF = sub_cell.material.FF;
            
            top_cell.power = top_cell.Jsc*top_cell.Voc*top_cell.material.FF;
            sub_cell.power = sub_cell.Jsc*sub_cell.Voc*sub_cell.material.FF;
            top_cell.efficiency = top_cell.Jsc*top_cell.Voc*top_cell.material.FF/light.sunshine.energy;
            sub_cell.efficiency = sub_cell.Jsc*sub_cell.Voc*sub_cell.material.FF/light.sunshine.energy;
            sub_cell.internal_efficiency = sub_cell.Jsc*sub_cell.Voc*sub_cell.material.FF/sub_cell.inputenergy;
            
            % Total stucture
            total_cell.top_cell = top_cell;
            total_cell.sub_cell = sub_cell;
            total_cell.absorption = top_cell.absorption + sub_cell.absorption;              % calculate total cell absorption
            total_cell.Jsc = top_cell.Jsc + sub_cell.Jsc;                                   % calculate total cell Jsc
            total_cell.power = top_cell.power + sub_cell.power;                             % calculate total cell power
            total_cell.efficiency = total_cell.power/light.sunshine.energy;                 % calculate total cell efficiency
            total_cell.transmission = (sub_cell.inputspectra - sub_cell.absorption).*(1 - sub_cell.bot_light_trapping_loss);         % calculate light transmitted through the structure
            %total_cell.transmission = top_cell.inputspectra - top_cell.absorption - top_cell.light_trapping_loss - sub_cell.absorption - sub_cell.light_trapping_loss;         % calculate light transmitted through the structure

        end
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Solar cell calculations%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        % Calculate absorption via the Beer-Lambert Law
        function absorption = beer_lambert_absorption(spectra, material, thickness)
            absorption = spectra.*(1-exp(-material.alpha.*thickness));
        end
        
        function absorptivity = beer_lambert_absorptivity(material, thickness)
            absorptivity = (1-exp(-material.alpha.*thickness));
        end
        
        function green_Si_absorption = green_Si_absorption(spectra, material, thickness)
            aa = 0.935;
            bb = 0.67;
            xx = aa.*(material.alpha.*thickness).^bb;
            Wop = thickness.*(2+xx)./(1 + xx);
            absorption_parameter = (1 - exp(-2.*material.alpha.*Wop))./(1 - (1-1./(material.n.^2)).*exp(-2.*material.alpha.*Wop));
            green_Si_absorption = spectra.*absorption_parameter;
        end
        
        % Calculate the collection probability as a function of thickness
        % material-dependent function
        function collection_probability = collection_probability(thickness,material)
            if strcmp(material.name,'antimony_trisulfide')==1;
                %collection_probability = 1;
                %collection_probability =(material.Ld/thickness)*tanh(thickness/material.Ld);  %using the hyperbolic tan method
                lambda_2 = material.Vbi/(2*(1.38e-23)*300/1.6e-19) - sqrt( (thickness/material.Ld)^2 + (material.Vbi/(2*(1.3806488e-23)*300/1.602e-19))^2 );
                collection_probability =  (exp(lambda_2/2) - 1)/(lambda_2/2); % use these two lines for the Taretto 2003 ApplPhysA PIN diode method.
            
            elseif strcmp(material.name,'model material')==1;
                %collection_probability = 1;
                %collection_probability =(material.Ld/thickness)*tanh(thickness/material.Ld);  %using the hyperbolic tan method
                lambda_2 = material.Vbi/(2*(1.38e-23)*300/1.6e-19) - sqrt( (thickness/material.Ld)^2 + (material.Vbi/(2*(1.3806488e-23)*300/1.602e-19))^2 );
                collection_probability =  (exp(lambda_2/2) - 1)/(lambda_2/2); % use these two lines for the Taretto 2003 ApplPhysA PIN diode method.
               
            elseif strcmp(material.name,'model material with alpha')==1;
                %collection_probability = 1;
                %collection_probability =(material.Ld/thickness)*tanh(thickness/material.Ld);  %using the hyperbolic tan method
                lambda_2 = material.Vbi/(2*(1.38e-23)*300/1.6e-19) - sqrt( (thickness/material.Ld)^2 + (material.Vbi/(2*(1.3806488e-23)*300/1.602e-19))^2 );
                collection_probability =  (exp(lambda_2/2) - 1)/(lambda_2/2); % u
            
            elseif strcmp(material.name,'silicon')==1;
               collection_probability = material.collection_probability;
               %collection_probability = 0.958;
            
            else 
                lambda_2 = material.Vbi/(2*(1.38e-23)*300/1.6e-19) - sqrt( (thickness/material.Ld)^2 + (material.Vbi/(2*(1.3806488e-23)*300/1.602e-19))^2 );
                collection_probability =  (exp(lambda_2/2) - 1)/(lambda_2/2); % use these two lines for the Taretto 2003 ApplPhysA PIN diode method.
            end
        end
        
        % Calculate Jsc
        function Jsc = Jsc(cell,wl)     % in units of A/m^2
            if strcmp(cell.material.name,'antimony_trisulfide')==1;
                
                n_elec_absorbed_photons = cell.absorption./nnl_root.energy_of_photon(wl);
                n_elec_absorbed_photons(cell.material.wlEg_index:end)=0;
                Jsc=1.6e-19*trapz(wl,n_elec_absorbed_photons)*cell.collection_probability;
%                Jsc=cell.absorption;
            else
                n_elec_absorbed_photons = cell.absorption./nnl_root.energy_of_photon(wl);
                n_elec_absorbed_photons(cell.material.wlEg_index:end)=0;
                
                
                Jsc=1.6e-19*cell.collection_probability*trapz(wl,n_elec_absorbed_photons);
            end
        end
        
        function J0 = J0(cell, light)
            qq = 1.6e-19;
            hh = 6.623e-34;
            cc = 3e8;
            kk = 1.38e-23;
            input_n = 1;

             %energy = nnl_root.energy_of_photon(light.wl(end)):((nnl_root.energy_of_photon(light.wl(1)) - nnl_root.energy_of_photon(light.wl(end)))/length(light.wl)):nnl_root.energy_of_photon(light.wl(1));
             absorptivity_ordered = zeros(1,length(light.wl));
             energy_ordered = zeros(1,length(light.wl));
             
             energywl = nnl_root.energy_of_photon(light.wl);
             for ii=1:1:length(light.wl)-1                
                 absorptivity_ordered(ii) = cell.absorptivity(length(light.wl) - ii);
                 energy_ordered(ii) = energywl(length(light.wl) - ii);
             end
            energy_ordered(end)=energy_ordered(end-1) + energy_ordered(2)-energy_ordered(1);
            absorptivity_ordered(end)=absorptivity_ordered(end-1);
            %interp1(energy_ordered(2:end),absorptivity(2:end),energy(2:end));
            [val,Egindex] = min(abs(energy_ordered-nnl_root.energy_of_photon(cell.material.wlEg)));
            absorptivity_ordered(1,1:Egindex)=0; %set absorptivity below the Eg to zero - so that Jo only calculates 'radiative' recombination
            L0prime = 1.*(2.*(input_n.^2)/((hh^3)*(cc^2))).*(energy_ordered.^2)./( (exp(energy_ordered./(kk*298)) -1 ));
             %G0R_integrand = cell.absorptivity.*pi.*L0prime;
             G0R_integrand = absorptivity_ordered.*pi.*L0prime;
             G0R = trapz(energy_ordered,G0R_integrand);
             J0 = qq.*G0R;
            %xg = qq.*cell.material.Eg./(kk*298);
            %J0 = qq*1.*(2.*pi.*(input_n.^2)/(((hh)^3)*((cc)^2))).*((kk*298)^3).*(exp(-xg)).*(2 + 2.*xg + xg^2);
            
        end
        
        function Voc = Voc(cell,light)
            if strcmp(cell.material.name,'antimony_trisulfide')==1;
                Jsc = nnl_root.Jsc(cell, light.wl);
                J0 = nnl_root.J0(cell,light);
                Voc = ((1.38e-23)*298/1.6e-19).*(log(Jsc/J0) + log(cell.material.phi));                                    % set Voc according to material parameters for Sb2S3
                %Voc = cell.material.Voc;                                    % set Voc according to material parameters for Sb2S3
            elseif strcmp(cell.material.name,'model material')==1;
                Jsc = nnl_root.Jsc(cell, light.wl);
                J0 = nnl_root.J0(cell,light);
                Voc = ((1.38e-23)*298/1.6e-19).*(log(Jsc/J0) + log(cell.material.phi));                                    % set Voc according to material parameters for Sb2S3
                %Voc = cell.material.Voc;                                    % set Voc according to material parameters for Sb2S3
            elseif strcmp(cell.material.name,'model material with alpha')==1;
                Jsc = nnl_root.Jsc(cell, light.wl);
                J0 = nnl_root.J0(cell,light);
                Voc = ((1.38e-23)*298/1.6e-19).*(log(Jsc/J0) + log(cell.material.phi));                                    % set Voc according to material parameters for Sb2S3
                %Voc = cell.material.Voc;                                    % set Voc according to material parameters for Sb2S3
            elseif strcmp(cell.material.name,'silicon')==1;
                Voc = ((1.38e-23)*298/1.6e-19)*log(cell.Jsc/cell.material.J0 + 1);   % calculate Voc with solar cell equation 
               % collection_probability = 1;
            else
                Jsc = nnl_root.Jsc(cell, light.wl);
                J0 = nnl_root.J0(cell,light);
                Voc = ((1.38e-23)*298/1.6e-19).*(log(Jsc/J0) + log(cell.material.phi));                                    % set Voc according to material parameters for Sb2S3
                %Voc = cell.material.Voc;                                    % set Voc according to material parameters for Sb2S3
            end
        end
        
        function cell_efficiency = cell_efficiency(cell,incident_spectra)
            cell_efficiency = cell.Jsc*cell.Voc*cell.material.FF/incident_spectra.sunshine.energy;
        end

        
        
        function current = IDE_transport(input)

            current = zeros(1,length(input.unit_cell));
            
            for ii=1:length(input.unit_cell)
    
                xx=ii*input.grid_size;

                
                
                % With shortened transport over the electrodes
                                
                    if xx<=input.electrode_width/2;
                  
                    distance_a = 0;
                    distance_b = input.electrode_distance - input.electrode_width/2 - xx;

                 elseif xx <=(input.electrode_distance - input.electrode_width/2);
                 
                    distance_a = xx-input.electrode_width/2;
                    distance_b = input.electrode_distance - input.electrode_width/2 - xx;

                 elseif xx <= input.electrode_distance;
                 
                    distance_a = xx-input.electrode_width/2;
                    distance_b = 0;
                    
                    
                 end

%                     distance_a = xx-input.electrode_width/2;
%                     distance_b = input.electrode_distance - input.electrode_width/2 - xx;
%                 
                    lambda_2h = input.material.Vbi/(2*(1.38e-23)*300/1.6e-19) - sqrt( (distance_a/input.material.Ld_h)^2 + (input.material.Vbi/(2*(1.3806488e-23)*300/1.602e-19))^2 );
                    collection_probability_h =  (exp(lambda_2h/2) - 1)/(lambda_2h/2); % u

                    lambda_2e = input.material.Vbi/(2*(1.38e-23)*300/1.6e-19) - sqrt( (distance_b/input.material.Ld_e)^2 + (input.material.Vbi/(2*(1.3806488e-23)*300/1.602e-19))^2 );
                    collection_probability_e =  (exp(lambda_2e/2) - 1)/(lambda_2e/2); % u

                    current(ii) = (collection_probability_h*collection_probability_e)^(0.5);



    
            end
        end
        
        function tandemeff = tandemeff(tandeminput)
            
            for ii= 1:1:length(tandeminput.eV)
                tandem.j2(ii) = tandeminput.T.*nnl_root.jscintegrate(tandeminput.eV(ii),tandeminput.eV(end),tandeminput.light);
                tandem.j1(ii) = tandeminput.T^2.*nnl_root.jscintegrate(tandeminput.eV(1),tandeminput.eV(ii),tandeminput.light);
                tandem.limitingj(ii) = min(min([tandeminput.j2(ii) tandeminput.j1(ii)]));
            end
            
            tandemeff = tandeminput;

        end
        
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Tandem cell scans%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        function thickness_scan = thickness(top_cell,sub_cell,scan_parameters,light)    % scan thickness of the top cell and return the below parameters

            numsteps = (scan_parameters.end-scan_parameters.start)/scan_parameters.step + 1;
            dimwl = length(light.wl);

            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            thickness_scan.absorption = zeros(numsteps,dimwl);
            thickness_scan.efficiency = zeros(1,numsteps);
            thickness_scan.thickness = zeros(1,numsteps);
            thickness_scan.Jsc = zeros(1,numsteps);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            for ii=1:1:numsteps;

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                top_cell_calc.thickness = (ii-1)*scan_parameters.step;                      % only changing the top_cell thickness here, below needed for Green's calculation
%                 top_cell_calc.xx = top_cell_calc.aa.*(top_cell_calc.material.alpha*top_cell_calc.thickness).^(top_cell_calc.bb);
%                 top_cell_calc.W0p = top_cell_calc.thickness.*(2+top_cell_calc.xx)./(1 + top_cell_calc.xx);
%                 top_cell_calc.norm_W0p = top_cell_calc.W0p./top_cell_calc.thickness;
%                 top_cell_calc.Ta = exp(-top_cell_calc.material.alpha.*top_cell_calc.W0p);
%                 top_cell_calc.Tp = top_cell_calc.Ta;
%                 top_cell_calc.Tm = top_cell_calc.Ta;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                thickness_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;  % save these parameters to thickness_scan
                thickness_scan.efficiency(ii) = total_cell_calc.efficiency;
                thickness_scan.thickness(ii) = top_cell_calc.thickness;
                thickness_scan.top_cell_Jsc(ii) = total_cell_calc.top_cell.Jsc;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            end
            
            thickness_scan.maximum_efficiency = max(thickness_scan.efficiency);
            thickness_scan.optimum_thickness_index = find(thickness_scan.efficiency==thickness_scan.maximum_efficiency);
            thickness_scan.optimum_thickness = thickness_scan.thickness(thickness_scan.optimum_thickness_index);
            
        end

        function thicknessJsc_scan_cell = thicknessJsccell(top_cell,sub_cell,scan_parameters,light)    % scan thickness of the top cell and return the below parameters

            numsteps = scan_parameters.numsteps;
            dimwl = length(light.wl);

            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            thicknessJsc_scan.absorption = zeros(numsteps,dimwl);
            thicknessJsc_scan.parasitic_Jsc = zeros(1,numsteps);
            thicknessJsc_scan.top_cell_Jsc = zeros(1,numsteps);
            thicknessJsc_scan.efficiency = zeros(1,numsteps);
            thicknessJsc_scan.topcellefficiency = zeros(1,numsteps);
            thicknessJsc_scan.subcellefficiency = zeros(1,numsteps);
            thicknessJsc_scan.requiredtopcellefficiency = zeros(1,numsteps);
            thicknessJsc_scan.thickness = zeros(1,numsteps);
            thicknessJsc_scan.top_cell_Jsc = zeros(1,numsteps);
            thicknessJsc_scan.top_cell_Jscparasitic = zeros(1,numsteps);
            thicknessJsc_scan.sub_cell_Jsc = zeros(1,numsteps);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            for ii=1:1:numsteps;

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                top_cell_calc.thickness = scan_parameters.steps(ii);                      % only changing the top_cell thickness here, below needed for Green's calculation
%                 top_cell_calc.xx = top_cell_calc.aa.*(top_cell_calc.material.alpha*top_cell_calc.thickness).^(top_cell_calc.bb);
%                 top_cell_calc.W0p = top_cell_calc.thickness.*(2+top_cell_calc.xx)./(1 + top_cell_calc.xx);
%                 top_cell_calc.norm_W0p = top_cell_calc.W0p./top_cell_calc.thickness;
%                 top_cell_calc.Ta = exp(-top_cell_calc.material.alpha.*top_cell_calc.W0p);
%                 top_cell_calc.Tp = top_cell_calc.Ta;
%                 top_cell_calc.Tm = top_cell_calc.Ta;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                thicknessJsc_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;  % save these parameters to thickness_scan
                thicknessJsc_scan.efficiency(ii) = total_cell_calc.efficiency;
                thicknessJsc_scan.topcellefficiency = total_cell_calc.top_cell.efficiency;
                thicknessJsc_scan.subcellefficiency = total_cell_calc.sub_cell.efficiency;
                thicknessJsc_scan.thickness(ii) = top_cell_calc.thickness;
                thicknessJsc_scan.top_cell_Jsc(ii) = total_cell_calc.top_cell.Jsc;
                thicknessJsc_scan.sub_cell_Jsc(ii) = total_cell_calc.sub_cell.Jsc;
                thicknessJsc_scan.top_cell_Jscparasitic = zeros(1,numsteps);
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            end
            
            thicknessJsc_scan.maximum_efficiency = max(thicknessJsc_scan.efficiency);
            thicknessJsc_scan.optimum_thickness_index = find(thicknessJsc_scan.efficiency==thicknessJsc_scan.maximum_efficiency);
            thicknessJsc_scan.optimum_thickness = thicknessJsc_scan.thickness(thicknessJsc_scan.optimum_thickness_index);
            
        end

        function thicknessJsc_scan = thicknessJsc(sub_cell,scan_parameters,light)    % scan thickness of the top cell and return the below parameters

            numsteps = scan_parameters.numsteps;
            dimwl = length(light.wl);

            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            %thicknessJsc_scan.absorption = zeros(numsteps,dimwl);
            absorption = zeros(1,dimwl);
            thicknessJsc_scan.parasitic_Jsc = zeros(1,numsteps);
            thicknessJsc_scan.top_cell_Jsc = zeros(1,numsteps);
            thicknessJsc_scan.sub_cell_powerloss = zeros(1,numsteps);
            thicknessJsc_scan.requiredtopcellefficiency = zeros(1,numsteps);
            %%%%%%%%%%%%%%%%%%%%%%%%%
            sub_cell_calc = sub_cell;
            
            for ii=1:1:numsteps;

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                absorption(1,:) = nnl_root.beer_lambert_absorption(light.sunshine.spectra, scan_parameters.material, scan_parameters.steps(ii));                      % only changing the top_cell thickness here, below needed for Green's calculation
                thicknessJsc_scan.absorption(ii,:) = nnl_root.beer_lambert_absorption(light.sunshine.spectra, scan_parameters.material, scan_parameters.steps(ii));                      % only changing the top_cell thickness here, below needed for Green's calculation
%                 top_cell_calc.xx = top_cell_calc.aa.*(top_cell_calc.material.alpha*top_cell_calc.thickness).^(top_cell_calc.bb);
%                 top_cell_calc.W0p = top_cell_calc.thickness.*(2+top_cell_calc.xx)./(1 + top_cell_calc.xx);
%                 top_cell_calc.norm_W0p = top_cell_calc.W0p./top_cell_calc.thickness;
%                 top_cell_calc.Ta = exp(-top_cell_calc.material.alpha.*top_cell_calc.W0p);
%                 top_cell_calc.Tp = top_cell_calc.Ta;
%                 top_cell_calc.Tm = top_cell_calc.Ta;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
               
                n_absorbed_photons = absorption(1,:)./nnl_root.energy_of_photon(light.wl);
                
                n_elec_absorbed_photons = n_absorbed_photons;
                n_elec_absorbed_photons(scan_parameters.material.wlEg_index:end)=0;
                thicknessJsc_scan.top_cell_Jsc(1,ii) = 1.6e-19*trapz(light.wl,n_elec_absorbed_photons);
                
                n_parasitic_absorbed_photons = n_absorbed_photons;
                n_parasitic_absorbed_photons(1:scan_parameters.material.wlEg_index)=0;
                SiwlEg_index = find(wlev(light.wl)==floor(1.12));
                n_parasitic_absorbed_photons(SiwlEg_index:end)=0;
                
                thicknessJsc_scan.parasitic_Jsc(1,ii) = 1.6e-19*trapz(light.wl,n_parasitic_absorbed_photons);
                thicknessJsc_scan.sub_cell_powerloss(1,ii) = thicknessJsc_scan.parasitic_Jsc(1,ii).*0.706.*0.828;
                
                inputspectra = light.sunshine.spectra;
                inputspectra(1:scan_parameters.material.wlEg_index) = 0;
                inputspectra(scan_parameters.material.wlEg_index:end) = light.sunshine.spectra(scan_parameters.material.wlEg_index:end) - absorption(1,scan_parameters.material.wlEg_index:end);
                sub_cell_calc.inputspectra = inputspectra;
                sub_cell_calc.optical_thickness = (sub_cell.top_light_trapping + sub_cell.bot_light_trapping)*sub_cell.thickness;    % optical thickness = real thickness x path length enhancement
                sub_cell_calc.absorption = nnl_root.green_Si_absorption(sub_cell_calc.inputspectra,sub_cell.material,sub_cell.thickness);    % calculate the sub_cell absorption with the optical thickness
                sub_cell_calc.collection_probability = nnl_root.collection_probability(sub_cell.thickness,sub_cell.material);
                sub_cell_calc.Jsc = nnl_root.Jsc(sub_cell_calc,light.wl);
                sub_cell_calc.Voc = nnl_root.Voc(sub_cell_calc, light);   
                sub_cell_calc.FF = sub_cell.material.FF;
                sub_cell_calc.power = sub_cell_calc.Jsc*sub_cell_calc.Voc*sub_cell.material.FF;
                sub_cell_calc.efficiency = sub_cell_calc.Jsc*sub_cell_calc.Voc*sub_cell.material.FF/light.sunshine.energy;
                           
                thicknessJsc_scan.requiredtopcellefficiency(1,ii) = 0.25 - sub_cell_calc.efficiency;
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            end
           thicknessJsc_scan.thickness = scan_parameters.steps; 
                       
        end
        
        function alpha_Ld_scan = alpha_Ld(top_cell,sub_cell,scan_parameters,light)    % scan thickness of the top cell and return the below parameters

            numsteps = scan_parameters.steps;
            dimwl = length(light.wl);

            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            %thicknessJsc_scan.absorption = zeros(numsteps,dimwl);
            alpha_Ld_scan.t_opt = zeros(1,numsteps);
            alpha_Ld_scan.top_cell_efficiency = zeros(1,numsteps);
            alpha_Ld_scan.sub_cell_efficiency = zeros(1,numsteps);
            alpha_Ld_scan.total_efficiency = zeros(1,numsteps);
            %%%%%%%%%%%%%%%%%%%%%%%%%
            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;
            
            for ii=1:1:numsteps;

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                top_cell_calc.material.Ld = scan_parameters.scanexp(ii);
                optimum_top_cell_thickness = nnl_root.optimise_top_cell(top_cell_calc,sub_cell_calc,light);
                top_cell_calc.thickness = optimum_top_cell_thickness;
                top_cell_calc.xx = top_cell_calc.aa.*(top_cell_calc.material.alpha*top_cell_calc.thickness).^(top_cell_calc.bb);
                top_cell_calc.W0p = top_cell_calc.thickness.*(2+top_cell_calc.xx)./(1 + top_cell_calc.xx);
                top_cell_calc.norm_W0p = top_cell_calc.W0p./top_cell_calc.thickness;
                top_cell_calc.Ta = exp(-top_cell_calc.material.alpha.*top_cell_calc.W0p);
                top_cell_calc.Tp = top_cell_calc.Ta;
                top_cell_calc.Tm = top_cell_calc.Ta;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                alpha_Ld_scan.t_opt(ii) = total_cell_calc.top_cell.thickness;
                alpha_Ld_scan.top_cell_efficiency(ii) = total_cell_calc.top_cell.efficiency;
                alpha_Ld_scan.sub_cell_efficiency(ii) = total_cell_calc.sub_cell.efficiency;
                alpha_Ld_scan.total_efficiency(ii) = total_cell_calc.efficiency;
                alpha_Ld_scan.Ld(ii) = total_cell_calc.top_cell.material.Ld;
            end
            
                       
        end
        
        function light_trapping_scan = light_trapping_scan(top_cell,sub_cell,scan_parameters,light)

            numsteps = scan_parameters.steps;
            dimwl = length(light.wl);

            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            light_trapping_scan.absorption = zeros(numsteps,dimwl);
            light_trapping_scan.efficiency = zeros(1,numsteps);
            light_trapping_scan.top_cell_efficiency = zeros(1,numsteps);
            light_trapping_scan.sub_cell_efficiency = zeros(1,numsteps);
            light_trapping_scan.steps = zeros(1,numsteps);
            light_trapping_scan.thickness = zeros(1,numsteps);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            % switching if we're varying light trapping in the top, sub or both layers.
%             if strcmp(scan_parameters.location, 'top_top') == 1
%                 for ii=1:1:numsteps;
%                     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     top_cell_calc.top_light_trapping(:) = ((numsteps-ii)/(numsteps)).*scan_parameters.start + (ii/numsteps).*scan_parameters.stop; 
%                     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
%                     light_trapping_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;
%                     light_trapping_scan.efficiency(ii) = total_cell_calc.efficiency;
%                     light_trapping_scan.top_cell_efficiency(ii) = total_cell_calc.top_cell.efficiency;
%                     light_trapping_scan.sub_cell_efficiency(ii) = total_cell_calc.sub_cell.efficiency;
%                     light_trapping_scan.steps(ii) = ii;
%                 end
%             end
            if strcmp(scan_parameters.location, 'top_top') == 1
            for ii=1:1:numsteps;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
                %wlEgindex = find(light.wl==floor(top_cell_calc.material.wlEg));
                %top_cell_calc.top_light_trapping(1,1:wlEgindex) =   (scan_parameters.stop-scan_parameters.start).*(ii-1)./(numsteps-1) + scan_parameters.start; 
                %top_cell_calc.top_light_trapping(1,wlEgindex:end) = 1; %  (scan_parameters.stop-scan_parameters.start).*(ii-1)./(numsteps-1) + scan_parameters.start; 
                
                top_cell_calc.top_light_trapping(:) =   (scan_parameters.stop-scan_parameters.start).*(ii-1)./(numsteps-1) + scan_parameters.start; 
                                
                %top_cell_calc.top_light_trapping_loss(1:dimwl) = 0;
                %top_cell_calc.top_light_trapping_loss(1:dimwl) = (1-(top_cell_calc.reflector_material.R).^((top_cell_calc.top_light_trapping + top_cell_calc.bot_light_trapping)/2));    % top cell front surface light trapping parasitic absorption factor
                 %plasmon_absorption_balance = nnl_root.plasmon_balance(top_cell_calc.material,top_cell_calc.reflector_material,light.wl);
                 %top_cell_calc.plasmon_metal_loss = plasmon_absorption_balance.absratio2;

                 %top_cell_calc.beerlambert = nnl_root.beer_lambert_absorption(1,top_cell_calc.material,top_cell_calc.thickness);
                 %top_cell_calc.top_light_trapping_loss = (top_cell_calc.top_light_trapping./top_cell_calc.material_lambertian).*top_cell_calc.plasmon_metal_loss;    % top cell front surface light trapping parasitic absorption factor
                 %top_cell_calc.top_light_trapping_abs = (1-top_cell_calc.top_light_trapping_loss).*((top_cell_calc.top_light_trapping./top_cell_calc.material_lambertian).*plasmon_absorption_balance.absratio1) + ( ((top_cell_calc.material_lambertian-top_cell_calc.top_light_trapping)./top_cell_calc.material_lambertian).*top_cell_calc.beerlambert);    % top cell front surface light trapping parasitic absorption factor
%                 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
                %optimum_top_cell_thickness = nnl_root.optimise_top_cell(top_cell_calc,sub_cell_calc,light);
                %top_cell_calc.thickness = optimum_top_cell_thickness;
                
                total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                %light_trapping_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;
                light_trapping_scan.efficiency(ii) = total_cell_calc.efficiency;
                light_trapping_scan.top_cell_efficiency(ii) = total_cell_calc.top_cell.efficiency;
                light_trapping_scan.sub_cell_efficiency(ii) = total_cell_calc.sub_cell.efficiency;
                light_trapping_scan.steps(ii) = top_cell_calc.top_light_trapping;
                light_trapping_scan.thickness(ii) = top_cell_calc.thickness;
            end

            end

            if strcmp(scan_parameters.location, 'top_bot') == 1
                for ii=1:1:numsteps;
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    top_cell_calc.bot_light_trapping(:) = (scan_parameters.stop-scan_parameters.start).*(ii-1)./(numsteps-1) + scan_parameters.start; 
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                    light_trapping_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;
                    light_trapping_scan.efficiency(ii) = total_cell_calc.efficiency;
                    light_trapping_scan.top_cell_efficiency(ii) = total_cell_calc.top_cell.efficiency;
                    light_trapping_scan.sub_cell_efficiency(ii) = total_cell_calc.sub_cell.efficiency;
                    light_trapping_scan.steps(ii) = top_cell_calc.bot_light_trapping;
                end
            end
            
            if strcmp(scan_parameters.location, 'sub_top') == 1
                for ii=1:1:numsteps;
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    sub_cell_calc.top_light_trapping(:) = (scan_parameters.stop-scan_parameters.start).*(ii-1)./(numsteps-1) + scan_parameters.start; 
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                    light_trapping_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;
                    light_trapping_scan.efficiency(ii) = total_cell_calc.efficiency;
                    light_trapping_scan.top_cell_efficiency(ii) = total_cell_calc.top_cell.efficiency;
                    light_trapping_scan.sub_cell_efficiency(ii) = total_cell_calc.sub_cell.efficiency;
                    light_trapping_scan.steps(ii) = sub_cell_calc.top_light_trapping;
                end
            end

            if strcmp(scan_parameters.location, 'sub_top') == 1
                for ii=1:1:numsteps;
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    sub_cell_calc.bot_light_trapping(:) = (scan_parameters.stop-scan_parameters.start).*(ii-1)./(numsteps-1) + scan_parameters.start; 
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                    light_trapping_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;
                    light_trapping_scan.efficiency(ii) = total_cell_calc.efficiency;
                    light_trapping_scan.top_cell_efficiency(ii) = total_cell_calc.top_cell.efficiency;
                    light_trapping_scan.sub_cell_efficiency(ii) = total_cell_calc.sub_cell.efficiency;
                    light_trapping_scan.steps(ii) = sub_cell_calc.bot_light_trapping;
                end
            end
            
            if strcmp(scan_parameters.location, 'both_bot') == 1
                for ii=1:1:numsteps;

                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    top_cell_calc.bot_light_trapping(:) = (scan_parameters.stop-scan_parameters.start).*(ii-1)./(numsteps-1) + scan_parameters.start; 
                    sub_cell_calc.bot_light_trapping(:) = (scan_parameters.stop-scan_parameters.start).*(ii-1)./(numsteps-1) + scan_parameters.start; 
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                    total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                    light_trapping_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;
                    light_trapping_scan.efficiency(ii) = total_cell_calc.efficiency;
                    light_trapping_scan.top_cell_efficiency(ii) = total_cell_calc.top_cell.efficiency;
                    light_trapping_scan.sub_cell_efficiency(ii) = total_cell_calc.sub_cell.efficiency;
                    light_trapping_scan.steps(ii) = top_cell_calc.bot_light_trapping;
                end
            end

        end

        function light_trapping_loss_scan = light_trapping_loss_scan(top_cell,sub_cell,scan_parameters,light)

            numsteps = scan_parameters.steps;
            dimwl = length(light.wl);

            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            light_trapping_loss_scan.absorption = zeros(numsteps,dimwl);
            light_trapping_loss_scan.efficiency = zeros(1,numsteps);
            light_trapping_loss_scan.top_cell_efficiency = zeros(1,numsteps);
            light_trapping_loss_scan.sub_cell_efficiency = zeros(1,numsteps);
            light_trapping_loss_scan.steps = zeros(1,numsteps);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            % switching if we're varying light trapping in the top, sub or both layers.
            if strcmp(scan_parameters.location, 'top_top') == 1
                for ii=1:1:numsteps;
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    top_cell_calc.top_light_trapping_loss(:) = ((numsteps-ii)/(numsteps)).*scan_parameters.start + (ii/numsteps).*scan_parameters.stop; 
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                    light_trapping_loss_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;
                    light_trapping_loss_scan.efficiency(ii) = total_cell_calc.efficiency;
                    light_trapping_loss_scan.top_cell_efficiency(ii) = total_cell_calc.top_cell.efficiency;
                    light_trapping_loss_scan.sub_cell_efficiency(ii) = total_cell_calc.sub_cell.efficiency;
                    light_trapping_loss_scan.steps(ii) = ii;
                end
            end
            
            if strcmp(scan_parameters.location, 'top_bot') == 1
                for ii=1:1:numsteps;
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    top_cell_calc.bot_light_trapping_loss(:) = ((numsteps-ii)/(numsteps)).*scan_parameters.start + (ii/numsteps).*scan_parameters.stop; 
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                    light_trapping_loss_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;
                    light_trapping_loss_scan.efficiency(ii) = total_cell_calc.efficiency;
                    light_trapping_loss_scan.top_cell_efficiency(ii) = total_cell_calc.top_cell.efficiency;
                    light_trapping_loss_scan.sub_cell_efficiency(ii) = total_cell_calc.sub_cell.efficiency;
                    light_trapping_loss_scan.steps(ii) = ii;
                end
            end
            
            if strcmp(scan_parameters.location, 'sub_top') == 1
                for ii=1:1:numsteps;
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    sub_cell_calc.top_light_trapping_loss(:) = ((numsteps-ii)/(numsteps)).*scan_parameters.start + (ii/numsteps).*scan_parameters.stop; 
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                    light_trapping_loss_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;
                    light_trapping_loss_scan.efficiency(ii) = total_cell_calc.efficiency;
                    light_trapping_loss_scan.top_cell_efficiency(ii) = total_cell_calc.top_cell.efficiency;
                    light_trapping_loss_scan.sub_cell_efficiency(ii) = total_cell_calc.sub_cell.efficiency;
                    light_trapping_loss_scan.steps(ii) = ii;
                end
            end
            
            if strcmp(scan_parameters.location, 'sub_bot') == 1
                for ii=1:1:numsteps;
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    sub_cell_calc.bot_light_trapping_loss(:) = ((numsteps-ii)/(numsteps)).*scan_parameters.start + (ii/numsteps).*scan_parameters.stop; 
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                    light_trapping_loss_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;
                    light_trapping_loss_scan.efficiency(ii) = total_cell_calc.efficiency;
                    light_trapping_loss_scan.top_cell_efficiency(ii) = total_cell_calc.top_cell.efficiency;
                    light_trapping_loss_scan.sub_cell_efficiency(ii) = total_cell_calc.sub_cell.efficiency;
                    light_trapping_loss_scan.steps(ii) = ii;
                end
            end
            
            if strcmp(scan_parameters.location, 'all_sub') == 1
                for ii=1:1:numsteps;

                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    top_cell_calc.sub_light_trapping_loss(:) = ((numsteps-ii)/(numsteps)).*scan_parameters.start + (ii/numsteps).*scan_parameters.stop; 
                    sub_cell_calc.sub_light_trapping_loss(:) = ((numsteps-ii)/(numsteps))*scan_parameters.start + (ii/numsteps)*scan_parameters.stop;
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                    total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                    light_trapping_loss_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;
                    light_trapping_loss_scan.efficiency(ii) = total_cell_calc.efficiency;
                    light_trapping_loss_scan.top_cell_efficiency(ii) = total_cell_calc.top_cell.efficiency;
                    light_trapping_loss_scan.sub_cell_efficiency(ii) = total_cell_calc.sub_cell.efficiency;
                    light_trapping_loss_scan.steps(ii) = ii;
                end
            end

        end
    
        function trapping_and_loss_scan = trapping_and_loss_scan(top_cell,sub_cell,scan_parameters,light)

            num_trapping_steps = scan_parameters.trapping_steps;
            num_loss_steps = scan_parameters.loss_steps;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            %trapping_and_loss_scan.absorption = zeros(numsteps,dimwl);
            trapping_and_loss_scan.efficiency = zeros(num_loss_steps,num_trapping_steps);
            trapping_and_loss_scan.top_cell_efficiency = zeros(num_loss_steps,num_trapping_steps);
            trapping_and_loss_scan.sub_cell_efficiency = zeros(num_loss_steps,num_trapping_steps);
            trapping_and_loss_scan.trapping_steps = zeros(1,num_trapping_steps);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            % switching if we're varying light trapping in the top, sub or both layers.
            if strcmp(scan_parameters.location, 'top_top') == 1
                for ii=1:1:num_trapping_steps;
                    for jj=1:1:num_loss_steps;
                        
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        top_cell_calc.top_light_trapping_loss(:) = ((num_loss_steps-jj)/(num_loss_steps)).*scan_parameters.loss_start + (jj/num_loss_steps).*scan_parameters.loss_stop; 
                        top_cell_calc.top_light_trapping(:) = (scan_parameters.trapping_stop-scan_parameters.trapping_start).*(ii-1)./(num_trapping_steps-1) + scan_parameters.trapping_start;
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                        trapping_and_loss_scan.efficiency(ii,jj) = total_cell_calc.efficiency;
                        trapping_and_loss_scan.top_cell_efficiency(ii,jj) = total_cell_calc.top_cell.efficiency;
                        trapping_and_loss_scan.sub_cell_efficiency(ii,jj) = total_cell_calc.sub_cell.efficiency;
                        trapping_and_loss_scan.trapping_steps(ii) = top_cell_calc.top_light_trapping;
                    end
                    
                end
            end
        end
        
        function thickness_both_scan = thickness_both_scan(top_cell,sub_cell,scan_parameters,light)

            num_top_thickness_steps = scan_parameters.top_thickness_steps+1;
            num_sub_thickness_steps = scan_parameters.sub_thickness_steps+1;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            %trapping_and_loss_scan.absorption = zeros(numsteps,dimwl);
            thickness_both_scan.efficiency = zeros(num_top_thickness_steps,num_sub_thickness_steps);
            thickness_both_scan.Jsc = zeros(num_top_thickness_steps,num_sub_thickness_steps);
            thickness_both_scan.top_cell_efficiency = zeros(num_top_thickness_steps,num_sub_thickness_steps);
            thickness_both_scan.sub_cell_efficiency = zeros(num_top_thickness_steps,num_sub_thickness_steps);
            thickness_both_scan.sub_cell_Jsc = zeros(num_top_thickness_steps,num_sub_thickness_steps);
            thickness_both_scan.sub_cell_Jsc = zeros(num_top_thickness_steps,num_sub_thickness_steps);
            %trapping_and_loss_scan.steps = zeros(1,numsteps);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            % switching if we're varying light trapping in the top, sub or both layers.
                for ii=1:1:num_top_thickness_steps;
                    for jj=1:1:num_sub_thickness_steps;
                        
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        top_cell_calc.thickness = ((num_top_thickness_steps-ii)/(num_top_thickness_steps)).*scan_parameters.top_thickness_start + (ii/num_top_thickness_steps).*scan_parameters.top_thickness_stop;
                        sub_cell_calc.thickness = ((num_sub_thickness_steps-jj)/(num_sub_thickness_steps)).*scan_parameters.sub_thickness_start + (jj/num_sub_thickness_steps).*scan_parameters.sub_thickness_stop; 
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                        thickness_both_scan.efficiency(ii,jj) = total_cell_calc.efficiency;
                        thickness_both_scan.Jsc(ii,jj) = total_cell_calc.Jsc;
                        thickness_both_scan.top_cell_efficiency(ii,jj) = total_cell_calc.top_cell.efficiency;
                        thickness_both_scan.sub_cell_efficiency(ii,jj) = total_cell_calc.sub_cell.efficiency;
                        thickness_both_scan.top_cell_Jsc(ii,jj) = total_cell_calc.top_cell.Jsc;
                        thickness_both_scan.sub_cell_Jsc(ii,jj) = total_cell_calc.sub_cell.Jsc;
                        
                        %light_trapping_loss_scan.steps(ii) = ii;
                    end
                end
                
            thickness_both_scan.maximum_efficiency_noSi = max(thickness_both_scan.efficiency(:,1));
            thickness_both_scan.optimum_thickness_noSi_index = find(thickness_both_scan.efficiency(:,1)==thickness_both_scan.maximum_efficiency_noSi);
            thickness_both_scan.maximum_efficiency_allSi = max(thickness_both_scan.efficiency(:,num_sub_thickness_steps));
            thickness_both_scan.optimum_thickness_allSi_index = find(thickness_both_scan.efficiency(:,num_sub_thickness_steps)==thickness_both_scan.maximum_efficiency_allSi);
            thickness_both_scan.optimum_thickness_noSi = scan_parameters.thickness_top_cell(thickness_both_scan.optimum_thickness_noSi_index);
            thickness_both_scan.optimum_thickness_allSi = scan_parameters.thickness_top_cell(thickness_both_scan.optimum_thickness_allSi_index);
                
        end
 
        function A0_scan = A0_scan(top_cell,sub_cell,scan_parameters,light)

            num_A0_steps = scan_parameters.A0_steps+1;
            num_light_steps = scan_parameters.light_steps+1;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            %trapping_and_loss_scan.absorption = zeros(numsteps,dimwl);
            A0_scan.efficiency = zeros(num_A0_steps,num_light_steps);
            A0_scan.Jsc = zeros(num_A0_steps,num_light_steps);
            A0_scan.top_cell_efficiency = zeros(num_A0_steps,num_light_steps);
            A0_scan.sub_cell_efficiency = zeros(num_A0_steps,num_light_steps);
            A0_scan.sub_cell_Jsc = zeros(num_A0_steps,num_light_steps);
            A0_scan.sub_cell_Jsc = zeros(num_A0_steps,num_light_steps);
            A0_scan.top_cell_thickness = zeros(num_A0_steps,num_light_steps);
            
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

                for ii=1:1:num_A0_steps;
                    for jj=1:1:num_light_steps;
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        top_cell_calc.material.Astar0 = 1*10^((scan_parameters.A0_stop-scan_parameters.A0_start).*(ii-1)./(num_A0_steps-1) + scan_parameters.A0_start); % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
                        top_cell_calc.material.alpha = 1e-7.*top_cell_calc.material.Astar0.*(((nnl_root.wlev(light.wl) - top_cell_calc.material.Eg)/(0.0256)).^(1/2));
                        top_cell_calc.material.wlEg = nnl_root.wlev(top_cell_calc.material.Eg);
                        top_cell_calc.material.wlEg_index = find(light.wl==floor(top_cell_calc.material.wlEg));
                        top_cell_calc.material.alpha(top_cell_calc.material.wlEg_index:end)=0;
                        top_cell_calc.top_light_trapping = (scan_parameters.light_stop-scan_parameters.light_start).*(jj-1)./(num_light_steps-1) + scan_parameters.light_start;
                        optimum_top_cell_thickness = nnl_root.optimise_top_cell(top_cell_calc,sub_cell_calc,light);
                        top_cell_calc.thickness = optimum_top_cell_thickness;
                        
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);

                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        A0_scan.efficiency(ii,jj) = total_cell_calc.efficiency;
                        A0_scan.Jsc(ii,jj) = total_cell_calc.Jsc;
                        A0_scan.top_cell_efficiency(ii,jj) = total_cell_calc.top_cell.efficiency;
                        A0_scan.sub_cell_efficiency(ii,jj) = total_cell_calc.sub_cell.efficiency;
                        A0_scan.top_cell_Jsc(ii,jj) = total_cell_calc.top_cell.Jsc;
                        A0_scan.sub_cell_Jsc(ii,jj) = total_cell_calc.sub_cell.Jsc;
                        A0_scan.top_cell_thickness(ii,jj) = total_cell_calc.top_cell.thickness;
                    end
                     fprintf(num2str(ii));%'nnl_root imported ok\n');
                end
             
        end
                
        function light_and_thickness_scan = light_and_thickness_scan(top_cell,sub_cell,scan_parameters,light)

            num_thickness_steps = scan_parameters.thickness_steps;
            num_light_steps = scan_parameters.light_steps;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            light_and_thickness_scan.efficiency = zeros(num_thickness_steps,num_light_steps);
            light_and_thickness_scan.Jsc = zeros(num_thickness_steps,num_light_steps);
            light_and_thickness_scan.top_cell_efficiency = zeros(num_thickness_steps,num_light_steps);
            light_and_thickness_scan.sub_cell_efficiency = zeros(num_thickness_steps,num_light_steps);
            light_and_thickness_scan.sub_cell_Jsc = zeros(num_thickness_steps,num_light_steps);
            light_and_thickness_scan.sub_cell_Jsc = zeros(num_thickness_steps,num_light_steps);
            
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

                for ii=1:1:num_thickness_steps;
                    for jj=1:1:num_light_steps;
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        top_cell_calc.thickness = (scan_parameters.thickness_stop-scan_parameters.thickness_start).*(ii-1)./(num_thickness_steps-1) + scan_parameters.thickness_start; % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
                        top_cell_calc.top_light_trapping = (scan_parameters.light_stop-scan_parameters.light_start).*(jj-1)./(num_light_steps-1) + scan_parameters.light_start;
                        
                        top_cell_calc.top_light_trapping_loss(1:length(light.wl)) = 0;
                        %top_cell_calc.top_light_trapping_loss(1:length(light.wl)) = (1-(top_cell_calc.reflector_material.R).^((top_cell_calc.top_light_trapping + top_cell_calc.bot_light_trapping)/2));    % top cell front surface light trapping parasitic absorption factor
%                         plasmon_absorption_balance = nnl_root.plasmon_balance(top_cell_calc.material,top_cell_calc.reflector_material,light.wl);
%                         top_cell_calc.plasmon_metal_loss = plasmon_absorption_balance.absratio2;
%                         top_cell_calc.beerlambert = nnl_root.beer_lambert_absorption(1,top_cell_calc.material,top_cell_calc.thickness);
%                         top_cell_calc.top_light_trapping_loss = (top_cell_calc.top_light_trapping./top_cell_calc.material_lambertian).*top_cell_calc.plasmon_metal_loss;    % top cell front surface light trapping parasitic absorption factor
%                         %top_cell_calc.top_light_trapping_abs(1:length(light.wl)) = (top_cell_calc.top_light_trapping./top_cell_calc.material_lambertian).*plasmon_absorption_balance.absratio2;    % top cell front surface light trapping parasitic absorption factor
%                         top_cell_calc.top_light_trapping_abs = (1-top_cell_calc.top_light_trapping_loss).*((top_cell_calc.top_light_trapping./top_cell_calc.material_lambertian).*plasmon_absorption_balance.absratio1) + ( ((top_cell_calc.material_lambertian-top_cell_calc.top_light_trapping)./top_cell_calc.material_lambertian).*top_cell_calc.beerlambert);
%                         
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);

                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        light_and_thickness_scan.efficiency(ii,jj) = total_cell_calc.efficiency;
                        light_and_thickness_scan.Jsc(ii,jj) = total_cell_calc.Jsc;
                        light_and_thickness_scan.top_cell_efficiency(ii,jj) = total_cell_calc.top_cell.efficiency;
                        light_and_thickness_scan.sub_cell_efficiency(ii,jj) = total_cell_calc.sub_cell.efficiency;
                        light_and_thickness_scan.top_cell_Jsc(ii,jj) = total_cell_calc.top_cell.Jsc;
                        light_and_thickness_scan.sub_cell_Jsc(ii,jj) = total_cell_calc.sub_cell.Jsc;
                     end
                end
             
        end
        
        function lightRf_and_thickness_scan = lightRf_and_thickness_scan(top_cell,sub_cell,scan_parameters,light)

            num_thickness_steps = scan_parameters.thickness_steps;
            num_light_steps = scan_parameters.light_steps;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            lightRf_and_thickness_scan.efficiency = zeros(num_thickness_steps,num_light_steps);
            lightRf_and_thickness_scan.Jsc = zeros(num_thickness_steps,num_light_steps);
            lightRf_and_thickness_scan.top_cell_efficiency = zeros(num_thickness_steps,num_light_steps);
            lightRf_and_thickness_scan.sub_cell_efficiency = zeros(num_thickness_steps,num_light_steps);
            lightRf_and_thickness_scan.sub_cell_Jsc = zeros(num_thickness_steps,num_light_steps);
            lightRf_and_thickness_scan.sub_cell_Jsc = zeros(num_thickness_steps,num_light_steps);
            
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

                for ii=1:1:num_thickness_steps;
                    for jj=1:1:num_light_steps;
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        top_cell_calc.thickness = (scan_parameters.thickness_stop-scan_parameters.thickness_start).*(ii-1)./(num_thickness_steps-1) + scan_parameters.thickness_start; % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
                        top_cell_calc.Rf = (scan_parameters.light_stop-scan_parameters.light_start).*(jj-1)./(num_light_steps-1) + scan_parameters.light_start;
                        
                        %top_cell_calc.top_light_trapping_loss(1:length(light.wl)) = 0;
                        %top_cell_calc.top_light_trapping_loss(1:length(light.wl)) = (1-(top_cell_calc.reflector_material.R).^((top_cell_calc.top_light_trapping + top_cell_calc.bot_light_trapping)/2));    % top cell front surface light trapping parasitic absorption factor
                        %plasmon_absorption_balance = nnl_root.plasmon_balance(top_cell_calc.material,top_cell_calc.reflector_material,light.wl);
                        %top_cell_calc.plasmon_metal_loss = plasmon_absorption_balance.absratio2;
                        %top_cell_calc.top_light_trapping_loss(1:length(light.wl)) = (top_cell_calc.top_light_trapping./top_cell_calc.material_lambertian).*top_cell_calc.plasmon_metal_loss;    % top cell front surface light trapping parasitic absorption factor

                        
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);

                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        lightRf_and_thickness_scan.efficiency(ii,jj) = total_cell_calc.efficiency;
                        lightRf_and_thickness_scan.Jsc(ii,jj) = total_cell_calc.Jsc;
                        lightRf_and_thickness_scan.top_cell_efficiency(ii,jj) = total_cell_calc.top_cell.efficiency;
                        lightRf_and_thickness_scan.sub_cell_efficiency(ii,jj) = total_cell_calc.sub_cell.efficiency;
                        lightRf_and_thickness_scan.top_cell_Jsc(ii,jj) = total_cell_calc.top_cell.Jsc;
                        lightRf_and_thickness_scan.sub_cell_Jsc(ii,jj) = total_cell_calc.sub_cell.Jsc;
                     end
                end
             
        end
       
        function intermediate_reflector_scan = intermediate_reflector(top_cell,sub_cell,scan_parameters,light)

            numsteps = scan_parameters.steps;
            dimwl = length(light.wl);
            %intermediate_reflector_index = find(light.wl==intermediate_reflector_scan_parameters.wavelength);
            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            intermediate_reflector_scan.absorption = zeros(numsteps,dimwl);
            intermediate_reflector_scan.efficiency = zeros(1,numsteps);
            intermediate_reflector_scan.top_cell_efficiency = zeros(1,numsteps);
            intermediate_reflector_scan.sub_cell_efficiency = zeros(1,numsteps);
            intermediate_reflector_scan.steps = zeros(1,numsteps);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            top_cell_calc.bot_intermediate_wl = scan_parameters.wavelength;
            
            for ii=1:1:numsteps;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                top_cell_calc.bot_intermediate_strength = (scan_parameters.stop-scan_parameters.start).*(ii-1)./(numsteps-1) + scan_parameters.start; 
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                
                intermediate_reflector_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;
                intermediate_reflector_scan.efficiency(ii) = total_cell_calc.efficiency;
                intermediate_reflector_scan.top_cell_efficiency(ii) = total_cell_calc.top_cell.efficiency;
                intermediate_reflector_scan.sub_cell_efficiency(ii) = total_cell_calc.sub_cell.efficiency;
                intermediate_reflector_scan.steps(ii) = top_cell_calc.bot_intermediate_strength;
            end

        end

        function reflector_and_thickness_scan = reflector_and_thickness_scan(top_cell,sub_cell,scan_parameters,light)

            num_thickness_steps = scan_parameters.thickness_steps;
            num_reflector_steps = scan_parameters.reflector_steps;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            reflector_and_thickness_scan.efficiency = zeros(num_thickness_steps,num_reflector_steps);
            reflector_and_thickness_scan.thickness = zeros(1,num_thickness_steps);
            reflector_and_thickness_scan.Jsc = zeros(num_thickness_steps,num_reflector_steps);
            reflector_and_thickness_scan.top_cell_efficiency = zeros(num_thickness_steps,num_reflector_steps);
            reflector_and_thickness_scan.sub_cell_efficiency = zeros(num_thickness_steps,num_reflector_steps);
            reflector_and_thickness_scan.top_cell_Jsc = zeros(num_thickness_steps,num_reflector_steps);
            reflector_and_thickness_scan.sub_cell_Jsc = zeros(num_thickness_steps,num_reflector_steps);
            reflector_and_thickness_scan.top_cell_numphot_abs = zeros(num_thickness_steps,num_reflector_steps);
            reflector_and_thickness_scan.sub_cell_numphot_abs = zeros(num_thickness_steps,num_reflector_steps);
            %reflector_and_thickness_scan.reflected = zeros(num_thickness_steps,num_reflector_steps);
            reflector_and_thickness_scan.numphot_reflected = zeros(num_thickness_steps,num_reflector_steps);
            reflector_and_thickness_scan.reflector = zeros(1,length(light.wl));
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            top_cell_calc.bot_intermediate_wl = scan_parameters.wavelength;
                
                for ii=1:1:num_thickness_steps;
                    for jj=1:1:num_reflector_steps;
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        
                        top_cell_calc.logthickness = (log(scan_parameters.thickness_stop)- log(scan_parameters.thickness_start)).*(ii-1)./(num_thickness_steps-1) + log(scan_parameters.thickness_start); % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
                        top_cell_calc.thickness = 10.^(top_cell_calc.logthickness); 
                        %top_cell_calc.thickness = (scan_parameters.thickness_stop-scan_parameters.thickness_start).*(ii-1)./(num_thickness_steps-1) + scan_parameters.thickness_start; % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
                        top_cell_calc.bot_intermediate_strength = (scan_parameters.reflector_stop-scan_parameters.reflector_start).*(jj-1)./(num_reflector_steps-1) + scan_parameters.reflector_start;
                        
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);

                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        reflector_and_thickness_scan.efficiency(ii,jj) = total_cell_calc.efficiency;
                        reflector_and_thickness_scan.Jsc(ii,jj) = total_cell_calc.Jsc;
                        reflector_and_thickness_scan.top_cell_efficiency(ii,jj) = total_cell_calc.top_cell.efficiency;
                        reflector_and_thickness_scan.sub_cell_efficiency(ii,jj) = total_cell_calc.sub_cell.efficiency;
                        reflector_and_thickness_scan.top_cell_Jsc(ii,jj) = total_cell_calc.top_cell.Jsc;
                        reflector_and_thickness_scan.sub_cell_Jsc(ii,jj) = total_cell_calc.sub_cell.Jsc;
                        reflector_and_thickness_scan.top_cell_numphot_abs(ii,jj) = trapz(light.wl,total_cell_calc.top_cell.absorption./nnl_root.energy_of_photon(total_cell_calc.top_cell.absorption));
                        reflector_and_thickness_scan.sub_cell_numphot_abs(ii,jj) = trapz(light.wl,total_cell_calc.sub_cell.absorption./nnl_root.energy_of_photon(total_cell_calc.sub_cell.absorption));
                        %reflector_and_thickness_scan.reflected = top_cell_calc.bot_intermediate_strength.*(total_cell_calc.top_cell.inputspectra - total_cell_calc.top_cell.absorption);
                        reflector_and_thickness_scan.reflector(scan_parameters.wlindex:end)=0 ;
                        reflector_and_thickness_scan.reflector(1:scan_parameters.wlindex)= top_cell_calc.bot_intermediate_strength ;
                        reflector_and_thickness_scan.reflected = reflector_and_thickness_scan.reflector.*(total_cell_calc.top_cell.inputspectra - total_cell_calc.top_cell.absorption);
                        %reflector_and_thickness_scan.reflected = total_cell_calc.top_cell.inputspectra - total_cell_calc.top_cell.absorption - total_cell_calc.sub_cell.absorption - total_cell_calc.transmission;
                        reflector_and_thickness_scan.numphot_reflected(ii,jj) = trapz(light.wl,(reflector_and_thickness_scan.reflected)./nnl_root.energy_of_photon(reflector_and_thickness_scan.reflected));
                        reflector_and_thickness_scan.thickness(ii) = top_cell_calc.thickness;
                        %reflector_and_thickness_scan.numphot_reflected(ii,jj) = reflector_and_thickness_scan.reflected./nnl_root.energy_of_photon(reflector_and_thickness_scan.reflected);
                     end
                end
             
        end
       
        function W0p_scan = W0p(top_cell,sub_cell,scan_parameters,light)    % scan thickness of the top cell and return the below parameters

            numsteps = (scan_parameters.end-scan_parameters.start)/scan_parameters.step + 1;
            dimwl = length(light.wl);

            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            W0p_scan.absorption = zeros(numsteps,dimwl);
            W0p_scan.total_absorption = zeros(numsteps,dimwl);
            W0p_scan.W0p = zeros(numsteps,dimwl);
            W0p_scan.efficiency = zeros(1,numsteps);
            W0p_scan.thickness = zeros(1,numsteps);
            W0p_scan.Jsc = zeros(1,numsteps);
            W0p_scan.scan = zeros(1,numsteps);
            W0p_scan.pathlengthenhancement = zeros(1,numsteps);
            W0p_scan.loss = zeros(numsteps,dimwl);
            W0p_scan.xx = zeros(numsteps,dimwl);
            W0p_scan.norm_W0p = zeros(1,numsteps);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            for ii=1:1:numsteps;

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %top_cell_calc.W0p = (ii-1)*scan_parameters.step.*top_cell_calc.W0p;                      % only changing the top_cell thickness here
                top_cell_calc.W0p = (scan_parameters.W0p_original - top_cell_calc.thickness).*(ii - 1)./(numsteps - 1) + top_cell_calc.thickness;                      % only changing the top_cell thickness here
                top_cell_calc.Ta = exp(-top_cell_calc.material.alpha.*top_cell_calc.W0p);
                top_cell_calc.Tp = top_cell_calc.Ta;
                top_cell_calc.Tm = top_cell_calc.Ta;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                W0p_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;  % save these parameters to thickness_scan
                W0p_scan.total_absorption(ii,:) = total_cell_calc.top_cell.total_absorption;
                W0p_scan.W0p(ii,:) = top_cell_calc.W0p;  % save these parameters to thickness_scan
                W0p_scan.efficiency(ii) = total_cell_calc.efficiency;
                W0p_scan.scan(ii) = ii*scan_parameters.step;
                W0p_scan.pathlengthenhancement(ii) = ii./(numsteps).*top_cell_calc.material_lambertian;
                W0p_scan.top_cell_Jsc(ii) = total_cell_calc.top_cell.Jsc;
                W0p_scan.loss(ii,:) = total_cell_calc.top_cell.bot_light_trapping_loss;
                W0p_scan.xx(ii,:) = total_cell_calc.top_cell.xx;
                W0p_scan.norm_W0p(ii) = mean(W0p_scan.W0p(ii,:))./total_cell_calc.top_cell.thickness;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            end
            
%             W0p_scan.maximum_efficiency = max(W0p_scan.efficiency);
%             W0p_scan.optimum_thickness_index = find(W0p_scan.efficiency==W0p_scan.maximum_efficiency);
%             W0p_scan.optimum_thickness = W0p_scan.thickness(W0p_scan.optimum_thickness_index);
            
        end

        function Rf_scan = Rf(top_cell,sub_cell,scan_parameters,light)    % scan thickness of the top cell and return the below parameters

            numsteps = scan_parameters.numsteps;
            dimwl = length(light.wl);

            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            Rf_scan.absorption = zeros(numsteps,dimwl);
            Rf_scan.total_absorption = zeros(numsteps,dimwl);
            Rf_scan.reflector_abs = zeros(numsteps,dimwl);
            Rf_scan.Rf = zeros(numsteps,dimwl);
            Rf_scan.efficiency = zeros(1,numsteps);
            Rf_scan.thickness = zeros(1,numsteps);
            Rf_scan.Jsc = zeros(1,numsteps);
            Rf_scan.scan = zeros(1,numsteps);
            Rf_scan.pathlengthenhancement = zeros(1,numsteps);
            Rf_scan.loss = zeros(numsteps,dimwl);
            Rf_scan.xx = zeros(numsteps,dimwl);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            for ii=1:1:numsteps;

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %top_cell_calc.W0p = (ii-1)*scan_parameters.step.*top_cell_calc.W0p;                      % only changing the top_cell thickness here
                %top_cell_calc.Rf = (scan_parameters.Rf_original - 0).*(ii - 1)./(numsteps - 1) + 0;                      % only changing the top_cell thickness here
                %top_cell_calc.Rf = 1 - ((1-top_cell_calc.material.n.^2).*(ii - 1)./(numsteps - 1) + top_cell_calc.material.n.^2)./(top_cell_calc.material.n.^2);
                %top_cell_calc.Rf =     1 - (((scan_parameters.end - scan_parameters.start)/(numsteps - 1)).*(ii - 1) + scan_parameters.start)./top_cell_calc.material.n.^2;
                top_cell_calc.Rf =     (((scan_parameters.end - scan_parameters.start)/(numsteps - 1)).*(ii - 1) + scan_parameters.start);
                
                optimum_top_cell_thickness = nnl_root.optimise_top_cell(top_cell_calc,sub_cell_calc,light);
                top_cell_calc.thickness = optimum_top_cell_thickness;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                top_cell_calc.xx = top_cell_calc.aa.*((top_cell_calc.material.alpha*top_cell_calc.thickness).^(top_cell_calc.bb));
                top_cell_calc.W0p = top_cell_calc.thickness.*(2+top_cell_calc.xx)./(1 + top_cell_calc.xx);
                top_cell_calc.norm_W0p = top_cell_calc.W0p./top_cell_calc.thickness;
                top_cell_calc.Ta = exp(-top_cell_calc.material.alpha.*top_cell_calc.W0p);
                top_cell_calc.Tp = top_cell_calc.Ta;
                top_cell_calc.Tm = top_cell_calc.Ta;
                
                total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                Rf_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;  % save these parameters to thickness_scan
                Rf_scan.total_absorption(ii,:) = total_cell_calc.top_cell.total_absorption;
                Rf_scan.reflector_abs(ii,:) = total_cell_calc.top_cell.reflector_abs;  % save these parameters to thickness_scan
                Rf_scan.Rf(ii,:) = top_cell_calc.Rf;  % save these parameters to thickness_scan
                Rf_scan.efficiency(ii) = total_cell_calc.efficiency;
                Rf_scan.scan(ii) = ii/numsteps;
                Rf_scan.pathlengthenhancement(ii) = ii./(numsteps).*top_cell_calc.material_lambertian;
                Rf_scan.top_cell_Jsc(ii) = total_cell_calc.top_cell.Jsc;
                Rf_scan.loss(ii,:) = total_cell_calc.top_cell.bot_light_trapping_loss;
                Rf_scan.xx(ii,:) = total_cell_calc.top_cell.xx;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            end
            
%             W0p_scan.maximum_efficiency = max(W0p_scan.efficiency);
%             W0p_scan.optimum_thickness_index = find(W0p_scan.efficiency==W0p_scan.maximum_efficiency);
%             W0p_scan.optimum_thickness = W0p_scan.thickness(W0p_scan.optimum_thickness_index);
            
        end
        
        function Rb_scan = Rb(top_cell,sub_cell,scan_parameters,light)    % scan thickness of the top cell and return the below parameters

            numsteps = scan_parameters.numsteps;
            dimwl = length(light.wl);

            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            Rb_scan.absorption = zeros(numsteps,dimwl);
            Rb_scan.total_absorption = zeros(numsteps,dimwl);
            Rb_scan.Rf = zeros(numsteps,dimwl);
            Rb_scan.efficiency = zeros(1,numsteps);
            Rb_scan.thickness = zeros(1,numsteps);
            Rb_scan.Jsc = zeros(1,numsteps);
            Rb_scan.scan = zeros(1,numsteps);
            Rb_scan.pathlengthenhancement = zeros(1,numsteps);
            Rb_scan.loss = zeros(numsteps,dimwl);
            Rb_scan.xx = zeros(numsteps,dimwl);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            for ii=1:1:numsteps;

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %top_cell_calc.W0p = (ii-1)*scan_parameters.step.*top_cell_calc.W0p;                      % only changing the top_cell thickness here
                top_cell_calc.Rb = (scan_parameters.end - scan_parameters.start).*(ii - 1)./(numsteps - 1) + 0;                      % only changing the top_cell thickness here
                %top_cell_calc.Rb = 1 - ((1-top_cell_calc.material.n.^2).*(ii - 1)./(numsteps - 1) + top_cell_calc.material.n.^2)./(top_cell_calc.material.n.^2);

                optimum_top_cell_thickness = nnl_root.optimise_top_cell(top_cell_calc,sub_cell_calc,light);
                top_cell_calc.thickness = optimum_top_cell_thickness;
                top_cell_calc.xx = top_cell_calc.aa.*(top_cell_calc.material.alpha*top_cell_calc.thickness).^(top_cell_calc.bb);
                top_cell_calc.W0p = top_cell_calc.thickness.*(2+top_cell_calc.xx)./(1 + top_cell_calc.xx);
                top_cell_calc.norm_W0p = top_cell_calc.W0p./top_cell_calc.thickness;
                top_cell_calc.Ta = exp(-top_cell_calc.material.alpha.*top_cell_calc.W0p);
                top_cell_calc.Tp = top_cell_calc.Ta;
                top_cell_calc.Tm = top_cell_calc.Ta;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                Rb_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;  % save these parameters to thickness_scan
                Rb_scan.total_absorption(ii,:) = total_cell_calc.top_cell.total_absorption;
                Rb_scan.Rb(ii,:) = top_cell_calc.Rb;  % save these parameters to thickness_scan
                Rb_scan.efficiency(ii) = total_cell_calc.efficiency;
                Rb_scan.scan(ii) = ii;
                Rb_scan.pathlengthenhancement(ii) = ii./(numsteps).*top_cell_calc.material_lambertian;
                Rb_scan.top_cell_Jsc(ii) = total_cell_calc.top_cell.Jsc;
                Rb_scan.loss(ii,:) = total_cell_calc.top_cell.bot_light_trapping_loss;
                Rb_scan.xx(ii,:) = total_cell_calc.top_cell.xx;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            end
        end
        
            function Ab_scan = Ab(top_cell,sub_cell,scan_parameters,light)    % scan thickness of the top cell and return the below parameters

            numsteps = scan_parameters.numsteps;
            dimwl = length(light.wl);

            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            Ab_scan.absorption = zeros(numsteps,dimwl);
            %Ab_scan.total_absorption = zeros(numsteps,dimwl);
            Ab_scan.Rf = zeros(numsteps,dimwl);
            Ab_scan.efficiency = zeros(1,numsteps);
            Ab_scan.thickness = zeros(1,numsteps);
            Ab_scan.Jsc = zeros(1,numsteps);
            Ab_scan.scan = zeros(1,numsteps);
            Ab_scan.pathlengthenhancement = zeros(1,numsteps);
            %Ab_scan.loss = zeros(numsteps,dimwl);
            Ab_scan.thickness = zeros(1,numsteps);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;
            filter_wl = wlev(top_cell.material.Eg);
            filter_index = find(light.wl==floor(filter_wl));
            
            for ii=1:1:numsteps;

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %top_cell_calc.W0p = (ii-1)*scan_parameters.step.*top_cell_calc.W0p;                      % only changing the top_cell thickness here
                top_cell_calc.Ab(1:filter_index) = (scan_parameters.end - scan_parameters.start).*(ii - 1)./(numsteps - 1) + 0;                      % only changing the top_cell thickness here
                top_cell_calc.Ab(filter_index:end) = (scan_parameters.end - scan_parameters.start).*(ii - 1)./(numsteps - 1) + 0;                      % only changing the top_cell thickness here
                top_cell_calc.Rb(1:filter_index) = 1-top_cell_calc.Ab(1:filter_index);
                top_cell_calc.Rb(filter_index:end) = 0.0;
                top_cell_calc.Tb(1:filter_index) = 1-top_cell_calc.Ab(1:filter_index) - top_cell_calc.Rb(1:filter_index);
                top_cell_calc.Tb(filter_index:end) = 1-top_cell_calc.Ab(filter_index:end) - top_cell_calc.Rb(filter_index:end);
                %top_cell_calc.Rb = 1 - ((1-top_cell_calc.material.n.^2).*(ii - 1)./(numsteps - 1) + top_cell_calc.material.n.^2)./(top_cell_calc.material.n.^2);
                
                
                optimum_top_cell_thickness = nnl_root.optimise_top_cell(top_cell_calc,sub_cell_calc,light);
                top_cell_calc.thickness = optimum_top_cell_thickness;
                top_cell_calc.xx = top_cell_calc.aa.*(top_cell_calc.material.alpha*top_cell_calc.thickness).^(top_cell_calc.bb);
                top_cell_calc.W0p = top_cell_calc.thickness.*(2+top_cell_calc.xx)./(1 + top_cell_calc.xx);
                top_cell_calc.norm_W0p = top_cell_calc.W0p./top_cell_calc.thickness;
                top_cell_calc.Ta = exp(-top_cell_calc.material.alpha.*top_cell_calc.W0p);
                top_cell_calc.Tp = top_cell_calc.Ta;
                top_cell_calc.Tm = top_cell_calc.Ta;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                Ab_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;  % save these parameters to thickness_scan
                %Ab_scan.total_absorption(ii,:) = total_cell_calc.top_cell.total_absorption;
                Ab_scan.Ab(ii) = top_cell_calc.Ab(1,1);  % save these parameters to thickness_scan
                Ab_scan.efficiency(ii) = total_cell_calc.efficiency;
                Ab_scan.scan(ii) = ii;
                Ab_scan.pathlengthenhancement(ii) = ii./(numsteps).*top_cell_calc.material_lambertian;
                Ab_scan.top_cell_Jsc(ii) = total_cell_calc.top_cell.Jsc;
                %Ab_scan.loss(ii,:) = total_cell_calc.top_cell.bot_light_trapping_loss;
                Ab_scan.thickness(ii) = total_cell_calc.top_cell.thickness;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            end
            
%             W0p_scan.maximum_efficiency = max(W0p_scan.efficiency);
%             W0p_scan.optimum_thickness_index = find(W0p_scan.efficiency==W0p_scan.maximum_efficiency);
%             W0p_scan.optimum_thickness = W0p_scan.thickness(W0p_scan.optimum_thickness_index);
            
        end
        
        function RbAb_scan = RbAb(top_cell,sub_cell,scan_parameters,light)    % scan thickness of the top cell and return the below parameters

            numsteps = scan_parameters.numsteps;
            dimwl = length(light.wl);

            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            RbAb_scan.absorption = zeros(numsteps,dimwl);
            RbAb_scan.total_absorption = zeros(numsteps,dimwl);
            RbAb_scan.Rf = zeros(numsteps,dimwl);
            RbAb_scan.efficiency = zeros(1,numsteps);
            RbAb_scan.thickness = zeros(1,numsteps);
            RbAb_scan.Jsc = zeros(1,numsteps);
            RbAb_scan.scan = zeros(1,numsteps);
            RbAb_scan.pathlengthenhancement = zeros(1,numsteps);
            RbAb_scan.loss = zeros(numsteps,dimwl);
            RbAb_scan.xx = zeros(numsteps,dimwl);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            for ii=1:1:numsteps;

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %top_cell_calc.W0p = (ii-1)*scan_parameters.step.*top_cell_calc.W0p;                      % only changing the top_cell thickness here
                top_cell_calc.Ab(1:dimwl) = scan_parameters.Ab;
                top_cell_calc.Rb(1:dimwl) = (scan_parameters.end - scan_parameters.start).*(ii - 1)./(numsteps - 1) + 0;                      % only changing the top_cell thickness here
                top_cell_calc.Tb(1:dimwl) = 1-top_cell_calc.Rb - top_cell_calc.Ab;
                
                %top_cell_calc.Rb = 1 - ((1-top_cell_calc.material.n.^2).*(ii - 1)./(numsteps - 1) + top_cell_calc.material.n.^2)./(top_cell_calc.material.n.^2);

                optimum_top_cell_thickness = nnl_root.optimise_top_cell(top_cell_calc,sub_cell_calc,light);
                top_cell_calc.thickness = optimum_top_cell_thickness;
                top_cell_calc.xx = top_cell_calc.aa.*(top_cell_calc.material.alpha*top_cell_calc.thickness).^(top_cell_calc.bb);
                top_cell_calc.W0p = top_cell_calc.thickness.*(2+top_cell_calc.xx)./(1 + top_cell_calc.xx);
                top_cell_calc.norm_W0p = top_cell_calc.W0p./top_cell_calc.thickness;
                top_cell_calc.Ta = exp(-top_cell_calc.material.alpha.*top_cell_calc.W0p);
                top_cell_calc.Tp = top_cell_calc.Ta;
                top_cell_calc.Tm = top_cell_calc.Ta;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                RbAb_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;  % save these parameters to thickness_scan
                RbAb_scan.total_absorption(ii,:) = total_cell_calc.top_cell.total_absorption;
                RbAb_scan.Rb(ii,:) = top_cell_calc.Rb;  % save these parameters to thickness_scan
                RbAb_scan.Ab(ii,:) = top_cell_calc.Ab;  % save these parameters to thickness_scan
                RbAb_scan.Tb(ii,:) = top_cell_calc.Tb;  % save these parameters to thickness_scan
                RbAb_scan.efficiency(ii) = total_cell_calc.efficiency;
                RbAb_scan.scan(ii) = ii;
                RbAb_scan.pathlengthenhancement(ii) = ii./(numsteps).*top_cell_calc.material_lambertian;
                RbAb_scan.top_cell_Jsc(ii) = total_cell_calc.top_cell.Jsc;
                RbAb_scan.loss(ii,:) = total_cell_calc.top_cell.bot_light_trapping_loss;
                RbAb_scan.xx(ii,:) = total_cell_calc.top_cell.xx;
                RbAb_scan.lighttopbottomcell(ii,:) = total_cell_calc.sub_cell.inputspectra;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            end
            
%             W0p_scan.maximum_efficiency = max(W0p_scan.efficiency);
%             W0p_scan.optimum_thickness_index = find(W0p_scan.efficiency==W0p_scan.maximum_efficiency);
%             W0p_scan.optimum_thickness = W0p_scan.thickness(W0p_scan.optimum_thickness_index);
            
        end
        
        function RbAbf_scan = RbAbf(top_cell,sub_cell,scan_parameters,light)    % scan thickness of the top cell and return the below parameters

            numsteps = scan_parameters.numsteps;
            dimwl = length(light.wl);

            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            RbAbf_scan.absorption = zeros(numsteps,dimwl);
            RbAbf_scan.total_absorption = zeros(numsteps,dimwl);
            RbAbf_scan.Rf = zeros(numsteps,dimwl);
            RbAbf_scan.efficiency = zeros(1,numsteps);
            RbAbf_scan.thickness = zeros(1,numsteps);
            RbAbf_scan.Jsc = zeros(1,numsteps);
            RbAbf_scan.scan = zeros(1,numsteps);
            RbAbf_scan.pathlengthenhancement = zeros(1,numsteps);
            RbAbf_scan.loss = zeros(numsteps,dimwl);
            RbAbf_scan.xx = zeros(numsteps,dimwl);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            for ii=1:1:numsteps;

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %top_cell_calc.W0p = (ii-1)*scan_parameters.step.*top_cell_calc.W0p;                      % only changing the top_cell thickness here
                filter_index = find(light.wl==floor(scan_parameters.filterwl));
                                
                top_cell_calc.Ab(1:filter_index) = scan_parameters.Ab;
                top_cell_calc.Ab(filter_index:end) = scan_parameters.Ab;
                top_cell_calc.Rb(1:filter_index) = (scan_parameters.end - scan_parameters.start).*(ii - 1)./(numsteps - 1) + 0;                      % only changing the top_cell thickness here
                top_cell_calc.Rb(filter_index:end) = 0;
                
                top_cell_calc.Tb(1:dimwl) = 1-top_cell_calc.Rb - top_cell_calc.Ab;
                
                %top_cell_calc.Rb = 1 - ((1-top_cell_calc.material.n.^2).*(ii - 1)./(numsteps - 1) + top_cell_calc.material.n.^2)./(top_cell_calc.material.n.^2);

                optimum_top_cell_thickness = nnl_root.optimise_top_cell(top_cell_calc,sub_cell_calc,light);
                top_cell_calc.thickness = optimum_top_cell_thickness;
                top_cell_calc.xx = top_cell_calc.aa.*(top_cell_calc.material.alpha*top_cell_calc.thickness).^(top_cell_calc.bb);
                top_cell_calc.W0p = top_cell_calc.thickness.*(2+top_cell_calc.xx)./(1 + top_cell_calc.xx);
                top_cell_calc.norm_W0p = top_cell_calc.W0p./top_cell_calc.thickness;
                top_cell_calc.Ta = exp(-top_cell_calc.material.alpha.*top_cell_calc.W0p);
                top_cell_calc.Tp = top_cell_calc.Ta;
                top_cell_calc.Tm = top_cell_calc.Ta;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                RbAbf_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;  % save these parameters to thickness_scan
                RbAbf_scan.total_absorption(ii,:) = total_cell_calc.top_cell.total_absorption;
                RbAbf_scan.Rb(ii,:) = top_cell_calc.Rb;  % save these parameters to thickness_scan
                RbAbf_scan.Ab(ii,:) = top_cell_calc.Ab;  % save these parameters to thickness_scan
                RbAbf_scan.Tb(ii,:) = top_cell_calc.Tb;  % save these parameters to thickness_scan
                RbAbf_scan.efficiency(ii) = total_cell_calc.efficiency;
                RbAbf_scan.scan(ii) = ii;
                RbAbf_scan.pathlengthenhancement(ii) = ii./(numsteps).*top_cell_calc.material_lambertian;
                RbAbf_scan.top_cell_Jsc(ii) = total_cell_calc.top_cell.Jsc;
                RbAbf_scan.loss(ii,:) = total_cell_calc.top_cell.bot_light_trapping_loss;
                RbAbf_scan.xx(ii,:) = total_cell_calc.top_cell.xx;
                RbAbf_scan.lighttopbottomcell(ii,:) = total_cell_calc.sub_cell.inputspectra;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            end
            
%             W0p_scan.maximum_efficiency = max(W0p_scan.efficiency);
%             W0p_scan.optimum_thickness_index = find(W0p_scan.efficiency==W0p_scan.maximum_efficiency);
%             W0p_scan.optimum_thickness = W0p_scan.thickness(W0p_scan.optimum_thickness_index);
            
        end
        
        function RbAbLambda_scan = RbAbLambda(top_cell,sub_cell,scan_parameters,light)    % scan thickness of the top cell and return the below parameters

            numsteps = scan_parameters.numsteps;
            numwlsteps = scan_parameters.numwlsteps;
            dimwl = length(light.wl);

            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            %RbAbLambda_scan.absorption = zeros(numsteps,dimwl);
            %RbAbLambda_scan.total_absorption = zeros(numsteps,dimwl);
            %RbAbLambda_scan.Rf = zeros(numsteps,dimwl);
            RbAbLambda_scan.efficiency = zeros(numsteps,numwlsteps);
            RbAbLambda_scan.thickness = zeros(numsteps, numwlsteps);
            RbAbLambda_scan.topJsc = zeros(numsteps,numwlsteps);
            RbAbLambda_scan.subJsc = zeros(numsteps,numwlsteps);
            RbAbLambda_scan.Rb = zeros(numsteps, numwlsteps);
            RbAbLambda_scan.Ab = zeros(numsteps, numwlsteps);
            RbAbLambda_scan.Tb = zeros(numsteps, numwlsteps);
            RbAbLambda_scan.Rbscan = zeros(1,numsteps);
            RbAbLambda_scan.wlscan = zeros(1,numwlsteps);
            %RbAbLambda_scan.pathlengthenhancement = zeros(1,numsteps);
            %RbAbLambda_scan.loss = zeros(numsteps,dimwl);
            %RbAbLambda_scan.xx = zeros(numsteps,dimwl);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            for ii=1:1:numsteps;
                for jj=1:1:numwlsteps;
                scanwl = (scan_parameters.wlend - scan_parameters.wlstart).*(jj - 1)./(numwlsteps - 1);
                wlindex = find(light.wl==floor(scanwl));
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %top_cell_calc.W0p = (ii-1)*scan_parameters.step.*top_cell_calc.W0p;                      % only changing the top_cell thickness here
                top_cell_calc.Ab(1:dimwl) = scan_parameters.Ab;
                top_cell_calc.Rb(1:floor(wlindex)) = (scan_parameters.end - scan_parameters.start).*(ii - 1)./(numsteps - 1) + 0;                      % only changing the top_cell thickness here
                top_cell_calc.Rb(floor(wlindex):end)=0;
                top_cell_calc.Tb(1:dimwl) = 1-top_cell_calc.Rb - top_cell_calc.Ab;
                
                %top_cell_calc.Rb = 1 - ((1-top_cell_calc.material.n.^2).*(ii - 1)./(numsteps - 1) + top_cell_calc.material.n.^2)./(top_cell_calc.material.n.^2);

                optimum_top_cell_thickness = nnl_root.optimise_top_cell(top_cell_calc,sub_cell_calc,light);
                top_cell_calc.thickness = optimum_top_cell_thickness;
                top_cell_calc.xx = top_cell_calc.aa.*(top_cell_calc.material.alpha*top_cell_calc.thickness).^(top_cell_calc.bb);
                top_cell_calc.W0p = top_cell_calc.thickness.*(2+top_cell_calc.xx)./(1 + top_cell_calc.xx);
                top_cell_calc.norm_W0p = top_cell_calc.W0p./top_cell_calc.thickness;
                top_cell_calc.Ta = exp(-top_cell_calc.material.alpha.*top_cell_calc.W0p);
                top_cell_calc.Tp = top_cell_calc.Ta;
                top_cell_calc.Tm = top_cell_calc.Ta;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %RbAbLambda_scan.absorption(ii,:) = total_cell_calc.top_cell.absorption;  % save these parameters to thickness_scan
                %RbAbLambda_scan.total_absorption(ii,:) = total_cell_calc.top_cell.total_absorption;
                RbAbLambda_scan.Rb(ii,jj) = top_cell_calc.Rb(1,1);  % save these parameters to thickness_scan
                RbAbLambda_scan.Ab(ii,jj) = top_cell_calc.Ab(1,1);  % save these parameters to thickness_scan
                RbAbLambda_scan.Tb(ii,jj) = top_cell_calc.Tb(1,1);  % save these parameters to thickness_scan
                RbAbLambda_scan.efficiency(ii,jj) = total_cell_calc.efficiency;
                RbAbLambda_scan.thickness(ii,jj) = top_cell_calc.thickness;
                RbAbLambda_scan.Rbscan(ii) = top_cell_calc.Rb(1,1);
                RbAbLambda_scan.wlscan(jj) = scanwl;
                %RbAbLambda_scan.pathlengthenhancement(ii) = ii./(numsteps).*top_cell_calc.material_lambertian;
                RbAbLambda_scan.topJsc(ii,jj) = total_cell_calc.top_cell.Jsc;
                RbAbLambda_scan.subJsc(ii,jj) = total_cell_calc.sub_cell.Jsc;
                %RbAbLambda_scan.loss(ii,:) = total_cell_calc.top_cell.bot_light_trapping_loss;
                %RbAbLambda_scan.xx(ii,:) = total_cell_calc.top_cell.xx;
                %RbAbLambda_scan.lighttopbottomcell(ii,:) = total_cell_calc.sub_cell.inputspectra;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                end
             end
            
%             W0p_scan.maximum_efficiency = max(W0p_scan.efficiency);
%             W0p_scan.optimum_thickness_index = find(W0p_scan.efficiency==W0p_scan.maximum_efficiency);
%             W0p_scan.optimum_thickness = W0p_scan.thickness(W0p_scan.optimum_thickness_index);
            
        end
        
        
        function RbRf_scan = RbRf(top_cell,sub_cell,scan_parameters,light)    % scan thickness of the top cell and return the below parameters

            Rb_steps = scan_parameters.Rb_steps;
            Rf_steps = scan_parameters.Rf_steps;
            dimwl = length(light.wl);

            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            RbRf_scan.absorption = zeros(Rb_steps,Rf_steps);
            RbRf_scan.refl_absorption = zeros(Rb_steps,Rf_steps);
            %RbRf_scan.total_absorption = zeros(numsteps,dimwl);
            RbRf_scan.Rb = zeros(1,Rb_steps);
            RbRf_scan.Rf = zeros(1,Rf_steps);
            RbRf_scan.efficiency = zeros(Rb_steps,Rf_steps);
            RbRf_scan.thickness = zeros(Rb_steps,Rf_steps);
            RbRf_scan.Jsc = zeros(Rb_steps,Rf_steps);
            %RbRf_scan.scan = zeros(Rb_steps,Rf_steps);
            %RbRf_scan.pathlengthenhancement = zeros(1,numsteps);
            %RbRf_scan.loss = zeros(numsteps,dimwl);
            %RbRf_scan.xx = zeros(numsteps,dimwl);
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

            for ii=1:1:Rb_steps;
                for jj=1:1:Rf_steps;
                    
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %top_cell_calc.W0p = (ii-1)*scan_parameters.step.*top_cell_calc.W0p;                      % only changing the top_cell thickness here
                top_cell_calc.Rb = (scan_parameters.Rb_stop - scan_parameters.Rb_start).*(ii - 1)./(Rb_steps - 1) + 0;                      % only changing the top_cell thickness here
                top_cell_calc.Rf = (scan_parameters.Rf_stop - scan_parameters.Rf_start).*(jj - 1)./(Rf_steps - 1) + 0;                      % only changing the top_cell thickness here
                %top_cell_calc.Rb = 1 - ((1-top_cell_calc.material.n.^2).*(ii - 1)./(numsteps - 1) + top_cell_calc.material.n.^2)./(top_cell_calc.material.n.^2);

                optimum_top_cell_thickness = nnl_root.optimise_top_cell(top_cell_calc,sub_cell_calc,light);
                top_cell_calc.thickness = optimum_top_cell_thickness;
                top_cell_calc.xx = top_cell_calc.aa.*((top_cell_calc.material.alpha*top_cell_calc.thickness).^(top_cell_calc.bb));
                top_cell_calc.W0p = top_cell_calc.thickness.*(2+top_cell_calc.xx)./(1 + top_cell_calc.xx);
                top_cell_calc.norm_W0p = top_cell_calc.W0p./top_cell_calc.thickness;
                top_cell_calc.Ta = exp(-top_cell_calc.material.alpha.*top_cell_calc.W0p);
                top_cell_calc.Tp = top_cell_calc.Ta;
                top_cell_calc.Tm = top_cell_calc.Ta;
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                RbRf_scan.absorption(ii,jj) = trapz(light.wl,total_cell_calc.top_cell.absorption);  % save these parameters to thickness_scan
                RbRf_scan.refl_absorption(ii,jj) = trapz(light.wl,total_cell_calc.top_cell.reflector_abs);  % save these parameters to thickness_scan
                %RbRf_scan.total_absorption(ii,:) = total_cell_calc.top_cell.total_absorption;
                RbRf_scan.Rb(1,ii) = top_cell_calc.Rb;  % save these parameters to thickness_scan
                RbRf_scan.Rf(1,jj) = top_cell_calc.Rf;  % save these parameters to thickness_scan
                RbRf_scan.efficiency(ii,jj) = total_cell_calc.efficiency;
                %RbRf_scan.pathlengthenhancement(ii) = ii./(numsteps).*top_cell_calc.material_lambertian;
                RbRf_scan.top_cell_Jsc(ii,jj) = total_cell_calc.top_cell.Jsc;
                RbRf_scan.thickness(ii,jj) = total_cell_calc.top_cell.thickness;
                %RbRf_scan.loss(ii,:) = total_cell_calc.top_cell.bot_light_trapping_loss;
                %RbRf_scan.xx(ii,:) = total_cell_calc.top_cell.xx;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
           
                end
            fprintf(num2str(ii));%'nnl_root imported ok\n');
                    
            end
%             W0p_scan.maximum_efficiency = max(W0p_scan.efficiency);
%             W0p_scan.optimum_thickness_index = find(W0p_scan.efficiency==W0p_scan.maximum_efficiency);
%             W0p_scan.optimum_thickness = W0p_scan.thickness(W0p_scan.optimum_thickness_index);
            
        end
        
        function top_cell_optimum_thickness = optimise_top_cell(top_cell,sub_cell,light)
         
            numsteps = 500;
            grain = 3000/numsteps;
            
            efficiency = zeros(1,numsteps);
            thickness = zeros(1,numsteps);
            
            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;
            
            for ii=1:1:numsteps;

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                top_cell_calc.thickness = (ii-1)*grain;                      % only changing the top_cell thickness here
                top_cell_calc.xx = top_cell_calc.aa.*(top_cell_calc.material.alpha*top_cell_calc.thickness).^(top_cell_calc.bb);
                top_cell_calc.W0p = top_cell_calc.thickness.*(2+top_cell_calc.xx)./(1 + top_cell_calc.xx);
                top_cell_calc.norm_W0p = top_cell_calc.W0p./top_cell_calc.thickness;
                top_cell_calc.Ta = exp(-top_cell_calc.material.alpha.*top_cell_calc.W0p);
                top_cell_calc.Tp = top_cell_calc.Ta;
                top_cell_calc.Tm = top_cell_calc.Ta;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %clear total_cell_calc
                total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                efficiency(ii) = total_cell_calc.efficiency;
                thickness(ii) = top_cell_calc.thickness;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
            end
            
            maxeff= max(efficiency);
            maxeffindex = find(efficiency==maxeff);
            top_cell_optimum_thickness = thickness(1,maxeffindex);
            
        end
    
                
        function reflector_and_A0_scan = reflector_and_A0_scan(top_cell,sub_cell,scan_parameters,light)

            num_A0_steps = scan_parameters.A0_steps+1;
            num_reflector_steps = scan_parameters.reflector_steps+1;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            %trapping_and_loss_scan.absorption = zeros(numsteps,dimwl);
            reflector_and_A0_scan.efficiency = zeros(num_A0_steps,num_reflector_steps);
            reflector_and_A0_scan.Jsc = zeros(num_A0_steps,num_reflector_steps);
            reflector_and_A0_scan.top_cell_efficiency = zeros(num_A0_steps,num_reflector_steps);
            reflector_and_A0_scan.sub_cell_efficiency = zeros(num_A0_steps,num_reflector_steps);
            reflector_and_A0_scan.sub_cell_Jsc = zeros(num_A0_steps,num_reflector_steps);
            reflector_and_A0_scan.sub_cell_Jsc = zeros(num_A0_steps,num_reflector_steps);
            reflector_and_A0_scan.thickness = zeros(num_A0_steps,num_reflector_steps);
            
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

                for ii=1:1:num_A0_steps;
                    for jj=1:1:num_reflector_steps;
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        top_cell_calc.material.Astar0 = 1*10^((scan_parameters.A0_stop-scan_parameters.A0_start).*(ii-1)./(num_A0_steps-1) + scan_parameters.A0_start); % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
                        top_cell_calc.material.alpha = 1e-7.*top_cell_calc.material.Astar0.*((nnl_root.wlev(light.wl) - top_cell_calc.material.Eg).^(1/2));
                        top_cell_calc.material.wlEg = nnl_root.wlev(top_cell_calc.material.Eg);
                        top_cell_calc.material.wlEg_index = find(light.wl==floor(top_cell_calc.material.wlEg));
                        top_cell_calc.material.alpha(top_cell_calc.material.wlEg_index:end)=0;
                        top_cell_calc.bot_intermediate_strength = (scan_parameters.reflector_stop-scan_parameters.reflector_start).*(jj-1)./(num_reflector_steps-1) + scan_parameters.reflector_start;
                        
                        optimum_top_cell_thickness = nnl_root.optimise_top_cell(top_cell_calc,sub_cell_calc,light);
                        top_cell_calc.thickness = optimum_top_cell_thickness;
                        top_cell_calc.xx = top_cell_calc.aa.*(top_cell_calc.material.alpha*top_cell_calc.thickness).^(top_cell_calc.bb);
                        top_cell_calc.W0p = top_cell_calc.thickness.*(2+top_cell_calc.xx)./(1 + top_cell_calc.xx);
                        top_cell_calc.norm_W0p = top_cell_calc.W0p./top_cell_calc.thickness;
                        top_cell_calc.Ta = exp(-top_cell_calc.material.alpha.*top_cell_calc.W0p);
                        top_cell_calc.Tp = top_cell_calc.Ta;
                        top_cell_calc.Tm = top_cell_calc.Ta;
                        
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);

                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        reflector_and_A0_scan.efficiency(ii,jj) = total_cell_calc.efficiency;
                        reflector_and_A0_scan.Jsc(ii,jj) = total_cell_calc.Jsc;
                        reflector_and_A0_scan.top_cell_efficiency(ii,jj) = total_cell_calc.top_cell.efficiency;
                        reflector_and_A0_scan.sub_cell_efficiency(ii,jj) = total_cell_calc.sub_cell.efficiency;
                        reflector_and_A0_scan.top_cell_Jsc(ii,jj) = total_cell_calc.top_cell.Jsc;
                        reflector_and_A0_scan.sub_cell_Jsc(ii,jj) = total_cell_calc.sub_cell.Jsc;
                        reflector_and_A0_scan.thickness(ii,jj) = top_cell_calc.thickness;
                    end
                fprintf(num2str(ii));%'nnl_root imported ok\n');    
                end
             
        end        
        
        function beta = figureofmerit(top_cell,sub_cell,lambertian,light)
            
            beta.beta = zeros(1,length(light.wl));
            beta.beta_integ = 0;
            ones = zeros(1,length(light.wl));
            ones(:) =1;
            wlEg = nnl_root.wlev(top_cell.material.Eg);
            wlEg_index = find(light.wl==floor(wlEg));
 
            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;
            lambertian_calc = lambertian;
            
            optimum_top_cell_thickness = nnl_root.optimise_top_cell(top_cell_calc,sub_cell_calc,light);
            top_cell_calc.thickness = optimum_top_cell_thickness;
            total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);
            beta.thickness = optimum_top_cell_thickness;
            % beta.beta = ( 1 ./ (lambertian_calc.top_cell.collection_probability.*lambertian.top_cell.absorption + lambertian_calc.sub_cell.collection_probability.*lambertian.sub_cell.absorption )) .*(total_cell_calc.top_cell.collection_probability.*total_cell_calc.top_cell.absorption  -  (total_cell_calc.top_cell.inputspectra - total_cell_calc.sub_cell.inputspectra - total_cell_calc.top_cell.absorption)).*              ( total_cell_calc.sub_cell.Voc.*total_cell_calc.sub_cell.FF./(total_cell_calc.top_cell.Voc.*total_cell_calc.top_cell.FF - total_cell_calc.sub_cell.Voc.*total_cell_calc.sub_cell.FF )); 
            % beta.beta = ( 1 ./ (lambertian_calc.top_cell.collection_probability.*lambertian.top_cell.absorption)) .*(total_cell_calc.top_cell.collection_probability.*total_cell_calc.top_cell.absorption  -  (total_cell_calc.top_cell.inputspectra - total_cell_calc.sub_cell.inputspectra - total_cell_calc.top_cell.absorption)).*              ( total_cell_calc.sub_cell.Voc.*total_cell_calc.sub_cell.FF./(total_cell_calc.top_cell.Voc.*total_cell_calc.top_cell.FF - total_cell_calc.sub_cell.Voc.*total_cell_calc.sub_cell.FF )); 
            beta.power = (total_cell_calc.top_cell.collection_probability.*total_cell_calc.top_cell.absorption.*total_cell_calc.top_cell.Voc.*total_cell_calc.top_cell.FF + total_cell_calc.sub_cell.collection_probability.*total_cell_calc.sub_cell.absorption.*total_cell_calc.sub_cell.Voc.*total_cell_calc.sub_cell.FF)./(lambertian_calc.top_cell.collection_probability.*lambertian_calc.top_cell.absorption.*lambertian_calc.top_cell.Voc.*lambertian_calc.top_cell.FF + lambertian_calc.sub_cell.collection_probability.*lambertian_calc.sub_cell.absorption.*lambertian_calc.sub_cell.Voc.*lambertian_calc.sub_cell.FF); 
            beta.power_integ = trapz(light.wl,beta.power)/trapz(light.wl,ones); 
            beta.power_integ2 = total_cell_calc.power./lambertian_calc.power; 
            beta.beta = ( 1 ./ (lambertian_calc.top_cell.collection_probability.*lambertian_calc.top_cell.absorption)) .*(total_cell_calc.top_cell.collection_probability.*total_cell_calc.top_cell.absorption  -  (total_cell_calc.top_cell.parasitic_light_trapping).*              ( total_cell_calc.sub_cell.Voc.*total_cell_calc.sub_cell.FF./(total_cell_calc.top_cell.Voc.*total_cell_calc.top_cell.FF - total_cell_calc.sub_cell.Voc.*total_cell_calc.sub_cell.FF ))); 
            beta.beta(wlEg_index:end) = 0;
            ones(wlEg_index:end) = 0;
            beta.beta_integ = trapz(light.wl,beta.beta)/trapz(light.wl,ones);
                
            topcellJparasitic = (1.6e-19)*(trapz(light.wl,total_cell_calc.top_cell.parasitic_light_trapping./nnl_root.energy_of_photon(light.wl))); 
            beta.beta_integ2 = (1/lambertian_calc.top_cell.Jsc)*(total_cell_calc.top_cell.Jsc - topcellJparasitic* (total_cell_calc.sub_cell.Voc.*total_cell_calc.sub_cell.FF./(total_cell_calc.top_cell.Voc.*total_cell_calc.top_cell.FF - total_cell_calc.sub_cell.Voc.*total_cell_calc.sub_cell.FF)));
            beta.beta_integ3 = (1/lambertian_calc.top_cell.Jsc)*(total_cell_calc.top_cell.Jsc - topcellJparasitic* (total_cell_calc.sub_cell.Voc/(total_cell_calc.top_cell.Voc - total_cell_calc.sub_cell.Voc)));
            
             
%              beta.beta = ( 1 ./ (lambertian_calc.top_cell.collection_probability.*lambertian.top_cell.absorption + lambertian_calc.sub_cell.collection_probability.*lambertian.sub_cell.absorption)) .*(total_cell_calc.top_cell.collection_probability.*total_cell_calc.top_cell.absorption + total_cell_calc.sub_cell.collection_probability.*total_cell_calc.sub_cell.absorption -  (total_cell_calc.top_cell.parasitic_light_trapping).*              ( total_cell_calc.sub_cell.Voc.*total_cell_calc.sub_cell.FF./(total_cell_calc.top_cell.Voc.*total_cell_calc.top_cell.FF - total_cell_calc.sub_cell.Voc.*total_cell_calc.sub_cell.FF ))); 
%              beta.beta_integ = trapz(light.wl,beta.beta)/trapz(light.wl,ones);
        
        
        
        end
     
        function phi_scan = phi_scan(top_cell,sub_cell,scan_parameters,light)

            num_steps = scan_parameters.steps+1;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            %trapping_and_loss_scan.absorption = zeros(numsteps,dimwl);
            phi_scan.efficiency = zeros(1,num_steps);
            phi_scan.Voc = zeros(1,num_steps);
            phi_scan.Jsc = zeros(1,num_steps);
%             phi_scan.top_cell_efficiency = zeros(num_A0_steps,num_light_steps);
%             phi_scan.sub_cell_efficiency = zeros(num_A0_steps,num_light_steps);
%             phi_scan.sub_cell_Jsc = zeros(num_A0_steps,num_light_steps);
%             phi_scan.sub_cell_Jsc = zeros(num_A0_steps,num_light_steps);
%             phi_scan.top_cell_thickness = zeros(num_A0_steps,num_light_steps);
%             
            %%%%%%%%%%%%%%%%%%%%%%%%%

            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;

                for ii=1:1:num_steps;
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        top_cell_calc.material.phi = 1*10^((scan_parameters.stop-scan_parameters.start).*(ii-1)./(num_steps-1) + scan_parameters.start); % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
%                         top_cell_calc.material.alpha = 1e-9.*top_cell_calc.material.Astar0.*((nnl_root.wlev(light.wl) - top_cell_calc.material.Eg).^(1/2));
%                         top_cell_calc.material.wlEg = nnl_root.wlev(top_cell_calc.material.Eg);
%                         top_cell_calc.material.wlEg_index = find(light.wl==floor(top_cell_calc.material.wlEg));
%                         top_cell_calc.material.alpha(top_cell_calc.material.wlEg_index:end)=0;
%                         top_cell_calc.top_light_trapping = (scan_parameters.light_stop-scan_parameters.light_start).*(jj-1)./(num_light_steps-1) + scan_parameters.light_start;
%                         optimum_top_cell_thickness = nnl_root.optimise_top_cell(top_cell_calc,sub_cell_calc,light);
                        %top_cell_calc.thickness = optimum_top_cell_thickness;
                        
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        total_cell_calc = nnl_root.calculate_cell(top_cell_calc,sub_cell_calc,light);

                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        phi_scan.efficiency(1,ii) = total_cell_calc.top_cell.efficiency;
                        phi_scan.Voc(1,ii) = total_cell_calc.top_cell.Voc;
                        phi_scan.Jsc(1,ii) = total_cell_calc.top_cell.Jsc;
%                         phi_scan.Jsc(ii,jj) = total_cell_calc.Jsc;
%                         phi_scan.top_cell_efficiency(ii,jj) = total_cell_calc.top_cell.efficiency;
%                         phi_scan.sub_cell_efficiency(ii,jj) = total_cell_calc.sub_cell.efficiency;
%                         phi_scan.top_cell_Jsc(ii,jj) = total_cell_calc.top_cell.Jsc;
%                         phi_scan.sub_cell_Jsc(ii,jj) = total_cell_calc.sub_cell.Jsc;
%                         phi_scan.top_cell_thickness(ii,jj) = total_cell_calc.top_cell.thickness;
                end
             
        end
        
        function top_cell_requirements = requirements(top_cell,sub_cell,scan_parameters,light)
        
            top_cell_calc = top_cell;
            sub_cell_calc = sub_cell;
            num_steps = scan_parameters.steps+1;
            
            top_cell_calc.thickness = 0;
            lightfilter = zeros(1,length(light.wl));
            loss = zeros(1,length(light.wl));
            loss(1,:) = scan_parameters.loss;

            top_cell_requirements.top_efficiency = zeros(1,num_steps);
            top_cell_requirements.sub_efficiency = zeros(1,num_steps);
            top_cell_requirements.sub_power = zeros(1,num_steps);
            top_cell_requirements.required_power = zeros(1,num_steps);
            top_cell_requirements.required_efficiency = zeros(1,num_steps);
            
            for ii=1:1:num_steps;
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        lightfilter_eV = ((scan_parameters.stop-scan_parameters.start).*(ii-1)./(num_steps-1) + scan_parameters.start); % Astar0 with dimensions of L^-2 T M^(-1/2), value taken from  TW fit: check dimensions? in m^-1! no!
                        lightfilter(1,:) =nnl_root.wlev(light.wl)<=lightfilter_eV;
                        
                        sub_cell_calc.inputspectra = light.sunshine.spectra.*lightfilter.*(1-loss);
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        %sub_cell.inputspectra = (top_cell.light_to_cell - top_cell.absorption).*(1-top_cell.intermediate_reflector).*(1 - top_cell.bot_light_trapping_loss);  % multiplicative light_loss for spectrum incident on substrate cell
                        sub_cell_calc.inputenergy = trapz(light.wl,sub_cell_calc.inputspectra);
                        sub_cell_calc.optical_thickness = (sub_cell_calc.top_light_trapping + sub_cell_calc.bot_light_trapping)*sub_cell_calc.thickness;    % optical thickness = real thickness x path length enhancement
                        sub_cell_calc.absorption = nnl_root.green_Si_absorption(sub_cell_calc.inputspectra,sub_cell_calc.material,sub_cell_calc.thickness);    % calculate the sub_cell absorption with the optical thickness

                        sub_cell_calc.collection_probability = nnl_root.collection_probability(sub_cell_calc.thickness,sub_cell_calc.material);

                        sub_cell_calc.Jsc = nnl_root.Jsc(sub_cell_calc,light.wl);
                        sub_cell_calc.Voc = nnl_root.Voc(sub_cell_calc, light);   
                        sub_cell_calc.FF = sub_cell_calc.material.FF;

                        sub_cell_calc.power = sub_cell_calc.Jsc*sub_cell_calc.Voc*sub_cell_calc.material.FF;
                        sub_cell_calc.efficiency = sub_cell_calc.Jsc*sub_cell_calc.Voc*sub_cell_calc.material.FF/light.sunshine.energy;
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        
                        top_cell_requirements.sub_efficiency(1,ii) = sub_cell_calc.efficiency;
                        top_cell_requirements.sub_power(1,ii) = sub_cell_calc.power;
            end
             
                top_cell_requirements.required_power = scan_parameters.desired_efficiency*light.sunshine.energy - top_cell_requirements.sub_power;
                top_cell_requirements.required_efficiency = top_cell_requirements.required_power/light.sunshine.energy;
                top_cell_requirements.required_efficiency_frac = top_cell_requirements.required_efficiency./(scan_parameters.sq.I./100);
           
            
        end
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% Conductor functions%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       
        function rsh_scan = rsh(conductor,scan_parameters)    % scan thickness of the top cell and return the below parameters

            Vloss_steps = scan_parameters.Vloss_steps;
            pitch_steps = scan_parameters.pitch_steps;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%                                                   % this is the list of parameters you want stored in the scan structure
            rsh_scan.rsh = zeros(length(Vloss_steps),length(pitch_steps));
            %%%%%%%%%%%%%%%%%%%%%%%%%

            for ii=1:1:length(Vloss_steps);
                for jj=1:1:length(pitch_steps);
                rsh_scan.rsh(ii,jj) = 8*Vloss_steps(ii)./(conductor.J.*pitch_steps(jj).^2); 
                end
            end
            
        end
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%Plasmon functions%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function plasmon_balance = plasmon_balance(material1,material2,wavelengths)
            plasmon_balance.kk0 = (2*pi./(wavelengths.*1e-9));
            plasmon_balance.kkx = plasmon_balance.kk0 .* (((material1.eps.*material2.eps)./(material1.eps + material2.eps)).^(1/2)); 
            plasmon_balance.kkx_real = abs(real(plasmon_balance.kkx)); 
            plasmon_balance.kkx_im = abs(imag(plasmon_balance.kkx)); 
            plasmon_balance.kkx = complex(plasmon_balance.kkx_real,plasmon_balance.kkx_im);
            
            plasmon_balance.kkz1 = ((plasmon_balance.kk0.^2).*material1.eps - plasmon_balance.kkx.^2 ).^(1/2);
            plasmon_balance.kkz1_real = abs(real(plasmon_balance.kkz1)); 
            plasmon_balance.kkz1_im = abs(imag(plasmon_balance.kkz1)); 
            plasmon_balance.kkz1 = complex(plasmon_balance.kkz1_real,plasmon_balance.kkz1_im);
            
            plasmon_balance.kkz2 = ((plasmon_balance.kk0.^2).*material2.eps - plasmon_balance.kkx.^2 ).^(1/2);
            plasmon_balance.kkz2_real = abs(real(plasmon_balance.kkz2)); 
            plasmon_balance.kkz2_im = abs(imag(plasmon_balance.kkz2)); 
            plasmon_balance.kkz2 = complex(plasmon_balance.kkz2_real,plasmon_balance.kkz2_im);
            
            plasmon_balance.normabsratio1 = imag(plasmon_balance.kkz2).* imag(material1.eps).* (1 + ((abs(plasmon_balance.kkx)).^2)./((abs(plasmon_balance.kkz1)).^2) );
            plasmon_balance.normabsratio2 = imag(plasmon_balance.kkz1).* imag(material2.eps).* (1 + ((abs(plasmon_balance.kkx)).^2)./((abs(plasmon_balance.kkz2)).^2) );
            plasmon_balance.absratio1 = plasmon_balance.normabsratio1./(plasmon_balance.normabsratio1 + plasmon_balance.normabsratio2);
            plasmon_balance.absratio2 = plasmon_balance.normabsratio2./(plasmon_balance.normabsratio1 + plasmon_balance.normabsratio2);
            
        end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%PV price functions%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
        function price = price(watts,timemins)
            electricity_price = 0.25;
            timehrs = timemins/60;
            kwhr = watts.*timehrs./1000;
            price = kwhr.*electricity_price;
            fprintf(sprintf('%g,%g\n',kwhr,price));
        end
        
        function pvpriceperkwhr = pvpriceperkwhr(pvsystem)
%                      
            pvpriceperkwhr.years = 1:1:pvsystem.yearsofrunning;
            pvpriceperkwhr.futureelecprice = zeros(1,pvsystem.yearsofrunning);
            pvpriceperkwhr.futureelecprice(1,:)=pvsystem.currentelecprice.*(1+pvsystem.elecincreaseperyear).^(pvpriceperkwhr.years);
            pvpriceperkwhr.efficiency = ((pvsystem.endoflifeeff - 1)./(pvsystem.yearsofrunning-1)).*(pvpriceperkwhr.years-1) + 1;
            pvpriceperkwhr.yearlyelec = pvsystem.kw.*pvsystem.avgsolarinsolation.*365.*pvpriceperkwhr.efficiency;
            pvpriceperkwhr.totalelec = sum(pvpriceperkwhr.yearlyelec);
            %pvpriceperkwhr.costofinvestment = pvsystem.opportunitycost.*pvpriceperkwhr.years.*pvsystem.cost;
            %pvpriceperkwhr.costofinvestmentatend = pvpriceperkwhr.costofinvestment(pvsystem.yearsofrunning);
            pvpriceperkwhr.costofinvestment(1,:) = nnl_root.compoundinterest(pvsystem.cost,pvsystem.opportunitycost/100,pvpriceperkwhr.years);
            pvpriceperkwhr.costofinvestmentatend = pvpriceperkwhr.costofinvestment.interest(pvsystem.yearsofrunning);
            pvpriceperkwhr.price = (pvsystem.cost + pvpriceperkwhr.costofinvestmentatend + pvsystem.inverterreplacementcost)./(pvpriceperkwhr.totalelec);
            pvpriceperkwhr.mortgagerepaymentseachyear = zeros(1,pvsystem.yearsofrunning);
            pvpriceperkwhr.mortgagerepaymentseachyear(1,:) = 12.*pvsystem.mortgageswitch.*nnl_root.mortgage(pvsystem.mortgageprincipal,pvsystem.mortgagerate,pvsystem.mortgageyears);
            pvpriceperkwhr.savingseachyear = pvpriceperkwhr.yearlyelec.*(pvpriceperkwhr.futureelecprice - pvpriceperkwhr.price) - pvpriceperkwhr.mortgagerepaymentseachyear;
            pvpriceperkwhr.cumulativesavings = cumsum(pvpriceperkwhr.savingseachyear);
        end
        
        function pvpriceperkwhract = pvpriceperkwhract(pvsystem)
%                      
            pvpriceperkwhr.years = 1:1:pvsystem.yearsofrunning;
            pvpriceperkwhr.futureelecprice = zeros(1,pvsystem.yearsofrunning);
            pvpriceperkwhr.futureelecprice(1,:)=pvsystem.currentelecprice.*(1+pvsystem.elecincreaseperyear).^(pvpriceperkwhr.years);
            pvpriceperkwhr.efficiency = ((pvsystem.endoflifeeff - 1)./(pvsystem.yearsofrunning-1)).*(pvpriceperkwhr.years-1) + 1;
            pvpriceperkwhr.yearlyelec = pvsystem.kw.*pvsystem.avgsolarinsolation.*365.*pvpriceperkwhr.efficiency;
            pvpriceperkwhr.totalelec = sum(pvpriceperkwhr.yearlyelec);
            pvpriceperkwhr.costofinvestment = pvsystem.opportunitycost.*pvpriceperkwhr.years.*pvsystem.cost;
            pvpriceperkwhr.costofinvestmentatend = pvpriceperkwhr.costofinvestment(pvsystem.yearsofrunning);
            pvpriceperkwhr.price = (pvsystem.cost + pvpriceperkwhr.costofinvestmentatend + pvsystem.inverterreplacementcost)./(pvpriceperkwhr.totalelec);
            pvpriceperkwhr.savingseachyear = pvpriceperkwhr.yearlyelec.*(pvpriceperkwhr.futureelecprice - pvpriceperkwhr.price);
            pvpriceperkwhr.cumulativesavings = cumsum(pvpriceperkwhr.savingseachyear);
        end
        
        function pvpricescan = pvpricescan(maxrate,numsteps)
            step = maxrate/numsteps;
            pvpricescan.scan = step:step:maxrate;
            pvpricescan.price = pvpricescan.scan;
            for ii=1:1:numsteps;
                opportunitycost = (ii-1).*step/numsteps;
                pvpricescan.price(ii) = nnl_root.pvprice(opportunitycost);
            end
        end
        
        function amount = compoundinterest(principal, rate,years)
            amount.amount = principal.*(1+rate).^(years);
            amount.interest = amount.amount - principal;
        end
        
        function amount = simpleinterest(principal, rate,years)
            amount.amount = principal + principal.*rate.*years;
            amount.interest = amount.amount - principal;
        end
        
        function mortgagemonthlyrepayments = mortgage(principal,rate,years)
            mortgagemonthlyrepayments = (rate/12).*principal./(1 - (1+rate./12).^(-years.*12));
            
        end
        
 
        
    end
end
